;;;===================================================
;;; OM-CHANT
;;; Control CHANT synthesis from OpenMusic
;;;
;;; LIBRARY MAIN FILE
;;; Jean Bresson, IRCAM 2010
;;;===================================================


(in-package :om)

(defparameter *chant-files* '("chant-sdif" 
                               "chant-synth"
                               "chant-auto-rules" 
                               "filters"
                               "chant-prefs"))

(mapc #'(lambda (file) (compile&load (om-relative-path '("sources") file))) *chant-files*)

(fill-library '(("Synthesize" nil nil (chant-synth) nil)
                ("SDIF" nil nil (chant-patch write-chant-sdif-file gen-chant-sdif-buffer) nil)
                ("Tools" nil nil (vowel-formants autoamp fcomp autobw cslope autobend autotex spcor) nil))
              )


(doc-library "OM-CHANT is a library for the control of CHANT synthesis from OpenMusic.

The CHANT synthesizer (X. Rodet, Y. Potard, IRCAM - 1984) is set up according to one of 11 available patches, corresponding to different configurations of a FOF generator, a NOISE generator, a SOUND file player and a resonant FILTER.

Each of these modules is controlled by a continuous sequence of states. The synthesizer automatically interpolates between two successive states specified for each modules.
The CHANT synthesizer can be controlled by a file in the SDIF format. 

OM-CHANT provides tools for creating this control file and for calling the CHANT synthesizer in OM visual programs.
Some automatic calculation andcorrection rules adapted from previous Chant implementations are also available.
" 
             (find-library "OM-Chant"))

; (gen-lib-reference (find-library "OM-Chant"))

(unless (fboundp 'om::set-lib-release) (defmethod om::set-lib-release (version &optional lib) nil))


(om::set-lib-release 1.2)


(print "
;;;===================================================
;;; OM-CHANT (c) IRCAM 2010
;;; Control of CHANT in OpenMusic
;;;===================================================
")


;;;======================
;;; RELEASE NOTES
;;;======================
;;; 1.1
;;; - formant-filters returned as symbols ex. '(soprano o) instead of strings
;;; - chant-synth reformatted with keywords
;;; - chant-synth new input : normalize-level
;;;
;;; 1.2
;;; - New CHANT release compatibility (fixes SDIF dependencies and fromsf calls)