;;;===================================================
;;; OM-CHANT
;;; Control CHANT synthesis from OpenMusic
;;;
;;; CHANT path / OM preferences
;;; Jean Bresson, IRCAM 2010
;;;===================================================


(in-package :om)


(pushr 'chant *external-prefs*)

(defmethod get-external-name ((module (eql 'chant))) "CHANT")
(defmethod get-external-icon ((module (eql 'chant))) (list 600 (find-library "OM-Chant")))

(defmethod get-external-module-vals ((module (eql 'chant)) modulepref) (get-pref modulepref :chant-options))
(defmethod get-external-module-path ((module (eql 'chant)) modulepref) (get-pref modulepref :chant-path))
(defmethod set-external-module-vals ((module (eql 'chant)) modulepref vals) (set-pref modulepref :chant-options vals))
(defmethod set-external-module-path ((module (eql 'chant)) modulepref path) 
  (set-pref modulepref :chant-path path))

;;; params folder
(defun def-chant-options ()
  (list (om-external-app '("Chant") "tosf")))

(defmethod get-external-def-vals ((module (eql 'chant))) 
  (list :chant-path (om-external-app '("Chant") "chant")
        :chant-options (def-chant-options)))

(defmethod save-external-prefs ((module (eql 'chant))) 
  `(:chant-path ,(om-save-pathname *chant-path*) 
    :chant-options (list ,(om-save-pathname *tosf-path*))))


(defmethod put-external-preferences ((module (eql 'chant)) moduleprefs)
  (let ((list-prefs (get-pref moduleprefs :chant-options)))
    (when list-prefs 
      (if (and (nth 0 list-prefs) (probe-file (nth 0 list-prefs)))
       (setf *tosf-path* (nth 0 list-prefs))
       (progn
         (setf (nth 0 list-prefs) (nth 0 
                                       (nth (+ 1 (position :chant-options (get-external-def-vals module)))
                                            (get-external-def-vals module))))
         (set-pref moduleprefs :chant-options list-prefs)
         ))
      )
    (when (get-pref moduleprefs :chant-path)
      (setf *chant-path* (find-true-external (get-pref moduleprefs :chant-path))))
    ))


(defmethod show-external-prefs-dialog ((module (eql 'chant)) prefvals)
  (let* ((rep-list (copy-list prefvals))
         (dialog (om-make-window 'om-dialog
                                 :window-title "CHANT Options"
                                 :size (om-make-point 300 160)
                                 :position :centered
                                 :resizable nil :maximize nil :close nil))
         tosftext)
    (om-add-subviews dialog
                     (om-make-dialog-item 'om-static-text  (om-make-point 20 20) (om-make-point 200 20) 
                                          "TOSF path:" :font *controls-font*)
                     (setf tosftext (om-make-dialog-item 'om-static-text  (om-make-point 20 45) (om-make-point 220 50) 
                                                           (om-namestring (nth 0 prefvals))
                                                           :font *om-default-font1*))
                     (om-make-dialog-item 'om-button (om-make-point 240 44) (om-make-point 50 24) "..." 
                                          :di-action (om-dialog-item-act item
                                                       (declare (ignore item))
                                                       (let ((newpath (om-choose-file-dialog 
                                                                       :directory (om-namestring (nth 0 prefvals)))))
                                                         (when newpath 
                                                           (om-set-dialog-item-text tosftext (om-namestring newpath))
                                                           ))))
                     ;;; boutons
                     (om-make-dialog-item 'om-button (om-make-point 20 110) (om-make-point 80 20) "Restore"
                                          :di-action (om-dialog-item-act item
                                                       (om-set-dialog-item-text tosftext (namestring (nth 0 (def-chant-options))))
                                                       ))
      
      (om-make-dialog-item 'om-button (om-make-point 130 110) (om-make-point 80 20) "Cancel"
                           :di-action (om-dialog-item-act item
                                        (om-return-from-modal-dialog dialog nil)))
      
      (om-make-dialog-item 'om-button (om-make-point 210 110) (om-make-point 80 20) "OK"
                           :di-action (om-dialog-item-act item
                                        (let ((argerror nil)
                                              (tosftxt (om-dialog-item-text tosftext)))
                                          (setf (nth 0 rep-list) tosftxt)
                                          (if argerror
                                              (om-message-dialog (format nil "Error in a CHANT option.~% Preferences could not be recorded.")) 
                                            (om-return-from-modal-dialog dialog rep-list))
                                          ))
                           :default-button t :focus t)
                     
      )
    (om-modal-dialog dialog)))

(when (find-pref-module :externals)
  (put-external-preferences 'chant (find-pref-module :externals)))



