;;;===================================================
;;;
;;; OM-Spat
;;; Representation and rendering of spatial scene descriptions
;;;
;;; IRCAM - Resprsentations Musicales 
;;;
;;; Authors/Contributors :	Jean Bresson 
;;;  				Marlon Schumacher (CIRMMT/IDMIL/DCS, McGill University) 
;;; 				Gilbert Nouno, Carlos Agon
;;; Spat renderer : 		Thibaut Carpentier (IRCAM - 2010)
;;;
;;;===================================================


(in-package :om)

(compile&load (namestring (om-relative-path '("sources") "spat-matrix")))
(compile&load (namestring (om-relative-path '("sources") "spat-sdif")))
(compile&load (namestring (om-relative-path '("sources") "spat-prefs")))
(compile&load (namestring (om-relative-path '("sources") "spat-renderer")))
(compile&load (namestring (om-relative-path '("sources") "sdif-player")))


(om::fill-library '((nil nil (spat-matrix) (save-spat-sdif spat-synth) nil)))

(unless (fboundp 'om::set-lib-release) (defmethod om::set-lib-release (version &optional lib) nil))


(om::set-lib-release 2.1)

(doc-library "
OM-Spat is a library for the creation and rendering of spatial scenes in OpenMusic.

The scenes are represented as matrix objects (class SPAT-MATRIX) containing the information and spatial data of a number of sources.

OM-Spat implements a storage of the source trajectories and spatial attributes using the SDIF format. Generated SDIF files can be rendered by the Spat kernel or streamed to real-time environments via the Spat-SDIF-Player (applications to be installed separately).

Design and development: Jean Bresson, IRCAM Representations Musicales. 
Contributions: Marlon Schumacher, CIRMMT/IDMIL/DCS, Mc Gill University.
OM-Spat is inspired from previous works by Carlos Agon and Gilber Nouno. 
Spat renderer by Thibaut Carpentier, IRCAM Acoustic and Cognitive Spaces.
Spat-SDIF-Player by Jean Bresson and Marlon Schumacher.
")


(om-print "
;;;============================================================
;;; OM-Spat 2.1
;;; Representation and rendering of spatial scene descriptions
;;; (c) IRCAM - Representations Musicales - 2010
;;;============================================================
")

;;; (gen-lib-reference (find-library "OM-Spat"))


