;;;===================================================
;;;
;;; OM-SuperVP
;;; SuperVP sound analysis and processing for OpenMusic
;;;
;;; Requires perliminary installation of SuperVP (also included in AudioSculpt)
;;; Set SuperVP path in OM preferences (externals) once the library is loaded.
;;;
;;;
;;; LIBRARY MAIN FILE
;;; Authors: Jean Bresson, Jean Lochard (IRCAM - 2006-2010)
;;;
;;;===================================================


(in-package :om)

;--------------------------------------------------
;Variable definiton with files to load 
;--------------------------------------------------

(defvar *initfile* *load-pathname*)

(defun lib-src-file (name)
  (make-pathname :directory (append (pathname-directory *initfile*) (list "sources")) 
                 :name name))

(defvar *OMSVP-files* nil)
(setf *OMSVP-files* '("general"
                      "normalize"
                      "analysis"
                      "processing"
                      "synthesis"
                      "channels"
                      "formants"
                      "preferences"
                      ))

;--------------------------------------------------
;Loading files 
;--------------------------------------------------

(mapc #'(lambda (filename) 
          (compile&load (namestring (lib-src-file filename))))
      *OMSVP-files*)

;--------------------------------------------------
; OM subpackages initialization
; ("sub-pack-name" subpacke-lists class-list function-list class-alias-list)
;--------------------------------------------------
(defvar *svppackages-list* nil)
(setf *subpackages-list*
      '(("Analysis" nil nil (fft f0-estimate transient-detection formant-analysis) nil)
        ("Treatments" nil nil (supervp-transposition supervp-timestretch
                                                     supervp-frequencyshift supervp-breakpointfilter
                                                     supervp-formantfilter supervp-bandfilter
                                                     supervp-clipping supervp-freeze
                                                     supervp-surfacefilter
                                                     supervp-gain) nil)
        ("Processing" nil nil (supervp-processing 
                               supervp-cross-synthesis 
                               supervp-sourcefilter-synthesis
                               supervp-normalize supervp-merge supervp-split) nil)
        ("Tools" nil nil (make-spec-env) nil)
        ))

;--------------------------------------------------
;filling packages
;--------------------------------------------------
(om::fill-library *subpackages-list*)

(doc-library "OM-SuperVP is a library for the control of SuperVP from OpenMusic.

It contains functions for sound analysis (FFT, Transients, Fundamental frequency, Formants) and processing (Time/Frequency transformations, filters, cross-synthesis, etc.).

The library requires SuperVP to be installed, authorized, and declared as an OM external (see Preferences/Extrernals).
SuperVP can also be selected as a sound normalizer in OM sound processing (see Preferences/Audio).

" 
             (find-library "OM-SuperVP"))

; (gen-lib-reference (find-library "OM-SuperVP"))

(print "
;;;===========================================================================
;;; OM-SuperVP 2.5
;;; SuperVP sound analysis and processing for OpenMusic
;;;
;;; Requires perliminary installation of SuperVP (or AudioSculpt)
;;; Set path to supervp in the OM preferences ('Externals' tab)
;;;
;;; J. Bresson, J. Lochard, IRCAM (2006-2010)
;;;===========================================================================
")
