;;;===================================================
;;;
;;; OM-SuperVP
;;; SuperVP sound analysis and processing for OpenMusic
;;;
;;; Requires perliminary installation of SuperVP (also included in AudioSculpt)
;;; Set SuperVP path in OM preferences (externals) once the library is loaded.
;;;
;;;
;;; NORMALIZATION
;;; The normalization is integrated in the 'normalizers' options in OM sound package & preferences
;;; Authors: Jean Bresson (IRCAM - 2006-2010)
;;;
;;;===================================================

(in-package :om)
  

;; !! rescale > O
(defun svp-rescale (path newpath &optional (rescale 0.0) (resolution 16))
  (if (probe-file *svp-path*)
    (progn
      (handle-new-file-exists newpath)
      (om-print "==========================")
      (om-print "SUPERVP NORMALIZATION")
      (om-print "==========================")
      
      (om-print (format nil "Input file: ~s" (namestring path)))
      (om-print (format nil "Level: ~D dB" rescale))
      (om-print (format nil "Resolution: ~D bits" resolution))
      
      (om-cmd-line (format nil "~s -S~S -norm ~D -Osa~D ~S" 
                                  (om-path2cmdpath *svp-path*)
                                  (om-path2cmdpath path) 
                                  (abs rescale)
                                  resolution
                                  (om-path2cmdpath newpath)) *sys-console*)
      )
    (progn 
      (om-beep-msg "NORMALIZE: SuperVP not found. Set path in Preferences.")
      nil
      ))
  (or (probe-file newpath)
      (om-beep-msg "NORMALIZE: Error in SuperVP processing. No output file created."))
      )

(defmethod! supervp-normalize ((self sound) &optional (outfile "normout.aif") (rescale nil) (resolution nil))
            :initvals '(nil "normout.aif" nil nil)
            :indoc '("sound or sound file pathname" "output file name" "scale factor" "output resolution")
            :icon 950
            :doc "Normalizes <self> in a new sound file <outfile>.

If <rescale> and <resolution> are NIL (default) they will be set according to the values in OM audio preferences.
"
      (supervp-normalize (om-sound-file-name self) outfile rescale resolution))

(defmethod! supervp-normalize ((self pathname) &optional (outfile "normout.aif") (rescale nil) (res nil))
            (let ((resolution (or res *audio-res* 16))
                  (rescaledb (or rescale (if (numberp *normalize-level*) *normalize-level* 0.0)))
                  (out (if outfile
                           (if (pathnamep outfile) outfile (outfile outfile))
                         (om-choose-file-dialog :prompt "Save as..." :directory (outfile nil)))))
              (svp-rescale self out rescaledb resolution)))


(defmethod general-normalize ((norm (eql :supervp)) inpath outpath val &optional resolution)
  (supervp-normalize inpath outpath val resolution))

(pushr :supervp *loaded-normalizers*)

(defmethod get-def-normalize-value ((self (eql :supervp))) 0.0)
(defmethod get-module-name ((self (eql :supervp))) "SuperVP")







