;;;===================================================
;;; OM2Csound
;;; Control of Csound sound synthesis from OpenMusic
;;; www.csounds.com
;;;
;;; CSOUND SYNTHESIS CALL
;;; J. Bresson, IRCAM 2005
;;;===================================================

(in-package :om)

(defmethod! CSOUND-SYNTH ((sco pathname) (orc pathname) &optional (out-name nil) normalize resolution)
  :icon '(410)
  (setf *flags* *csound-defflags*)
  (if (probe-file *CSOUND-PATH*)
      (let* ((outpath (handle-new-file-exists
                       (corrige-sound-filename 
                        (if out-name out-name (pathname-name sco)) *om-outfiles-folder*)))
             (tmppath (handle-new-file-exists 
                       (om-make-pathname :directory outpath :name (pathname-name outpath) :type "tmp")))
             (csout (if (or normalize *normalize*) tmppath outpath)))
   
        (print "======================================")
        (print "CSOUND SYNTHESIS...")
        (print "======================================")
        (print (format nil "~%Orchestra: ~s~%Score: ~s~%Output: ~s" orc sco csout))
        
        (when (probe-file outpath)
          (print (string+ "Removing existing file: " (namestring outpath)))
          (om-delete-file outpath))
           
        (om-cmd-line 
         (print (format nil "~s ~A ~s ~s -o ~s" 
                        (om-path2cmdpath *CSOUND-PATH*)
                        *flags*
                        (om-path2cmdpath orc)
                        (om-path2cmdpath sco)
                        (om-path2cmdpath csout)
                        ))
         *sys-console*)
        
        (if (null (probe-file csout))
            (om-message-dialog "!!! Error in CSound synthesis !!!")
          
          (progn
            (om-print "END CSOUND SYNTHESIS")
            (when (or normalize *normalize*)
              (let ((real-out (om-normalize tmppath outpath (or normalize *normalize-level*) resolution)))
                (if real-out 
                    (add-tmp-file tmppath)
                  (rename-file tmppath outpath))))
            ))
        
        (when *delete-inter-file* (clean-tmp-files))
        (probe-file outpath))
    (om-beep-msg "CSOUND NOT FOUND")))



(defmethod! CSOUND-SYNTH ((sco t) (orc t) &optional out-name normalize resolution)
            (csound-synth (convert-input-to-csound sco "sco") (convert-input-to-csound orc "orc") 
                          out-name 
                          normalize resolution))


(defmethod convert-input-to-csound ((self string) &optional type)
  (pathname self))

(defmethod convert-input-to-csound ((self pathname) &optional type)
  self)

(defmethod convert-input-to-csound ((self list) &optional type)
  (let ((path (tmpfile (if type (string+ "temcsoundfile." type) "temcsoundfile"))))
    (WITH-OPEN-FILE (out path :direction :output :if-does-not-exist :create :if-exists :supersede)
      (loop for item in self do (write-line item out)))
    (push path *tmpparfiles*)
    path))
    

(defmethod convert-input-to-csound ((self textfile) &optional type)
   (let ((path (tmpfile (if type (string+ "temcsoundfile." type) "temcsoundfile"))))
     (when (buffer-text self)
       (om-buffer-write-file (buffer-text self) path :if-exists :supersede))
         (push path *tmpparfiles*)
         path))


