; OMChroma
; High-level control of sound synthesis in OM
;
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
; (c) Ircam 2000 - 2010
;Authors: C. Agon, M. Stroppa, J. Bresson, S. Lemouton

(in-package :om)


(defvar *OMchroma-files* nil)
   
(setf *OMchroma-files* (list
                         ;CHROMA; KERNEL SYSTEM
                         ;SYS
                        "sources;kernel;init-chroma"
                        "sources;kernel;globals"
                        "sources;kernel;utils-cr"
  
                        ;PLS
                        "sources;kernel;pls;utils-pls"
                        "sources;kernel;pls;all-purp"
                        "sources;kernel;pls;type"
                        "sources;kernel;pls;tbl"
                        "sources;kernel;pls;fun"
                        "sources;kernel;pls;ctl"
                        "sources;kernel;pls;raw"
                        "sources;kernel;pls;ve"
                        "sources;kernel;pls;wt"
                        "sources;kernel;pls;wt-frz"
                        
                        ;DG
                        "sources;kernel;dg;utils-dg" ;new, ms_1109
                        "sources;kernel;dg;df"
                        "sources;kernel;dg;dv" ;new, ms_1109
                        "sources;kernel;dg;spectrum"
                        
                        ;VPS
                        "sources;kernel;vps;globals-vps"
                        "sources;kernel;vps;symbolic-pitch"
                        "sources;kernel;vps;pitch-conversions"
                        "sources;kernel;vps;utils-vps"
                        "sources;kernel;vps;vps"
                        "sources;kernel;vps;chord"
                        "sources;kernel;vps;spectrum"
                        "sources;kernel;vps;user-vps"
                        "sources;kernel;vps;modif-vps"
                        "sources;kernel;vps;modif-empty-vps"
                        "sources;kernel;vps;match"
                        
                        ;MODELS
                        "sources;kernel;models;analysis-data"
                        "sources;kernel;models;f0-data"
                        "sources;kernel;models;cseq-data"
                        "sources;kernel;models;additive-data"
                        "sources;kernel;models;formant-data"
                        "sources;kernel;models;specenv-data"
                        "sources;kernel;models;model"
                        "sources;kernel;models;regions"
                        "sources;kernel;models;f0-model"
                        "sources;kernel;models;specenv-model"
                        "sources;kernel;models;cseq-model"                     
                        "sources;kernel;models;cseq-regions"
                        "sources;kernel;models;partials-model"
                        "sources;kernel;models;partials-regions"
                        "sources;kernel;models;modif-models"
                        "sources;kernel;models;modif-regions"
                        "sources;kernel;models;user-model"
                        "sources;kernel;models;time-processing"
                        "sources;kernel;models;utils-model"
  
                        ;CTL2
                        "sources;kernel;ctl2;globals-ctl2"
                        "sources;kernel;ctl2;ctl2001"
                        "sources;kernel;ctl2;utils-ctl2001"


                        ;----------------------------------------
                        ;OMCHROMA; GRAPHICAL OBJECTS
                        ; - global sources
                        "sources;omchroma;general-parsing"
                        
                        ; - Csound sources
                        "sources;omchroma;csound;cs-utils"
                        "sources;omchroma;csound;cs-tables"
                        "sources;omchroma;csound;csound-evt"
                        "sources;omchroma;csound;csound-tools"
                        "sources;omchroma;csound;csound-parsing"

                        ;- user funs
                        "sources;omchroma;user-funs;user-funs"
                        "sources;omchroma;user-funs;methods"
                       
                        ;- omchroma classes
                        "sources;omchroma;chroma-reference"
                        "sources;omchroma;doc-chroma"

                        ;- new
                        "sources;omchroma2;vps-tools"
                        "sources;omchroma2;vpseditor"
                        "sources;omchroma2;models"
                        "sources;omchroma2;processing"
                        "sources;omchroma2;controls"
                        "sources;omchroma2;gen-model-data" 
                     
			"sources;chromaspat;chromaspat"
                        "sources;chromaspat;classes;stereo-1"
           
                        ))

(defun load-chroma-classes (dir &optional pack)
  (loop for item in (om-directory dir :files t :directories t) do
        (if (directoryp item) 
            (let ((thepackage 
                   (or (and pack (find (car (last (pathname-directory item)))
                                       (subpackages pack) :key 'name :test 'string-equal))
                       pack)))
               (load-chroma-classes item thepackage))    
        (when (string-equal (pathname-type item) "lisp")
          (load item)
          (let ((classname (intern (string-upcase (pathname-name item)))))
            (when (and pack (find-class classname nil))
              (addclass2pack classname pack)))))))



(eval-when (eval compile load)
    (mapc #'(lambda (filename) 
	      ;(compile&load (namestring (make-local-path *load-pathname* filename)))
              (load (namestring (make-local-path *load-pathname* filename)))
              ) 
	  *OMchroma-files*))

(om::fill-library '((nil nil nil (synthesize chroma-prisma) nil)
                    ("Csound" 
                     (("Csound Tables" nil (cs-table gen-cs-table gen01 gen02 gen-02 gen05 gen07 gen-07) nil nil)
                      ("Basic Classes" nil nil nil nil)
                      ("Advanced Classes" nil nil nil nil))
                     nil nil nil)
                    ("Vertical Pitch Structures" 
                     (("Inspect" nil nil (get-vps-freqs get-vps-freq-ratios get-vps-intervals 
                                                        get-vps-amps get-vps-maxamp get-vps-minamp
                                                        ;;; Lisp functions:
                                                        cr::get-homogeneity cr::get-harmonicity cr::get-surface 
                                                        cr::get-density cr::get-sd cr::get-cs cr::get-virt-fund cr::match) nil)
                      ("Processing" nil nil (low-pass-filter high-pass-filter band-pass-filter
                                                             main-partials n-main-partials stretch-vps pitch-transpose
                                                             xpose-begin xpose-end
                                                             revert-vps mirror-vps) nil)
                      ("Conversions" nil nil (pch->fq fq->pch fq->ratio) nil))
                     (cr::fql cr::ptl cr::crl cr::arl cr::spl cr::cil cr::ail) nil nil)
                    ("Models" 
                     (("Inspect" nil nil (model-max-amp model-max-freq model-min-freq model-nb-evts) nil)
                      ("Tools" nil nil (make-cr-fun time-map-fun add-random) nil))
                     (cr-model) (model-data expand-model) nil)
                    )
                  (find-library "OMChroma"))



  (let* ((chromapack (or *current-lib* (find-library "OMChroma")))
       (cspack (find "Csound" (subpackages chromapack) :key 'name :test 'string-equal))
       (basicpack (find "Basic Classes" (subpackages cspack) :key 'name :test 'string-equal))
       (advpack (find "Advanced Classes" (subpackages cspack) :key 'name :test 'string-equal)))
  (load-chroma-classes (om-relative-path '("sources" "omchroma" "csound" "classes" "Basic") nil) basicpack)
  (load-chroma-classes (om-relative-path '("sources" "omchroma" "csound" "classes" "Advanced") nil) advpack)
  )


; (cl-user::clean-sources (om-make-pathname :directory *load-pathname*))


(unless (fboundp 'om::set-lib-release) (defmethod om::set-lib-release (version &optional lib) nil))

(om::set-lib-release 4.0 (find-library "OMChroma"))


(print "
 ======================================
 OMChroma
 ======================================
 High-level control of sound synthesis in OM
 (c) Ircam 2000-2011
 C. Agon, M. Stroppa, J. Bresson, S. Lemouton
 ======================================
")


; (gen-lib-reference *chromapack*)
; (init-chroma-ref-dir)
; (gen-chroma-reference *chroma-ref-entries* *chroma-ref-directory* :title "OMChroma Class Reference" :general-doc *chroma-general-doc* :doc-base *chroma-slots-doc* :appendix *chroma-appendix*)
