; *************************************************************
; | OMprisma 2.2,                                             |
; |  2010 M.Schumacher (CIRMMT, McGill University)           |
; | in collaboration with IRCAM's Music Representations Group |
; *************************************************************
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher, J.Bresson

(in-package :om)

; %%%%%%%%%%%%% INITIALIZATION %%%%%%%%%%%%%%

(require-library "OMChroma" t)
(setf *auto-init-table-id* 1000)
(set-pref (find-pref-module :externals) :csound-options (list "-f -m7 -A -N -g -b8192 -B8192" "f 1 0 513 7 0 256 1 256 0" 999 513))
;(setf *cs-max-points* 999)

;set background picture
(defmethod get-fonde-pict ((self cs-spat-evt)) *my-bg-pict*)
(setq *my-bg-pict* (om-load-and-store-picture "dibox" 'omprisma))
(defparameter *omprisma-version* 2.2)
(defparameter *release-date* (subseq (sys::date-string nil nil) 0 10))

; gen-reference
; (omprisma-reference)

; %%%%%%%%%%%%%%% LOAD FILES %%%%%%%%%%%%%%%%
(mapcar #'(lambda (file) (compile&load file))
        (list 
         (om-relative-path '("sources") "abstract-classes")
         (om-relative-path '("sources") "ls-configs")
         (om-relative-path '("sources") "distance-cues")
         (om-relative-path '("sources") "macros+udos")
         (om-relative-path '("sources") "omprisma-reference")
         (om-relative-path '("sources") "parsing-funs")
         (om-relative-path '("sources") "spat-trajectory-evt") 
         (om-relative-path '("sources") "trajectories") 
         (om-relative-path '("sources") "utilities") 
         (om-relative-path '("sources") "visualizations")
         (om-relative-path '("sources") "prisma-sdif")
         (om-relative-path '("sources") "general-setup")
         (om-relative-path '("sources" "renderers" ) "ambi")
         (om-relative-path '("sources" "renderers" ) "spat") 
         (om-relative-path '("sources" "renderers" ) "babo")
         (om-relative-path '("sources" "renderers" ) "dbap")
         (om-relative-path '("sources" "renderers" ) "panning")
         (om-relative-path '("sources" "renderers" ) "rvbap")        
         (om-relative-path '("sources" "renderers" ) "vbap") 
         (om-relative-path '("sources" "renderers" ) "vimic") 
         ))

#|
(defun recursive-compile-classes  (dir &optional pack)
  (loop for item in (om-directory dir) do
        (print item)
        (if (directoryp item) 
            (let ((thepackage 
                   (or (and pack (find (car (last (pathname-directory item)))
                                       (subpackages pack) :key 'name :test 'string-equal))
                       pack)))
               (recursive-load-classes item thepackage))    
        (when (string-equal (pathname-type item) "lisp")
          (c; evaluate this line to generate the documentation
;; (omprisma-reference)ompile&load item)
          (let ((classname (intern (string-upcase (pathname-name item)))))
            (when (and pack (find-class classname nil))
              (addclass2pack classname pack))))))))
|#

(defun recursive-compile-classes  (dir &optional pack)
  (loop for item in (om-directory dir) do
        (print item)
        (if (directoryp item) 
            (let ((thepackage 
                   (or (and pack (find (car (last (pathname-directory item)))
                                       (subpackages pack) :key 'name :test 'string-equal))
                       pack)))
               (recursive-load-classes item thepackage))    
        (when (string-equal (pathname-type item) "lisp")
          (compile&load item)
          (let ((classname (intern (string-upcase (pathname-name item)))))
            (when (and pack (find-class classname nil))
              (addclass2pack classname pack)))))))

(defun recursive-load-classes (dir &optional pack)
  (loop for item in (om-directory dir) do
        (print item)
        (if (directoryp item) 
            (let ((thepackage 
                   (or (and pack (find (car (last (pathname-directory item)))
                                       (subpackages pack) :key 'name :test 'string-equal))
                       pack)))
               (recursive-load-classes item thepackage))    
        (when (string-equal (pathname-type item) "lisp")
          (load item)
          (let ((classname (intern (string-upcase (pathname-name item)))))
            (when (and pack (find-class classname nil))
              (print (list classname pack))
              (addclass2pack classname pack)))))))

(om::fill-library '(
                    ("Ambisonics" 
                     nil nil nil nil)
                    ("BABO" 
                     nil nil nil nil)
                    ("DBAP" 
                     nil nil nil nil)
                    ("Panning" 
                     nil nil nil nil)
                    ("RVBAP" 
                     nil nil nil nil)
                    ("SPAT" 
                     nil nil nil nil)
                    ("VBAP" 
                    nil nil nil nil)   
                    ("ViMiC" 
                     nil nil nil nil)
                    ("Trajectories" nil nil (gen-trajectory traj-interpol traj-scale traj-mirror traj-rotate traj-perturb traj-dur) nil)                
                    ("Utilities" (
                                  ("coordinates" nil nil (xy->ad ad->xy aed->xyz xyz->aed) nil)
                                  ("temporal" nil nil (add-dur tempo-traj dist-delay) nil)
                                  ("tables" nil nil (gen-table scale-cs-table) nil)
                                  ("BPFs" nil nil (splines bpf-interpol value->bpf) nil)
                                  ("Loudspeaker-config" nil nil (speaker-setup) nil))
                                 nil nil nil)
                    ("Distance-Cues" (
                                      ("distance-attenuation" nil nil (atten-invprop atten-exp) nil)
                                      ("air-absorption" nil nil (air-valente air-icst air-furse) nil)
                                      ("doppler-effect" nil nil (doppler) nil)
                                      nil nil nil))
                    ("Class-specific" (
                                       ("Ambisonics" nil nil (ambi-setup spread->order) nil)
                                       ("BABO" nil nil (babo-setup) nil)
                                       ("DBAP" nil nil (dbap-setup) nil)
                                       ("Panning" nil nil (pan-setup) nil)
                                       ("SPAT" nil nil (spat-setup spat-room spat-wall) nil)                                  
                                       ("VBAP" nil nil (vbap-setup) nil)
                                       ("Rvbap" nil nil (rvbap-setup rvbap-reverb) nil)
                                       ("ViMiC" nil nil (vimic-setup microphone-array mic-pattern) nil))
                                      nil nil nil)))
                                      ;)))
 
(recursive-load-classes (om-relative-path '("sources" "classes") nil) (find-library "OMPrisma"))
;(recursive-compile-classes (om-relative-path '("sources" "classes" "vbap") nil) (find-library "OMPrisma"))


(print "
 =======================================
|               OMPrisma                |
|                                       |
| Library for Spatial Sound Synthesis   |
|            in OpenMusic               |
|                                       |
|    2009-2011  Marlon Schumacher      |
| IDMIL/DCS/CIRMMT, McGill University,  |
|  in collab. with IRCAM RepMus team.   |
 =======================================
")
