(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			 CLASS AMBI.CONTINUOUS
;      	    continuous control of ambisonic sound spatialization
;                     with optional distance cues
;----------------------------------------------------------------------------

(defclass! ambi.continuous (prisma-ambi prisma-distance-cues prisma-continuous)
  			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "ambi.continuous" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "ambi.continuous")) 16) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "ambi.continuous")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *ambi-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)


   ( order-env
     :type GEN-07
     :initarg :order-env
     :initform (make-cs-table  'Gen-07  '(0 512) '(3 3) 5 "?" 513)
     :accessor order-env)

   ( order-freqenv
     :type GEN-07 ;; NB: if 0 1/duration (1 cycle per event)
     :initarg :order-freqenv 
     :initform 0
     :accessor order-freqenv)

   )

  (:documentation 
"
;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   	  AMBI.CONTINUOUS.ORC 
; 	   	    	 AMBISONIC SPATIAL SOUND RENDERER
;     		    WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: 3rd-order ambisonics
; Continuous control of position and encoding order
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Coded: 11/2010
; 
; Panning algorithm based on: BFORMENC1
; Author: Richard Furse, Bruce Wiggins and Fons Adriaensen, 
; following code by Samuel Groner 2008	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 			[sec]
;	p3	= duration 			[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 			[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 		[linear]
;	p7	= starting point in file 	[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 		[GEN number]
;	p9	= Y-position envelope 		[GEN number]
;	p10	= Z-position envelope 		[GEN number]
;	p11	= ambisonic order envelope 	[GEN number]
;	p12	= X-position frequency-envelope [GEN number]
;	p13	= Y-position frequency-envelope [GEN number]
;	p14	= Z-position frequency-envelope [GEN number]
;	p15	= order frequency-envelope	[GEN number]
;	--------Distance Cues-------------------------------
;	p16	= distance-attenuation function [GEN number]
;	p17	= air-absorption function   	[GEN number]
;	p18	= doppler-effect function  	[GEN number]
;	----------------------------------------------------
;	p19	= center-radius			[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS:
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f401		time of travel 		(lookup table)
;____________________________________________________________________________
")
  (:icon 14)
  )

(defmethod ignore-pfields ((self ambi.continuous)) '("p5" "p6" "p7"))

