;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   	  AMBI.CONTINUOUS.ORC 
; 	   	    	 AMBISONIC SPATIAL SOUND RENDERER
;     		    WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: 3rd-order ambisonics
; Continuous control of position and encoding order
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Coded: 11/2010
; 
; Panning algorithm based on: BFORMENC1
; Author: Richard Furse, Bruce Wiggins and Fons Adriaensen, 
; following code by Samuel Groner 2008	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 			[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 			[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 	[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 		[GEN number]
;	p9	= Y-position envelope 		[GEN number]
;	p10	= Z-position envelope 		[GEN number]
;	p11	= ambisonic order envelope 	[GEN number]
;	p12	= X-position frequency-envelope [GEN number]
;	p13	= Y-position frequency-envelope [GEN number]
;	p14	= Z-position frequency-envelope [GEN number]
;	p15	= order frequency-envelope	[GEN number]
;	--------Distance Cues-------------------------------
;	p16	= distance-attenuation function [GEN number]
;	p17	= air-absorption function   	[GEN number]
;	p18	= doppler-effect function  	[GEN number]
;	----------------------------------------------------
;	p19	= center-radius			[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS:
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f401			time of travel 		(lookup table)
;____________________________________________________________________________
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 16
0dbfs 	= 8388607

instr 1 
;; SOUNDFILE PLAYER *****************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope
   
;; SPATIAL SOUND RENDERER *************

ixpos_envelope 		= p8
iypos_envelope 		= p9
izpos_envelope		= p10
ixpos_frequency_envelope 	= p11
iypos_frequency_envelope 	= p12
izpos_frequency_envelope 	= p13

iattenfunction		= (p14 = 0 ? 200 : (p14 = 1 ? 201 : (p14 = 2 ? 202 : p14)))
iairfunction		= (p15 = 1 ? 301 : (p15 = 2 ? 302 : p15))  
itimefunction		= (p16 = 1 ? 401 : p16)
icentersize			= p17

iorder_envelope			= p18
iorder_frequency_envelope	= p19

;;--- COPY SOUND SOURCE

$ambi_copy_source

;;--- ENVELOPES
     	     	
$ambi_envelopes

;;--- Cartesian to Polar Coordinates

$xyz2aed_continuous

	kindex = kdistance1*0.02

;;--- DISTANCE CUES

timeoftravel:
cggoto (itimefunction = 0), airabsorption
$timeoftravel_continuous
;asound1 timeoftravel_continuous asound1, kdistance1, itimefunction

airabsorption:
cggoto (iairfunction = 0), attenuation
$airabsorption_continuous
;asound1 airabsorption_continuous asound1, kdistance1, iairfunction

attenuation:
cggoto (iattenfunction = 0), panning
$attenuation_continuous
;asound1 attenuation_continuous asound1, kdistance1, iattenfunction

;;--- PANNING ALGORITHM

panning:		
	; order-control (decrease encoding-order whithin center-zone)	
	korder = (kdistance1 > icentersize ? korder : korder*kdistance1*(1/icentersize))	
	kordg1 limit korder, 0, 1
	kordg2 limit (korder-1), 0, 1
	kordg3 limit (korder-2), 0, 1	

cggoto (gidims = 3), with_height
cggoto (giorder = 1), first_order_2d
cggoto (giorder = 2), second_order_2d
cggoto (giorder = 3), third_order_2d
cggoto (giorder = 0), uhj
with_height:
cggoto (giorder = 1), first_order_3d
cggoto (giorder = 2), second_order_3d
cggoto (giorder = 3), third_order_3d


first_order_2d:
        aw, ax, ay, az bformenc1 asound1, -kazimuth1, 0
        outc aw, ax*kordg1, ay*kordg1
goto end

first_order_3d:
        aw, ax, ay, az bformenc1 asound1, -kazimuth1, kelevation1
        outc aw, ax*kordg1, ay*kordg1, az*kordg1
goto end

second_order_2d:
        aw, ax, ay, az, ar, as, at, au, av bformenc1 asound1, -kazimuth1, 0
        outc aw,ax*kordg1,ay*kordg1,au*kordg2,av*kordg2
goto end

second_order_3d:
        aw, ax, ay, az, ar, as, at, au, av bformenc1 asound1, -kazimuth1, kelevation1
        outc aw, ax*kordg1, ay*kordg1, az*kordg1, ar*kordg2, as*kordg2, at*kordg2, au*kordg2, av*kordg2
goto end

third_order_2d:
        aw, ax, ay, az, ar, as, at, au, av, ak, al, am, an, ao, ap, aq bformenc1 asound1, -kazimuth1, 0
        outc aw,ax*kordg1,ay*kordg1,au*kordg2,av*kordg2,ap*kordg3,aq*kordg3
goto end     

third_order_3d:
        aw, ax, ay, az, ar, as, at, au, av, ak, al, am, an, ao, ap, aq bformenc1 asound1, -kazimuth1, kelevation1
           	outc aw, ax*kordg1, ay*kordg1, az*kordg1, ar*kordg2, as*kordg2, at*kordg2, au*kordg2, av*kordg2, ak*kordg3, al*kordg3, am*kordg3, an*kordg3, ao*kordg3, ap*kordg3, aq*kordg3 
goto end  

uhj:
	aw, ax, ay, az bformenc1 asound1, -kazimuth1, 0
	aWreal, aWimaginary	hilbert aw
	aXreal, aXimaginary	hilbert ax
	aYreal, aYimaginary	hilbert ay
	aWXreal		=  0.0928*aXreal + 0.4699*aWreal
	aWXimaginary	=  0.2550*aXimaginary - 0.1710*aWimaginary
	aLeft			=  aWXreal + aWXimaginary + 0.3277*aYreal
	aRight		=  aWXreal - aWXimaginary - 0.3277*aYreal
	outc aLeft, aRight

end:	
endin
