(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			 CLASS AMBI.DISCRETE
;      	    discrete control of ambisonic sound spatialization
;                     with optional distance cues
;----------------------------------------------------------------------------

(defclass! ambi.discrete (prisma-ambi prisma-distance-cues prisma-discrete)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "ambi.discrete" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "ambi.discrete")) 16) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "ambi.discrete")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *ambi-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)

   ( order
	:type number
        :initarg :order
        :initform 3
        :accessor order)

   )

  (:documentation 
"
;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   		AMBI.DISCRETE.ORC 
; 	   	     	    AMBISONIC SPATIAL SOUND RENDERER
;     		        WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: 3rd-order ambisonics
; Discrete control of source position and encoding order
; Distance Cues: Attenuation, Air-absorption, Time-of-travel
; Coded: 11/2010
; 
; Panning algorithm based on: BFORMENC1
; Author: Richard Furse, Bruce Wiggins and Fons Adriaensen, 
; following code by Samuel Groner 2008	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 			[sec]
;	p3	= duration 			[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 			[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 		[linear]
;	p7	= starting point in file 	[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  			[cartesian X-coordinate]
;	p9	= Y-position 	 		[cartesian Y-coordinate]
;	p10	= Z-position 	 		[cartesian Z-coordinate]
;	p11	= ambisonic order	 	[float]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function [GEN number]
;	p13	= air-absorption function   	[GEN number]
;	p14	= timeoftravel function   	[GEN number]
;	----------------------------------------------------
;	p15	= center-radius			[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS:
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f401		time of travel 		(lookup table)
;____________________________________________________________________________	
")
  (:icon 14)
  )

(defmethod ignore-pfields ((self ambi.discrete)) '("p5" "p6" "p7"))
