;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   		AMBI.DISCRETE.ORC 
; 	   	     	    AMBISONIC SPATIAL SOUND RENDERER
;     		        WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: 3rd-order ambisonics
; Discrete control of source position and encoding order
; Distance Cues: Attenuation, Air-absorption, Time-of-travel
; Coded: 11/2010
; 
; Panning algorithm based on: BFORMENC1
; Author: Richard Furse, Bruce Wiggins and Fons Adriaensen, 
; following code by Samuel Groner 2008	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 			[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 			[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 	[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  			[cartesian X-coordinate]
;	p9	= Y-position 	 		[cartesian Y-coordinate]
;	p10	= Z-position 	 		[cartesian Z-coordinate]
;	p11	= ambisonic order	 		[float]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function [GEN number]
;	p13	= air-absorption function   	[GEN number]
;	p14	= timeoftravel function   	[GEN number]
;	----------------------------------------------------
;	p15	= center-radius			[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS:
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f401			time of travel 		(lookup table)
;____________________________________________________________________________	
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 16
0dbfs 	= 8388607

instr 1 
;; SOUNDFILE PLAYER *****************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope
   
;; SPATIAL SOUND RENDERER *****************************************************

ixpos 		= p8
iypos			= p9
izpos			= p10

iattenfunction	= (p11 = 0 ? 200 : (p11 = 1 ? 201 : (p11 = 2 ? 202 : p11)))
iairfunction	= (p12 = 1 ? 301 : (p12 = 2 ? 302 : p12)) 
itimefunction	= (p13 = 1 ? 401 : p13)
icentersize		= p14

iorder		limit p15, 0, 3

;;--- COPY SOUND SOURCE

$ambi_copy_source

;;--- Cartesian to Polar Coordinates

$xyz2aed_discrete

	index = idistance1*0.02
	
;;--- DISTANCE CUES

timeoftravel:
cggoto (itimefunction = 0), airabsorption
$timeoftravel_discrete
;asound1 timeoftravel_discrete asound1, idistance1, itimefunction

airabsorption:
cggoto (iairfunction = 0), attenuation
$airabsorption_discrete
;asound1 airabsorption_discrete asound1, idistance1, iairfunction

attenuation:
cggoto (iattenfunction = 0), panning
$attenuation_discrete
;asound1 attenuation_discrete asound1, idistance1, iattenfunction

;;--- PANNING ALGORITHM

panning:
	; order-control (decrease encoding order whithin center-zone)	
	iorder = (idistance1 > icentersize ? iorder : iorder*idistance1*(1/icentersize))	
	iordg1 limit iorder, 0, 1
	iordg2 limit (iorder-1), 0, 1
	iordg3 limit (iorder-2), 0, 1
	
cggoto (gidims = 3), with_height
cggoto (giorder = 1), first_order_2d
cggoto (giorder = 2), second_order_2d
cggoto (giorder = 3), third_order_2d
cggoto (giorder = 0), uhj
with_height:
cggoto (giorder = 1), first_order_3d
cggoto (giorder = 2), second_order_3d
cggoto (giorder = 3), third_order_3d


first_order_2d:
        aw, ax, ay, az bformenc1 asound1, -iazimuth1, 0
        outc aw, ax*iordg1, ay*iordg1
goto end

first_order_3d:
        aw, ax, ay, az bformenc1 asound1, -iazimuth1, ielevation1
        outc aw, ax*iordg1, ay*iordg1, az*iordg1
goto end

second_order_2d:
        aw, ax, ay, az, ar, as, at, au, av bformenc1 asound1, -iazimuth1, 0
        outc aw,ax*iordg1,ay*iordg1,au*iordg2,av*iordg2
goto end

second_order_3d:
        aw, ax, ay, az, ar, as, at, au, av bformenc1 asound1, -iazimuth1, ielevation1
        outc aw, ax*iordg1, ay*iordg1, az*iordg1, ar*iordg2, as*iordg2, at*iordg2, au*iordg2, av*iordg2
goto end

third_order_2d:
        aw, ax, ay, az, ar, as, at, au, av, ak, al, am, an, ao, ap, aq bformenc1 asound1, -iazimuth1, 0
        outc aw,ax*iordg1,ay*iordg1,au*iordg2,av*iordg2,ap*iordg3,aq*iordg3
goto end     

third_order_3d:
        aw, ax, ay, az, ar, as, at, au, av, ak, al, am, an, ao, ap, aq bformenc1 asound1, -iazimuth1, ielevation1
           	outc aw, ax*iordg1, ay*iordg1, az*iordg1, ar*iordg2, as*iordg2, at*iordg2, au*iordg2, av*iordg2, ak*iordg3, al*iordg3, am*iordg3, an*iordg3, ao*iordg3, ap*iordg3, aq*iordg3 
goto end  

uhj:
	aw, ax, ay, az bformenc1 asound1, -iazimuth1, 0
	aWreal, aWimaginary	hilbert aw
	aXreal, aXimaginary	hilbert ax
	aYreal, aYimaginary	hilbert ay
	aWXreal		=  0.0928*aXreal + 0.4699*aWreal
	aWXimaginary	=  0.2550*aXimaginary - 0.1710*aWimaginary
	aLeft			=  aWXreal + aWXimaginary + 0.3277*aYreal
	aRight		=  aWXreal - aWXimaginary - 0.3277*aYreal
	outc aLeft, aRight

end:	
endin
