(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			 CLASS BABO.CONTINUOUS
;     continous control of Rocchesso's Ball-in-a-Box Resonator Model
;----------------------------------------------------------------------------

(defclass! babo.continuous (prisma-babo prisma-continuous)
			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "babo.continuous" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "babo.continuous")) 4) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "babo.continuous")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *babo-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)


   ; RESONATOR MODEL -----------------------

   ( reso-params
	:type cs-table ;; NB: if 1,2,3 predefined functions (471,472,473) else tables
        :initarg :reso-params 
        :initform 1
        :accessor reso-params)
   ) 

  (:documentation 
"
;============================================================================
;      OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			        BABO.CONTINUOUS.ORC 
; 	   	      BALL-IN-A-BOX MODEL BY D.ROCHESSO
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: babo for arbitrary speaker setups
; Control of position, resonator-characteristics and frequency of envelopes
; 
;babo stands for ball-within-the-box, a physical model reverberator based on 
;D. Rocchesso, 'The Ball within the Box: a sound-processing metaphor', 
;Computer Music Journal, Vol 19, N.4, pp.45-47, Winter 1995.
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    				[if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= Z-position envelope 			[GEN number]
;	p11	= X-position frequency-envelope 	[GEN number]
;	p12	= Y-position frequency-envelope 	[GEN number]
;	p13	= Z-position frequency-envelope 	[GEN number]
;	--------Resonator Params-------------------------------
;	p14	= Reso-params				[GEN number]
;----------------------------------------------------------------------------
")
  (:icon 70)
  )

(defmethod ignore-pfields ((self babo.continuous)) '("p5" "p6" "p7"))