;============================================================================
;      OMprisma 2.0, © 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			        BABO.CONTINUOUS.ORC 
; 	   	      BALL-IN-A-BOX MODEL BY D.ROCHESSO
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: babo for arbitrary speaker setups
; Control of position, resonator-characteristics and frequency of envelopes
; 
;babo stands for ball-within-the-box, a physical model reverberator based on 
;D. Rocchesso, "The Ball within the Box: a sound-processing metaphor", 
;Computer Music Journal, Vol 19, N.4, pp.45-47, Winter 1995.
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 					[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    				[if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 				[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= Z-position envelope 			[GEN number]
;	p11	= X-position frequency-envelope 	[GEN number]
;	p12	= Y-position frequency-envelope 	[GEN number]
;	p13	= Z-position frequency-envelope 	[GEN number]
;	--------Resonator Params-------------------------------
;	p14	= Reso-params				[GEN number]
;----------------------------------------------------------------------------
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 4
0dbfs 	= 8388607

instr 1 
;; SOUNDFILE PLAYER **********************************************************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope
   
;; SPATIAL SOUND RENDERER *****************************************************

ixpos_envelope 		= p8
iypos_envelope 		= p9
izpos_envelope		= p10
ixpos_frequency_envelope 	= p11
iypos_frequency_envelope 	= p12
izpos_frequency_envelope 	= p13
iresoparams			= (p14 = 1 ? 471 : (p14 = 2 ? 472 : (p14 = 3 ? 473 : p14)))

;; table operations ------------------------

;get the diffusion value from the reso-table
idiffusion tab_i 8, iresoparams

;get the resonator size values from the reso-table
isizex tab_i 9, iresoparams
isizey tab_i 10, iresoparams
isizez tab_i 11, iresoparams

;;---------------- ENVELOPES ----------------
     	     	
$babo_envelopes

;;-------------- BALL IN A BOX ---------------

$babo_continuous

endin
