(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			 CLASS DBAP.CONTINUOUS
;      	  continous control of distance-based amplitude panning
;                     with optional distance-cues
;----------------------------------------------------------------------------

(defclass! dbap.continuous (prisma-dbap prisma-distance-cues prisma-continuous)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "dbap.continuous" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "dbap.continuous")) 4) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "dbap.continuous")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *dbap-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)
   

   ( blur-env
     :type GEN-07
     :initarg :blur-env
     :initform (make-cs-table  'Gen-07  '(0 512) '(0 0) 5 "?" 513)
     :accessor blur-env)

   ( blur-freqenv
	:type GEN-07 ;; NB: if 0 1/duration (1 cycle per event)
        :initarg :blur-freqenv 
        :initform 0
        :accessor blur-freqenv)

   ( speaker-weights
	:type GEN-02
        :initarg :speaker-weights
        :initform 0 ;; NB: if 0 equal weights (501), else tables
        :accessor speaker-weights)

   ( mode
	:type number
        :initarg :mode
        :initform 1
        :accessor mode)
   ) 

  (:documentation 
"
;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   	  DBAP.CONTINUOUS.ORC 
; 	   	       DISTANCE-BASED AMPLITUDE PANNING
;     		    WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: dbap for arbitrary speaker setups
; Trajectory control of sound source position and blur
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Coded: 10/2010, Marlon Schumacher
; 
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 					[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 				[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= Z-position envelope 			[GEN number]
;	p11	= blur envelope	 			[GEN number]
;	p12	= X-position frequency-envelope 	[GEN number]
;	p13	= Y-position frequency-envelope 	[GEN number]
;	p14	= Z-position frequency-envelope 	[GEN number]
;	p15	= blur frequency-envelope		[GEN number]
;	--------Distance Cues-------------------------------
;	p16	= distance-attenuation function 	[GEN number]
;	p17	= air-absorption function   		[GEN number]
;	p18	= doppler-effect function  		[GEN number]
;	----------------------------------------------------
;	p19	= speaker-weights				[GEN number]
;	p20	= mode					[int]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202		distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f401			doppler effect	 	(lookup table)
;	f501			speaker-params		(parameter table)
;----------------------------------------------------------------------------
")
  (:icon 71)
  )

(defmethod ignore-pfields ((self dbap.continuous)) '("p5" "p6" "p7"))