;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   	  DBAP.CONTINUOUS.ORC 
; 	   	       DISTANCE-BASED AMPLITUDE PANNING
;     		    WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: dbap for arbitrary speaker setups
; Trajectory control of sound source position and blur
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Coded: 10/2010, Marlon Schumacher
; 
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 					[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 				[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= Z-position envelope 			[GEN number]
;	p11	= blur envelope	 			[GEN number]
;	p12	= X-position frequency-envelope 	[GEN number]
;	p13	= Y-position frequency-envelope 	[GEN number]
;	p14	= Z-position frequency-envelope 	[GEN number]
;	p15	= blur frequency-envelope		[GEN number]
;	--------Distance Cues-------------------------------
;	p16	= distance-attenuation function 	[GEN number]
;	p17	= air-absorption function   		[GEN number]
;	p18	= doppler-effect function  		[GEN number]
;	----------------------------------------------------
;	p19	= speaker-weights				[GEN number]
;	p20	= mode					[int]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202		distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f401			doppler effect	 	(lookup table)
;	f501			speaker-params		(parameter table)
;----------------------------------------------------------------------------
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 8
0dbfs 	= 8388607

instr 1 
;; SOUNDFILE PLAYER *****************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope
   
   
;; SPATIAL SOUND RENDERER ************

ixpos_envelope 		= p8
iypos_envelope 		= p9
izpos_envelope		= p10

ixpos_frequency_envelope 	= p11
iypos_frequency_envelope 	= p12
izpos_frequency_envelope 	= p13

iattenfunction		= (p14 = 1 ? 201 : (p14 = 2 ? 202 : p14))
iairfunction		= (p15 = 1 ? 301 : (p15 = 2 ? 302 : p15))  
itimefunction		= (p16 = 1 ? 401 : p16)
;irefdistance		= p17

iblur_envelope			= p18
iblur_frequency_envelope	= p19
iweightfunction		= (p20 = 0 ? 501 : p20)
imode				= p21

;;--- COPY SOUND SOURCE

$dbap_copy_source

;;--- ENVELOPES
     	     	
$dbap_envelopes

;--- re-scale blur according to distance covariance
	
	kblur	=	(kblur*0.01)*gidistance_covariance

;--- calculate distance from coordinates

$distance_continuous

;;--- DISTANCE CUES

timeoftravel:
cggoto (itimefunction = 0), airabsorption
$timeoftravel_continuous
;asound1 timeoftravel_continuous asound1, kdistance1, itimefunction

airabsorption:
cggoto (iairfunction = 0), attenuation
$airabsorption_continuous
;asound1 airabsorption_continuous asound1, kdistance1, iairfunction

attenuation:
cggoto (iattenfunction = 0), modecheck
$dbap_attenuation_continuous

modecheck:
;;--- PANNING ALGORITHM
; check for mode --------------------------
; mode 1 = no normalization, no convex hull
; mode 2 = normalization, no convex hull
; mode 3 = normalization, convex hull

cggoto (imode = 1), mode1
cggoto (imode = 2), mode2
cggoto (imode = 3), mode3

mode1:	
$dbap_continuous1
goto end

mode2:
$dbap_continuous2			
goto end

mode3:
;not implemented yet
$dbap_continuous3
goto end

end:
endin
