(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			 CLASS DBAP.DISCRETE
;      	  discrete control of distance-based amplitude panning 
;                     with optional distance cues
;----------------------------------------------------------------------------

(defclass! dbap.discrete (prisma-dbap prisma-distance-cues prisma-discrete)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "dbap.discrete" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "dbap.discrete")) 8) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "dbap.discrete")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *dbap-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)


   ( blur
	:type number
        :initarg :blur
        :initform 0
        :accessor blur)

   ( speaker-weights
	:type GEN-02
        :initarg :speaker-weights
        :initform 0 ;; NB: if 0 equal weights (501), else tables
        :accessor speaker-weights)

   ( mode
	:type number
        :initarg :mode
        :initform 1
        :accessor mode)

   )

  (:documentation 
"
;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   	  DBAP.DISCRETE.ORC 
; 	   	       DISTANCE-BASED AMPLITUDE PANNING
;     		    WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: dbap for arbitrary speaker setups
; Trajectory control of sound source position and blur
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Coded: 10/2010, Marlon Schumacher
; 
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 					[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 				[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  				[cartesian X-coordinate]
;	p9	= Y-position 	 			[cartesian Y-coordinate]
;	p10	= Z-position 	 			[cartesian Y-coordinate]
;	p11	= blur		 		[percent]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function	[GEN number]
;	p13	= air-absorption function   		[GEN number]
;	p14	= time-of-travel function   		[GEN number]
;	----------------------------------------------------
;	p15	= speaker-weights				[GEN number]
;	p16	= mode					[int]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202		distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f401			doppler effect	 	(lookup table)
;	f501			speaker-params		(parameter table)
;____________________________________________________________________________
")
  (:icon 71)
  )

(defmethod ignore-pfields ((self dbap.discrete)) '("p5" "p6" "p7"))
