;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   	  DBAP.DISCRETE.ORC 
; 	   	       DISTANCE-BASED AMPLITUDE PANNING
;     		    WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: dbap for arbitrary speaker setups
; Trajectory control of sound source position and blur
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Coded: 10/2010, Marlon Schumacher
; 
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 					[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 				[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  				[cartesian X-coordinate]
;	p9	= Y-position 	 			[cartesian Y-coordinate]
;	p10	= Z-position 	 			[cartesian Y-coordinate]
;	p11	= blur		 			[percent]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function	[GEN number]
;	p13	= air-absorption function   		[GEN number]
;	p14	= time-of-travel function   		[GEN number]
;	----------------------------------------------------
;	p15	= speaker-weights				[GEN number]
;	p16	= mode					[int]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202		distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f401			doppler effect	 	(lookup table)
;	f501			speaker-params		(parameter table)
;____________________________________________________________________________
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 8
0dbfs 	= 8388607

instr 1 
;; SOUNDFILE PLAYER *****************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope

;; SPATIAL SOUND RENDERER ************

ixpos			= p8 
iypos			= p9
izpos			= p10

iattenfunction	= (p11 = 1 ? 201 : (p11 = 2 ? 202 : p11))
iairfunction	= (p12 = 1 ? 301 : (p12 = 2 ? 302 : p12))  
itimefunction	= (p13 = 1 ? 401 : p13)
;irefdistance	= p14

iblur			= p15
iweightfunction	= (p16 = 0 ? 501 : p16)
imode			= p17

;;--- COPY SOUND SOURCE

$dbap_copy_source

;--- calculate distance from coordinates

$distance_discrete

;--- re-scale blur according to distance covariance --> ?

iblur	=	iblur*0.01*gidistance_covariance

;;--- DISTANCE CUES

timeoftravel:
cggoto (itimefunction = 0), airabsorption
$timeoftravel_discrete
;asound1 timeoftravel_discrete asound1, idistance1, itimefunction

airabsorption:
cggoto (iairfunction = 0), attenuation
$airabsorption_discrete
;asound1 airabsorption_discrete asound1, idistance1, iairfunction

attenuation:
cggoto (iattenfunction = 0), modecheck
$dbap_attenuation_discrete

modecheck:
;;--- PANNING ALGORITHM
; check for mode --------------------------
; mode 1 = no normalization, no convex hull
; mode 2 = normalization, no convex hull
; mode 3 = normalization, convex hull

cggoto (imode = 1), mode1
cggoto (imode = 2), mode2
cggoto (imode = 3), mode3

mode1:	
$dbap_discrete1
goto end

mode2:
$dbap_discrete2			
goto end

mode3:
;not implemented yet
$dbap_discrete3
goto end

end:
endin
