(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			 CLASS PAN.CONTINUOUS
;       continuous control of panning via variable panning-laws
;                      with optional distance cues
;----------------------------------------------------------------------------

(defclass! pan.continuous (prisma-panning prisma-distance-cues prisma-continuous)
   			
  (
   (source-code :initform
                (load-buffer-textfile
                 (get-orc-source (get-orc "pan.continuous" :local))
                 'textfile "append")
                :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "pan.continuous")) 5) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "pan.continuous")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *panning-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)

   ( pan-fun
	:type GEN-07 ;; NB: if 0,1,2 predefined function (100, 101, 102) else tables
        :initarg :pan-fun
        :initform 0
        :accessor pan-fun)

   )

  (:documentation 
"
;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			     PAN.CONTINUOUS.ORC
; 			CONTINUOUS PANNING MODULE
;			WITH OPTIONAL DISTANCE CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: AMPLITUDE PANNING WITH DISTANCE CUES
; Continuous control of 2D position
; Coded: 04/2010
; All Envelopes with poscil
; Default SR = 96000, Default Precision: 24 bits, Default Channels: 5
; 
; Panning algorithm based on: \"Continuous Surround Panning for 5-Speaker Reproduction\"
; AES Conference:24th International Conference: Multichannel Audio, The New Reality (June 2003)
; Authors:  Peter G. Craven 
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	            [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= X-position frequency-envelope 	[GEN number]
;	p11	= Y-position frequency-envelope 	[GEN number]
;	p12	= panning-function 			[GEN number]
;	--------Distance Cues-------------------------------
;	p13	= distance-attenuation function 	[GEN number]
;	p14	= air-absorption function   		[GEN number]
;	p15	= doppler-effect function  		[GEN number]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f401		doppler effect	 	(lookup table)
;____________________________________________________________________________
")
  (:icon 09)
  )

(defmethod ignore-pfields ((self pan.continuous)) '("p5" "p6" "p7"))