(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			   CLASS PAN.DISCRETE
; 	   discrete control of panning via variable panning-laws
;		     with optional distance cues
;----------------------------------------------------------------------------

(defclass! pan.discrete (prisma-panning prisma-distance-cues prisma-discrete)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "pan.discrete" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "pan.discrete")) 2) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "pan.discrete")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *panning-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)

   ( pan-fun
	:type GEN-07 ;; NB: if 0,1,2 predefined function (100, 101, 102) else tables
        :initarg :pan-fun
        :initform 0
        :accessor pan-fun)

   )

  (:documentation 
"
;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			  	PAN.DISCRETE.ORC
;      DISCRETE PANNING MODULE WITH DEFINABLE PANNING-LAWS 
;			 AND OPTIONAL DISTANCE CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: AMPLITUDE PANNING WITH DISTANCE CUES
; Discrete control of 2d-position and panning-law
; Coded: 07/2009
; All Envelopes with poscil
; Default SR = 96000, Default Precision: 24 bits, Default Channels: 2
; 
; Panning algorithm based on: PAN , 5.0 panning based on Cravens Method
; Authors: N.A.
;___________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 			[sec]
;	p3	= duration 			[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 			[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 		[linear]
;	p7	= starting point in file 	[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  			[cartesian X-coordinate]
;	p9	= Y-position  			[cartesian X-coordinate]
;	p10	= panning function		[GEN number]
;            [0=equal power, 1=square root, 2=linear]
;	--------Distance Cues-------------------------------
;	p11	= distance-attenuation function [GEN number]
;	p12	= air-absorption function   	[GEN number]
;	----------------------------------------------------
;	GEN FUNCTIONS :
;	f100,101,102	panning functions	(lookup table)
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f401		time function		(lookup table)
;____________________________________________________________________________
")
  (:icon 09)
  )

(defmethod ignore-pfields ((self pan.discrete)) '("p5" "p6" "p7"))