;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			  	PAN.DISCRETE.ORC
;      DISCRETE PANNING MODULE WITH DEFINABLE PANNING-LAWS 
;			 AND OPTIONAL DISTANCE CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: AMPLITUDE PANNING WITH DISTANCE CUES
; Discrete control of 2d-position and panning-law
; Coded: 07/2009
; All Envelopes with poscil
; Default SR = 96000, Default Precision: 24 bits, Default Channels: 2
; 
; Stereo&Quad algorithm based on: PAN, 5.0 panning based on Cravens Method
; Authors: N.A.
;___________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 			[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 			[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 	[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  			[cartesian X-coordinate]
;	p9	= Y-position  			[cartesian X-coordinate]
;	p10	= panning function		[GEN number]
;            [0=equal power, 1=square root, 2=linear]
;	--------Distance Cues-------------------------------
;	p11	= distance-attenuation function [GEN number]
;	p12	= air-absorption function   	[GEN number]
;	----------------------------------------------------
;	GEN FUNCTIONS :
;	f100,101,102	panning functions	(lookup table)
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f401			time function		(lookup table)
;____________________________________________________________________________
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 2
0dbfs 	= 8388607

instr 1 
; SOUNDFILE PLAYER *****************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope
   
;; SPATIAL SOUND RENDERER ***********

ixpos 		= p8
iypos			= p9
izpos			= p10
iattenfunction	= (p11 = 0 ? 200 : (p11 = 1 ? 201 : (p11 = 2 ? 202 : p11)))
iairfunction	= (p12 = 1 ? 301 : (p12 = 2 ? 302 : p12)) 
itimefunction	= (p13 = 1 ? 401 : p13)
irefdistance	= p14
ipanfunction 	= (p15 = 0 ? 100 : (p15 = 1 ? 101 : (p15 = 2 ? 102 : p15)))

;;--- COPY SOUND SOURCE

$pan_copy_source

;;--- Cartesian to Polar Coordinates

$xyz2aed_discrete

	index = idistance1*0.02
	
;;--- DISTANCE CUES

timeoftravel:
cggoto (itimefunction = 0), airabsorption
$timeoftravel_discrete
;asound1 timeoftravel_discrete asound1, idistance1, itimefunction

airabsorption:
cggoto (iairfunction = 0), attenuation
$airabsorption_discrete
;asound1 airabsorption_discrete asound1, idistance1, iairfunction

attenuation:
cggoto (iattenfunction = 0), panning
$attenuation_discrete
;asound1 attenuation_discrete asound1, idistance1, iattenfunction

;;--- PANNING ALGORITHM
	
panning:

cggoto (gipanmode = 1), stereo
cggoto (gipanmode = 2), quad
cggoto (gipanmode = 3), ITU

stereo:
	afrontleft, afrontright, abackleft, abackright pan asound1, ixpos, iypos, ipanfunction, 1, 1
	outc afrontleft, afrontright
goto end
	
quad:	
	afrontleft, afrontright, abackleft, abackright pan asound1, ixpos, iypos, ipanfunction, 1, 1
	outc afrontleft, afrontright, abackright, abackleft
goto end

ITU:
	iradians taninv2 iypos, ixpos
	itheta = 3.141592653589793*.5 - iradians

	i1=sin (itheta)
	i2=sin (itheta*2)
	i3=sin (itheta*3)
	i4=sin (itheta*4)
	i5=cos (itheta)
	i6=cos (itheta*2)
	i7=cos (itheta*3)
	i8=cos (itheta*4)

	aleft		= asound1 * (0.16656 + 0.24162*i5 - 0.27215*i1 - 0.05322*i6 - 0.22189*i2 - 0.08418*i7 - 0.05939*i3 - 0.06994*i8 - 0.08435*i4)
	acenter 	= asound1 * (0.10492 + 0.33223*i5 + 0.26500*i6 + 0.16902*i7 + 0.05978*i8)
	aright	= asound1 * (0.16656 + 0.24162*i5 + 0.27215*i1 - 0.05322*i6 + 0.22189*i2 - 0.08418*i7 + 0.05939*i3 - 0.06994*i8 + 0.08435*i4)
	arightsurround	= asound1 * (0.35579 - 0.35965*i5 + 0.42548*i1 - 0.06361*i6 - 0.11778*i2 + 0.00012*i7 - 0.04692*i3 + 0.02722*i8 - 0.06146*i4)	
	aleftsurround	= asound1 * (0.35579 - 0.35965*i5 - 0.42548*i1 - 0.06361*i6 + 0.11778*i2 + 0.00012*i7 + 0.04692*i3 + 0.02722*i8 + 0.06146*i4)

	outc aleft, acenter, aright, arightsurround, aleftsurround
goto end	
	
end:	
endin

