(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			CLASS RVBAP.CONTINUOUS.ORC 
;   continuous control of reverberated vector based amplitude panning in 3d
;                       with optional distance cues
;----------------------------------------------------------------------------

(defclass! rvbap.continuous (prisma-rvbap prisma-distance-cues prisma-continuous)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "rvbap.continuous" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "rvbap.continuous")) 2) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "rvbap.continuous")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *rvbap-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)
   

   ( spread-env
	:type GEN-07
        :initarg :spread-env
        :initform (make-cs-table  'Gen-07  '(0 512) '(0 0) 5 "?" 513)
        :accessor spread-env)

   ( spread-freqenv
     :type GEN-07 ;; NB: if 0 1/duration (1 cycle per event)
     :initarg :spread-freqenv 
     :initform 0
     :accessor spread-freqenv)

   ; REVERB PARAMETERS -----------------------

   ( reverb-params
	:type GEN-02 ;; NB: if 1, 2 predefined function (461, 462) else tables
        :initarg :reverb-params 
        :initform 1
        :accessor reverb-params)
   )

  (:documentation 
"
;============================================================================
;      OMprisma 2.0,  2009 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			  RVBAP.CONTINUOUS.ORC 
; 	   	 REVERBERATED VECTOR-BASE AMPLITUDE PANNING
;     		    WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: rvbap up to 16 speakers in 3d
; Continuous control of position, source spread and frequency of envelopes
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects, Reverb (FDN)
; Reverb: FDN reverb based on physical modeling scattering junction of 
; lossless waveguides. By I.Varga, 2005 based on S.Costello's csound orc.
;
; 
; Panning algorithm based on: VBAP16 / RVBAP (reverberated VBAP)
; Author: Ville Pulkki
; Sibelius Academy Computer Music Studio
; Laboratory of Acoustics and Audio Signal Processing
; Helsinki University of Technology
; Helsinki, Finland
; May 2000	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	            [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= Z-position envelope 			[GEN number]
;	p11	= spread envelope	 		[GEN number]
;	p12	= X-position frequency-envelope 	[GEN number]
;	p13	= Y-position frequency-envelope 	[GEN number]
;	p14	= Z-position frequency-envelope 	[GEN number]
;	p15	= spread frequency-envelope		[GEN number]
;	--------Distance Cues-------------------------------
;	p16	= distance-attenuation function 	[GEN number]
;	p17	= air-absorption function   		[GEN number]
;	p18	= doppler-effect function  		[GEN number]
;	--------Reverb Settings-----------------------------
;	p19	= reverb-parameters table  	[GEN number]
;	----------------------------------------------------
;	p20	= center-radius			[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f401		doppler effect	 	(lookup table)
;	f461,462	reverb parameters	(lookup table)
;____________________________________________________________________________
")
  (:icon 46)
  )

(defmethod ignore-pfields ((self rvbap.continuous)) '("p5" "p6" "p7"))
