;============================================================================
;      OMprisma 2.0,  2009 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			  	RVBAP.CONTINUOUS.ORC 
; 	   	 REVERBERATED VECTOR-BASE AMPLITUDE PANNING
;     		    WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: rvbap up to 16 speakers in 3d
; Continuous control of position, source spread and frequency of envelopes
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects, Reverb (FDN)
; Reverb: FDN reverb based on physical modeling scattering junction of 
; lossless waveguides. By I.Varga, 2005 based on S.Costello's csound orc.
;
; 
; Panning algorithm based on: VBAP16 / RVBAP (reverberated VBAP)
; Author: Ville Pulkki
; Sibelius Academy Computer Music Studio
; Laboratory of Acoustics and Audio Signal Processing
; Helsinki University of Technology
; Helsinki, Finland
; May 2000	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 					[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 				[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= Z-position envelope 			[GEN number]
;	p11	= spread envelope	 			[GEN number]
;	p12	= X-position frequency-envelope 	[GEN number]
;	p13	= Y-position frequency-envelope 	[GEN number]
;	p14	= Z-position frequency-envelope 	[GEN number]
;	p15	= spread frequency-envelope		[GEN number]
;	--------Distance Cues-------------------------------
;	p16	= distance-attenuation function 	[GEN number]
;	p17	= air-absorption function   		[GEN number]
;	p18	= doppler-effect function  		[GEN number]
;	--------Reverb Settings-----------------------------
;	p19	= reverb-parameters table  	[GEN number]
;	----------------------------------------------------
;	p20	= center-radius			[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f401		doppler effect	 	(lookup table)
;	f461,462	reverb parameters	(lookup table)
;____________________________________________________________________________
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 16
0dbfs 	= 8388607

;initialize vbap (cube)
;vbaplsinit 3, 8, -45, -45,  45, -45, 135, -45, 225, -45, -45, 45, 45, 45, 135, 45, 225, 45 

instr 1 
;; SOUNDFILE PLAYER *****************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope

; SPATIAL SOUND RENDERER *************

ixpos_envelope 		= p8
iypos_envelope 		= p9
izpos_envelope		= p10

ixpos_frequency_envelope 	= p11
iypos_frequency_envelope 	= p12
izpos_frequency_envelope 	= p13

iattenfunction	= (p14 = 0 ? 200 : (p14 = 1 ? 201 : (p14 = 2 ? 202 : p14)))
iairfunction	= (p15 = 1 ? 301 : (p15 = 2 ? 302 : p15)) 
itimefunction	= (p16 = 1 ? 401 : p16)
icentersize		= p17

ispread_envelope			= p18
ispread_frequency_envelope	= p19

ireverbfunction	= (p20 = 1 ? 461 : (p20 = 2 ? 462 : p20))

;;--- COPY SOUND SOURCE

$rvbap_copy_source

;;--- ENVELOPES
     	     	
$rvbap_envelopes

;;--- Cartesian to Polar Coordinates

$xyz2aed_continuous

;;--- DISTANCE CUES

timeoftravel:
cggoto (itimefunction = 0), airabsorption
$timeoftravel_continuous
;asound1 timeoftravel_continuous asound1, kdistance1, itimefunction

airabsorption:
cggoto (iairfunction = 0), attenuation
$airabsorption_continuous
;asound1 airabsorption_continuous asound1, kdistance1, iairfunction

attenuation:
cggoto (iattenfunction = 0), panning
$attenuation_continuous
;asound1 attenuation_continuous asound1, kdistance1, iattenfunction


panning:
;;--- REVERBERATION ALGORITHM

	$rvbap_reverb_params

	krevrolloff rvbap_rolloff_continuous kdistance1, iattenfunction
	asound2 = asound2*sqrt(krevrolloff)
	;global reverbsend
	aglrevsound = asound2*(krevrolloff*krevrolloff)
	; local reverbsend
	alclrevsound= asound2*(1-(krevrolloff*krevrolloff))	

;;--- SPREAD CONTROL

		; increase spread when entering in center-zone
		if (kdistance1 < icentersize) then
			kfactor= kdistance1/icentersize
			kspread scale kfactor, kspread, 100 ;linear scaling... might be changed
			krevspread scale kfactor, iglbrevspread, 100
		else
			krevspread = 1 * iglbrevspread
		endif

;;--- RVBAP PANNING ALGORITHM

	$rvbap_reverb_panning_continuous
	
endin

