(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			CLASS RVBAP.DISCRETE.ORC 
;     discrete control of reverberated vector-base amplitude panning
;                       with optional distance cues
;----------------------------------------------------------------------------

(defclass! rvbap.discrete (prisma-rvbap prisma-distance-cues prisma-discrete)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "rvbap.discrete" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "rvbap.discrete")) 8) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "rvbap.discrete")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *rvbap-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)
   
   ( spread
     :type number
     :initarg :spread
     :initform 0
     :accessor spread) 

   ; REVERB PARAMETERS -----------------------
   
   ( reverb-params
     :type GEN-02 ;; NB: if 1, 2 predefined function (461, 462) else tables
     :initarg :reverb-params 
     :initform 1
     :accessor reverb-params)
   )

  (:documentation 
"
;============================================================================
;      OMprisma 2.0,  2009 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			      RVBAP.DISCRETE.ORC 
; 	   	   REVERBERATED VECTOR-BASE AMPLITUDE PANNING
;     		          WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: rvbap up to 16 speakers in 3d
; Discrete control of source position, source spread and reverberation (FDN)
; Distance Cues: Attenuation, Air-absorption, Time-of-travel
; Reverb: FDN reverb based on physical modeling scattering junction of 
; lossless waveguides. By I.Varga, 2005 based on S.Costello's csound orc.
; 
; Panning algorithm based on: VBAP
; Author: Ville Pulkki
; Sibelius Academy Computer Music Studio
; Laboratory of Acoustics and Audio Signal Processing
; Helsinki University of Technology
; Helsinki, Finland
; May 2000	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  				[cartesian X-coordinate]
;	p9	= Y-position 	 			[cartesian Y-coordinate]
;	p10	= Z-position 	 			[cartesian Y-coordinate]
;	p11	= source spread		 		[percent]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function	[GEN number]
;	p13	= air-absorption function   		[GEN number]
;	p14	= time-of-travel function   		[GEN number]
;	--------Reverb Settings-----------------------------
;	p15	= reverb-parameters table		[Gen number]
;	----------------------------------------------------
;	p16	= center-radius				[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f200, 201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f401		doppler effect	 	(lookup table)
;	f461,462	reverb parameters	(lookup table
;____________________________________________________________________________
")
  (:icon 46)
  )

(defmethod ignore-pfields ((self rvbap.discrete)) '("p5" "p6" "p7"))