;============================================================================
;      OMprisma 2.0,  2009 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			         RVBAP.DISCRETE.ORC 
; 	   	   REVERBERATED VECTOR-BASE AMPLITUDE PANNING
;     		     WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: rvbap up to 16 speakers in 3d
; Discrete control of source position, source spread and reverberation (FDN)
; Distance Cues: Attenuation, Air-absorption, Time-of-travel
; Reverb: FDN reverb based on physical modeling scattering junction of 
; lossless waveguides. By I.Varga, 2005 based on S.Costello's csound orc.
; 
; Panning algorithm based on: VBAP
; Author: Ville Pulkki
; Sibelius Academy Computer Music Studio
; Laboratory of Acoustics and Audio Signal Processing
; Helsinki University of Technology
; Helsinki, Finland
; May 2000	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 					[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 				[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  				[cartesian X-coordinate]
;	p9	= Y-position 	 			[cartesian Y-coordinate]
;	p10	= Z-position 	 			[cartesian Y-coordinate]
;	p11	= source spread		 		[percent]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function	[GEN number]
;	p13	= air-absorption function   		[GEN number]
;	p14	= time-of-travel function   		[GEN number]
;	--------Reverb Settings-----------------------------
;	p15	= reverb-parameters table		[Gen number]
;	----------------------------------------------------
;	p16	= center-radius				[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f200, 201,202	distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f401			doppler effect	 	(lookup table)
;	f461,462		reverb parameters		(lookup table
;____________________________________________________________________________
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 16
0dbfs 	= 8388607 ; 24 bits

;initialize vbap (cube)
;vbaplsinit 3, 8, -45, -45,  45, -45, 135, -45, 225, -45, -45, 45, 45, 45, 135, 45, 225, 45 

instr 1 
;; SOUNDFILE PLAYER ******************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope
   
; SPATIAL SOUND RENDERER **************

ixpos 		= p8
iypos			= p9
izpos			= p10

iattenfunction	= (p11 = 0 ? 200 : (p11 = 1 ? 201 : (p11 = 2 ? 202 : p11)))
iairfunction	= (p12 = 1 ? 301 : (p12 = 2 ? 302 : p12)) 
itimefunction	= (p13 = 1 ? 401 : p13)
icentersize		= p14

ispread		limit p15, 0, 100
ireverbfunction	= (p16 = 1 ? 461 : (p16 = 2 ? 462 : p16))


;;--- COPY SOUND SOURCE

$rvbap_copy_source

;;--- Cartesian to Polar Coordinates

$xyz2aed_discrete

;;--- DISTANCE CUES

timeoftravel:
cggoto (itimefunction = 0), airabsorption
$timeoftravel_discrete
;asound1 timeoftravel_discrete asound1, kdistance1, itimefunction

airabsorption:
cggoto (iairfunction = 0), attenuation
$airabsorption_discrete
;asound1 airabsorption_discrete asound1, idistance1, iairfunction

attenuation:
cggoto (iattenfunction = 0), panning
$attenuation_discrete
;asound1 attenuation_discrete asound1, idistance1, iattenfunction


panning:
;;--- REVERBERATION ALGORITHM

	$rvbap_reverb_params

	irevrolloff rvbap_rolloff_discrete idistance1, iattenfunction
	asound2 = asound2*sqrt(irevrolloff) ;*iglbrevlevel this is included in the reverberation
	;global reverbsend
	aglrevsound = asound2*(irevrolloff*irevrolloff)
	;local reverbsend
	alclrevsound= asound2*(1-(irevrolloff*irevrolloff))	

;;--- SPREAD CONTROL ;-> should maybe be a macro, too
	
	if (idistance1 < icentersize) then
		ifactor= idistance1/icentersize
		kspread scale ifactor, ispread, 100 		;linear scaling... might be changed
		krevspread scale ifactor, iglbrevspread, 100
	else
		kspread 	= ispread
		krevspread 	= iglbrevspread
	endif

;;--- RVBAP PANNING ALGORITHM

	$rvbap_reverb_panning_discrete
	
endin
