(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			  CLASS SPAT.CONTINUOUS
;      	   continuous control of 1st-order ambisonic spatialization
;                       with optional distance cues
;                          and shoebox-roommodel
;----------------------------------------------------------------------------

(defclass! spat.continuous (prisma-spat prisma-distance-cues prisma-continuous)
		
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "spat.continuous" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "spat.continuous")) 16) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "spat.continuous")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *ambi-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)
   
   ( order-env
     :type GEN-07
     :initarg :order-env
     :initform (make-cs-table  'Gen-07  '(0 512) '(3 3) 5 "?" 513)
     :accessor order-env)

   ( order-freqenv
     :type GEN-07 ;; NB: if 0 1/duration (1 cycle per event)
     :initarg :order-freqenv 
     :initform 0
     :accessor order-freqenv)


   ; ROOM MODEL -----------------------

   ( room-params
	:type cs-table ;; NB: if 1 predefined function (451) else tables
        :initarg :room-params 
        :initform 0
        :accessor room-params)

   )

  (:documentation 
"
;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   SPAT.CONTINUOUS.ORC 
; 	   	3RD ORDER AMBISONIC SPATIAL SOUND RENDERER
;     		           WITH DISTANCE-CUES
;		    AND 1ST ORDER SHOEBOX ROOMMODEL
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: 3rd-order ambisonics
; Continuous control of position, encoding order and frequency of envelopes
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Room Model: Shoebox (simulation of room-acoustics)
; 
; Panning algorithm based on: SPAT3D
; Author: Istvan Varga 2001
;___________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= Z-position envelope 			[GEN number]
;	p11	= ambisonic order envelope 		[GEN number]
;	p12	= X-position frequency-envelope 	[GEN number]
;	p13	= Y-position frequency-envelope 	[GEN number]
;	p14	= Z-position frequency-envelope 	[GEN number]
;	p15	= order frequency-envelope		[GEN number]
;	--------Distance Cues-------------------------------
;	p16	= distance-attenuation function	[GEN number]
;	p17	= air-absorption function   		[GEN number]
;	p18	= room-parameters table  		[GEN number]
;	----------------------------------------------------
;	p19	= center-radius				[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202		distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;	f451			room parameters	 	(lookup table)
;____________________________________________________________________________
")
  (:icon 37)
  )

(defmethod ignore-pfields ((self spat.continuous)) '("p5" "p6" "p7"))
