(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			   CLASS SPAT.DISCRETE
;      	   discrete control of 3rd-order ambisonic spatialization
;                       with optional distance cues
;                and 1st-order ambisonics shoebox-roommodel
;----------------------------------------------------------------------------

(defclass! spat.discrete (prisma-spat prisma-discrete)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "spat.discrete" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "spat.discrete")) 16) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "spat.discrete")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *ambi-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)
 
   ( order
	:type number
        :initarg :order
        :initform 3
        :accessor order)

   ; ROOM MODEL -----------------------

   ( room-params
	:type cs-table ;; NB: if 1 predefined function (451) else tables
        :initarg :room-params 
        :initform 0
        :accessor room-params)

   )

  (:documentation 
"
;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			    SPAT.DISCRETE.ORC 
; 	   	3RD ORDER AMBISONIC SPATIAL SOUND RENDERER
;     		             WITH DISTANCE-CUES
;		      AND 1ST ORDER SHOEBOX ROOMMODEL
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: 3rd-order ambisonics, 3D
; Discrete control of position, encoding order and room-model
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Room Model: Shoebox (simulation of room-acoustics)
;
; Coded: 07/2009
; All Envelopes with poscil
; Default SR = 96000, Default Precision: 24 bits, Default Channels: 16
; 
; Panning algorithm based on: SPAT3D
; Author: Istvan Varga 2001
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 			[sec]
;	p3	= duration 			[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 			[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 		[linear]
;	p7	= starting point in file 	[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  			[cartesian X-coordinate]
;	p9	= Y-position 	 		[cartesian Y-coordinate]
;	p10	= Z-position 	 		[cartesian Y-coordinate]
;	p11	= ambisonic order	 	[float]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function [GEN number]
;	p13	= air-absorption function   	[GEN number]
;	p14	= room-parameters table         [GEN number]
;	----------------------------------------------------
;	p15	= center-radius			[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f451		room parameters	 	(lookup table)
;____________________________________________________________________________
")
  (:icon 37)
  )

(defmethod ignore-pfields ((self spat.discrete)) '("p5" "p6" "p7"))