;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			      SPAT.DISCRETE.ORC 
; 	   	3RD ORDER AMBISONIC SPATIAL SOUND RENDERER
;     		      WITH DISTANCE-CUES
;		      AND 1ST ORDER SHOEBOX ROOMMODEL
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: 3rd-order ambisonics, 3D
; Discrete control of position, encoding order and room-model
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Room Model: Shoebox (simulation of room-acoustics)
;
; Coded: 07/2009
; All Envelopes with poscil
; Default SR = 96000, Default Precision: 24 bits, Default Channels: 16
; 
; Panning algorithm based on: SPAT3D
; Author: Istvan Varga 2001
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 			[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 			[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 	[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  			[cartesian X-coordinate]
;	p9	= Y-position 	 		[cartesian Y-coordinate]
;	p10	= Z-position 	 		[cartesian Y-coordinate]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function [GEN number]
;	p13	= air-absorption function   	   [GEN number]
;	p14	= time-function
;	p15	= reference-distance			[float]
;	----------------------------------------------------
;	p16	= room parameters
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f451		room parameters	 	(lookup table)
;____________________________________________________________________________
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 16
0dbfs 	= 8388607

instr 1 
;; SOUNDFILE PLAYER **********************************************************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope
   
;; SPATIAL SOUND RENDERER *****************************************************

ixpos 		= p8
iypos			= p9
izpos			= p10

iattenfunction	= (p11 = 0 ? 200 : (p11 = 1 ? 201 : (p11 = 2 ? 202 : p11)))
iairfunction	= (p12 = 1 ? 301 : (p12 = 2 ? 302 : p12)) 
;itimefunction	= p13
icentersize		= p14

iorder		limit p15, 0, 3
ispatfunction	= (p16 = 1 ? 451 : p16)

;-----------------------------------------
	denorm asound
;-----------------------------------------
;;--- COPY SOUND SOURCE

$spat_copy_source

;;--- Cartesian to Polar Coordinates

$xyz2aed_discrete

	index = idistance1*0.0

;;--- PANNING ALGORITHM

panning:

	
	; Order compensation inside center-zone 
	iorder = (idistance1 > icentersize ? iorder : iorder*idistance1*(1/icentersize))	
	iordg1 limit iorder, 0, 1
	;prints "iordg1 = %f, iordg2 = %f, iordg3 = %f\\n", iordg1, iordg2, iordg3

cggoto (gidims = 3), with_height
cggoto (giorder = 1), first_order_2d
cggoto (giorder = 0), uhj
with_height:
cggoto (giorder = 1), first_order_3d


first_order_2d:
	aw, ax, ay, az  spat3d asound1, ixpos, iypos, izpos, icentersize, ispatfunction, 2, 8, 8	
     	outc aw, ax*iordg1, ay*iordg1
goto end

first_order_3d:
        aw, ax, ay, az  spat3d asound1, ixpos, iypos, izpos, icentersize, ispatfunction, 3, 8, 8	
        outc aw, ax*iordg1, ay*iordg1, az*iordg1
goto end

uhj:
      aw, ax, ay, az  spat3d asound1, ixpos, iypos, izpos, icentersize, ispatfunction, 3, 8, 8   
	aWreal, aWimaginary	hilbert aw
	aXreal, aXimaginary	hilbert ax
	aYreal, aYimaginary	hilbert ay
	aWXreal		=  0.0928*aXreal + 0.4699*aWreal
	aWXimaginary	=  0.2550*aXimaginary - 0.1710*aWimaginary
	aLeft			=  aWXreal + aWXimaginary + 0.3277*aYreal
	aRight		=  aWXreal - aWXimaginary - 0.3277*aYreal
	outc aLeft, aRight
goto end 

end:
endin
