(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			 CLASS VBAP.CONTINUOUS
;      	  continuous control of vector based amplitude panning
;                       with optional distance cues
;----------------------------------------------------------------------------

(defclass! vbap.continuous (prisma-vbap prisma-distance-cues prisma-continuous)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "vbap.continuous" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "vbap.continuous")) 2) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "vbap.continuous")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *vbap-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)
   
   ( spread-env
	:type GEN-07
        :initarg :spread-env
        :initform (make-cs-table  'Gen-07  '(0 512) '(0 0) 5 "?" 513)
        :accessor spread-env)

   ( spread-freqenv
     :type GEN-07 ;; NB: if 0 1/duration (1 cycle per event)
     :initarg :spread-freqenv 
     :initform 0
     :accessor spread-freqenv)

   )

  (:documentation 
"
;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   	VBAP.CONTINUOUS.ORC 
; 	   	           VECTOR-BASE AMPLITUDE PANNING
;     		           WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: vbap up to 16 speakers
; Continuous control of position and source spread
; Distance Cues: Attenuation, Air-absorption, Doppler-Effects
; Coded: 10/2010
; 
; Panning algorithm based on: VBAP16
; Author: Ville Pulkki
; Sibelius Academy Computer Music Studio
; Laboratory of Acoustics and Audio Signal Processing
; Helsinki University of Technology
; Helsinki, Finland
; May 2000	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position envelope 			[GEN number]
;	p9	= Y-position envelope 			[GEN number]
;	p10	= Z-position envelope 			[GEN number]
;	p11	= spread envelope	 		[GEN number]
;	p12	= X-position frequency-envelope 	[GEN number]
;	p13	= Y-position frequency-envelope 	[GEN number]
;	p14	= Z-position frequency-envelope 	[GEN number]
;	p15	= spread frequency-envelope		[GEN number]
;	--------Distance Cues-------------------------------
;	p16	= distance-attenuation function 	[GEN number]
;	p17	= air-absorption function   		[GEN number]
;	p18	= doppler-effect function  		[GEN number]
;	----------------------------------------------------
;	p19	= center-radius				[float]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;	f401		doppler effect	 	(lookup table)
;____________________________________________________________________________
")
  (:icon 15)
  )

(defmethod ignore-pfields ((self vbap.continuous)) '("p5" "p6" "p7"))
