(in-package :om)

;============================================================================
;     OMprisma 2.0,  2010 Marlon Schumacher (CIRMMT / McGill University)
;============================================================================
;----------------------------------------------------------------------------
;			 CLASS VBAP.DISCRETE
;      	  discrete control of vector based amplitude panning in 2d
;                       with optional distance cues
;----------------------------------------------------------------------------

(defclass! vbap.discrete (prisma-vbap prisma-distance-cues prisma-discrete)
   			
  (
   (source-code :initform
                 (load-buffer-textfile
                  (get-orc-source (get-orc "vbap.discrete" :local))
                  'textfile "append")
                 :allocation :class :type textfile :accessor source-code)
   (numchan :initform (or (get-orc-channels (get-orc "vbap.discrete")) 8) :allocation :class  :accessor numchan)
   (cs-inits :initform (get-cs-inits (get-orc "vbap.discrete")) :allocation :class :type list :accessor cs-inits)
   (orc-header :initform *vbap-default-tables* :allocation :class :type list :accessor orc-header)
   (InstID :initform 1  :allocation :class  :accessor InstID)

   ( spread
	:type number
        :initarg :spread
        :initform 0
        :accessor spread)
   )

  (:documentation 
"
;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   	VBAP.DISCRETE.ORC 
; 	   	           VECTOR-BASE AMPLITUDE PANNING
;     		           WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: vbap up to 16 speakers in 3d
; Discrete control of source position and source spread
; Distance Cues: Attenuation, Air-absorption
; Coded: 10/2010
; 
; Panning algorithm based on: VBAP16
; Author: Ville Pulkki
; Sibelius Academy Computer Music Studio
; Laboratory of Acoustics and Audio Signal Processing
; Helsinki University of Technology
; Helsinki, Finland
; May 2000	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 				[sec]
;	******* Soundfile Player ***************************	
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 			[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  				[cartesian X-coordinate]
;	p9	= Y-position 	 			[cartesian Y-coordinate]
;	p10	= Z-position 	 			[cartesian Z-coordinate]
;	p11	= source spread		 		[percent]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function 	[GEN number]
;	p13	= air-absorption function   		[GEN number]
;	p14	= time-of-travel function   		[GEN number]
;	----------------------------------------------------
;	p15	= center-radius				[linear]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302	air absorption 		(lookup table)
;____________________________________________________________________________
")
  (:icon 15)
  )

(defmethod ignore-pfields ((self vbap.discrete)) '("p5" "p6" "p7"))