;============================================================================
;      OMprisma 2.0, 2010 Marlon Schumacher (CIRMMT / McGill University)
;----------------------------------------------------------------------------
;    Library for Spatial Sound Synthesis and Processing with OpenMusic
;============================================================================
;			   		VBAP.DISCRETE.ORC 
; 	   	           VECTOR-BASE AMPLITUDE PANNING
;     		       WITH OPTIONAL DISTANCE-CUES
;----------------------------------------------------------------------------
; Spatial Sound Rendering Concept: vbap up to 16 speakers in 3d
; Discrete control of source position and source spread
; Distance Cues: Attenuation, Air-absorption
; Coded: 10/2010
; 
; Panning algorithm based on: VBAP16
; Author: Ville Pulkki
; Sibelius Academy Computer Music Studio
; Laboratory of Acoustics and Audio Signal Processing
; Helsinki University of Technology
; Helsinki, Finland
; May 2000	
;____________________________________________________________________________
;	p-fields:
;	p1	= instrument number
;	p2	= action time 				[sec]
;	p3	= duration 					[sec]
;	******* Soundfile Player ***************************	
;	p4 	= soundfile 				[filename]
;	p5	= gain	    [if > 0 linear, if <= 0 in dB]
;	p6	= gain-envelope 				[linear]
;	p7	= starting point in file 		[sec]
;	******* Spatial Sound Renderer *********************
;	p8	= X-position  				[cartesian X-coordinate]
;	p9	= Y-position 	 			[cartesian Y-coordinate]
;	p10	= Z-position 	 			[cartesian Z-coordinate]
;	--------Distance Cues-------------------------------
;	p12	= distance-attenuation function 	[GEN number]
;	p13	= air-absorption function   		[GEN number]
;	p14	= time-of-travel function   		[GEN number]
;	p15	= reference-distance			[linear]
;	----------------------------------------------------
;	p16	= source spread		 		[percent]
;----------------------------------------------------------------------------
; 	GEN FUNCTIONS :
;	f200,201,202	distance attenuation 	(lookup table) 
;	f301,302		air absorption 		(lookup table)
;____________________________________________________________________________
; HEADER ********************************************************************

sr		= 96000
kr		= 96000
ksmps		= 1
nchnls	= 16
0dbfs 	= 8388607 ; 24 bits

instr 1 
;; SOUNDFILE PLAYER **************

idur			= p3
ifilename		= p4
igain			= (p5 > 0.0 ? (p5) : (10^(p5*0.05)))
igainenvelope	= p6
istartposition	= p7
ioscfrequency	= 1/idur

	asoundfile		diskin2	ifilename, 1, istartposition, 0, 0, 512, 262144, 0
	kgainenvelope 	poscil  igain, ioscfrequency, igainenvelope
	asound 		= asoundfile * kgainenvelope
   
;; SPATIAL SOUND RENDERER *********

ixpos 		= p8
iypos			= p9
izpos			= p10
iattenfunction	= (p11 = 0 ? 200 : (p11 = 1 ? 201 : (p11 = 2 ? 202 : p11)))
iairfunction	= (p12 = 1 ? 301 : (p12 = 2 ? 302 : p12)) 
itimefunction	= (p13 = 1 ? 401 : p13)
icentersize		= p14
ispread		limit p15, 0, 100

; maybe I should have two things: reference distance and center-radius -> does that make sense?

;;--- COPY SOUND SOURCE

$vbap_copy_source

;;--- Cartesian to Polar Coordinates

$xyz2aed_discrete

	index = idistance1*0.02
	
;;--- DISTANCE CUES

timeoftravel:
cggoto (itimefunction = 0), airabsorption
$timeoftravel_discrete
;asound1 timeoftravel_discrete asound1, kdistance1, itimefunction

airabsorption:
cggoto (iairfunction = 0), attenuation
$airabsorption_discrete
;asound1 airabsorption_discrete asound1, idistance1, iairfunction

attenuation:
cggoto (iattenfunction = 0), panning
$attenuation_discrete
;asound1 attenuation_discrete asound1, idistance1, iattenfunction

panning:
;;--- PANNING ALGORITHM
	; spread-control (increase spread of soundsource within center-zone)
	if (idistance1 < icentersize) then
		ifactor= idistance1/icentersize
		kspread scale ifactor, ispread, 100 	;linear scaling... might be changed
	else
		kspread = 1 * ispread
	endif
	;printk 0.1, kspread

ch4:
cggoto (nchnls > 4), ch8
	a1, a2, a3, a4 vbap4 asound, iazimuth1, ielevation1, kspread
	outc a1, a2, a3, a4
goto end

ch8:
cggoto (nchnls > 8), ch16
	a1, a2, a3, a4, a5, a6, a7, a8 vbap8 asound, iazimuth1, ielevation1, kspread
	outc a1, a2, a3, a4, a5, a6, a7, a8
goto end

ch16:
	a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16 vbap16 asound, iazimuth1, ielevation1, kspread
	outc a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16
goto end	
	
end:
endin
