;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)

;;; GEN-FUN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

(defmethod! gen-table (function table-id)
            :icon 605  
            :initvals '(sqrt "?")
            :numouts 1
            :doc "samples a function into a GEN-07 table"
            :indoc '("function in lambda mode" "table-id")
            :outdoc '("GEN-07")

            (let ((my-gen (simple-bpf-from-list (arithm-ser 0 50 0.05)
                                                (samplefun function 0.05 0 50 1)
                                                'gen-07 16)))
              (setf (id my-gen) table-id)
              (setf (size my-gen) 32769)
              my-gen))

;; for compatibility
(defmethod! make-gen (function table-id)
            (gen-table function table-id))

;;; FUNCTIONS FOR AIR-ABSORPTION %%%%%%%%%%

;---FROM: Valente, D. "A Hybrid Approach to Rendering Multi-Channel Audio Using Wave Field Synthesis and Stereophonic Techniques, with virtual Microphone control (ViMiC)"

(defmethod! air-valente ((distance number) (offset number) &key (ref-distance 1))
            :icon '(236)  
            :initvals '(0 1)
            :indoc '("distance" "offset (in Hz)" "ref-distance")
            :numouts 1
            :doc "3rd order polynomial best-fit air-absorption function"
   (let ((reldistance (/ distance ref-distance)))
         (if (om<= reldistance 1) (om-scale reldistance 20048 (+ 15849 offset) 0 1)
(clip (+ offset (+ 804 (+ 15849 (- (+ (* 18.919 (* reldistance reldistance)) (* -0.1668 (* reldistance reldistance reldistance))) (* 785.71 reldistance))))) 300 20048)
)))

(defmethod! air-valente ((distance list) (offset list) &key (ref-distance 1))
      (mapcar #'(lambda (d o) (air-valente d o ref-distance)) 
                    distance offset))

(defmethod! air-valente ((distance list) (offset number) &key (ref-distance 1))
   (air-valente distance (make-list (length distance) :initial-element offset) ref-distance))

;---FROM: Schacher J., Kocher, P. "Ambisonics Spatialization Tools for Max/MSP", ICMC2006

(defmethod! air-icst ((distance number) (cutoff number) &key (exponent 1) (ref-distance 1))
            :icon '(236)  
            :initvals '(0 500 1 1)
            :indoc '("distance" "cutoff" "exponent" "ref-distance")
            :numouts 1
            :doc "arbitrary hi-frequency cutoff as a function of distance"
   (let* (
         (reldistance (/ distance ref-distance))
         )     
      (if (om<= reldistance 1) 
          20048 
          (clip (+ cutoff (- 20048 (* (expt reldistance exponent) cutoff))) 300 20048)
      )))

(defmethod! air-icst ((distance list) (cutoff list) &key (exponent 1) (ref-distance 1))
   (mapcar #'(lambda (d cf) (air-icst d cf exponent ref-distance)) 
                    distance cutoff))

(defmethod! air-icst ((distance list) (cutoff number) &key (exponent 1) (ref-distance 1))
   (air-icst distance (make-list (length distance) :initial-element cutoff)))

(defmethod! air-icst ((distance number) (cutoff list) &key (exponent 1) (ref-distance 1))
   (air-icst (make-list (length cutoff) :initial-element distance)))

;---R.Furse---linear frequency rolloff as function of distance

(defmethod! air-furse ((distance number) (exponent number) &key (ref-distance 1))
            :icon '(236)  
            :initvals '(0 1 1)
            :indoc '("distance" "exponent" "ref-distance")
            :numouts 1
            :doc "air-absorption function by R.Furse"
            (let* ((reldistance (/ distance ref-distance)))     
              (if (om<= reldistance 1) 
                  20048
                (clip (* 100000 (expt reldistance (* -1 exponent))) 300 20048)
                )))

(defmethod! air-furse ((distance list) (exponent list) &key (ref-distance 1))
            (mapcar #'(lambda (d ex) (air-furse d ex ref-distance)) 
                    distance exponent))

(defmethod! air-furse ((distance list) (exponent number) &key (ref-distance 1))
   (air-furse distance (make-list (length distance) :initial-element exponent) ref-distance))


;;; FUNCTIONS FOR DISTANCE-ATTENUATION %%%%%%%%%%%
;------------------------------------
;---Inverse Proportional---

(defmethod! atten-invprop ((distance number) (roll-off number) &key (center-atten 6) (curve .5) (ref-distance 1))
            :icon '(236)   
            :initvals '(0 6 6 0.5 1)
            :indoc '("distance" "roll-off (db/dbl-distance)")
            :numouts 1
            :doc "inverse proportional distance-attenuation (db per doubling of distance)"

            (let ((reldistance (/ distance ref-distance))
                  (attenlin (db->lin center-atten))
                  (curve (clip curve 0. 10.)))
              (if (om<= reldistance 1) 
                  (lin->db (/ 1 (+ attenlin (* (- 1 attenlin) (expt reldistance curve)))))
                (lin->db (expt reldistance (om-scale roll-off 0 -1 0 6))))
              ))

(defmethod! atten-invprop ((distance list) (roll-off list) &key (center-atten 6) (curve .5) (ref-distance 1))
   (mapcar #'(lambda (d r) (atten-invprop d r center-atten curve ref-distance)) 
                    distance roll-off))

(defmethod! atten-invprop ((distance list) (roll-off number) &key (center-atten 6) (curve .5) (ref-distance 1))
   (atten-invprop distance (make-list (length distance) :initial-element roll-off) center-atten curve ref-distance))

(defmethod! atten-invprop ((distance number) (roll-off list) &key (center-atten 6) (curve .5) (ref-distance 1))
   (atten-invprop (make-list (length roll-off) :initial-element distance) center-atten curve ref-distance))


;------------------------------------
;---Exponential ---
(defmethod! atten-exp ((distance number) (roll-off number) &key (center-atten 6) (curve .5) (ref-distance 1))
            :icon '(236)   
            :initvals '(0 6 6 0.5 1)
            :indoc '("distance" "roll-off (db/m)" "center-attenuation" "curve" "reference-distance")
            :numouts 1
            :doc "exponential distance-attenuation"
   (let ((reldistance (/ distance ref-distance))
         (attenlin (db->lin center-atten))
         (curve (clip curve 0. 10.)))
     (if (om<= reldistance 1) 
         (lin->db (/ 1 (+ attenlin (* (- 1 attenlin) (expt reldistance curve)))))
       (+ roll-off (* reldistance (* -1 roll-off)))
       )))

(defmethod! atten-exp ((distance list) (roll-off list) &key (center-atten 6) (curve .5) (ref-distance 1))
            (mapcar #'(lambda (d r) (atten-exp d r center-atten curve ref-distance)) 
                    distance roll-off))

(defmethod! atten-exp ((distance list) (roll-off number) &key (center-atten 6) (curve .5) (ref-distance 1))
   (atten-exp distance (make-list (length distance) :initial-element roll-off) center-atten curve ref-distance))

(defmethod! atten-exp ((distance number) (roll-off list) &key (center-atten 6) (curve .5) (ref-distance 1))
   (atten-exp (make-list (length roll-off) :initial-element distance) center-atten curve ref-distance))

; --------------
; Doppler-Effect

(defmethod! time-of-flight ((distance number) (scalar number) (exponent number) &key (ref-distance 1))
            :icon '(236)  
            :initvals '(0 1 1 1)
            :indoc '("distance" "scalar" "exponent" "ref-distance")
            :numouts 1
            :doc "doppler function"
            (let* ((reldistance (/ distance ref-distance)))
              (* (expt (/ reldistance 50) exponent) (* scalar 0.1470588))
              ))

; for compatibility

(defmethod! doppler ((distance number) (scalar number) (exponent number) &key (ref-distance 1))
            (time-delay distance scalar exponent :ref-distance ref-distance))

(defmethod! time-delay ((distance number) (scalar number) (exponent number) &key (ref-distance 1))
            (time-delay distance scalar exponent :ref-distance ref-distance))