;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)


(defun cs-nchnls (nchnls)
  (make-instance 'cs-init :init-type :channels :name "nchnls" :Params '() :value nchnls)
  )

(defun cs-globals (globals)
  (loop for global in globals collect
        (make-instance 'cs-init :init-type :global :name (first global) :value (second global))
        ))

(defun cs-opcodes (opcodes)
  (loop for opcode in opcodes collect
        (make-instance 'cs-init :init-type :opcode :name (first opcode) :Params (second opcode) :value (third opcode))
        ))

(defun cs-macros (macros)
  (loop for macro in macros collect
        (make-instance 'cs-init :init-type :macro :name (first macro) :Params () :value (second macro))
        ))


;%%%%%%%%%% User-defined Opcodes for OMPrisma .orcs %%%%%%%%

(defparameter *envelope*
  (list
   "envelope"
   '("k" "ii")
   "ifunction, ifreqfunction xin
	if ifreqfunction != 0 then	
		kfreqindex	phasor 1/p3
		kfrequency	tablei kfreqindex, ifreqfunction, 1, 0, 0
		else
		kfrequency = 1/p3
	endif
	kvalue poscil 1, kfrequency, ifunction
	xout kvalue"))

(defparameter *trajectory*
  (list
   "trajectory"
   '("k" "ik")
   "ifunction, kfrequency xin
	kvalue poscil 1, kfrequency, ifunction
	xout kvalue"))

;-------- DISCRETE ----------

(defparameter *xyz2aed-discrete*
  (list 
   "xyz2aed_discrete" 
   '("iii" "iii")
   "ixpos, iypos, izpos xin
	idistancexy = (ixpos^2+iypos^2)^0.5			;distance (2D)
	iradiansxy taninv2 iypos, ixpos 		;azimuth
	iazimuth = 90-(iradiansxy*57.29577951308)	;radians to degree
	idistancexyz = (idistancexy^2+izpos^2)^0.5	;distance (3D)
	iradiansxyz taninv2 izpos, idistancexy		;elevation
	ielevation = iradiansxyz*57.29577951308	;radians to degree
	xout iazimuth, ielevation, idistancexyz"))

(defparameter *distance-discrete* 
  (list 
   "distance_discrete" 
   '("i" "iiiiii")
   "ixpos, iypos, izpos, ispkxpos, ispkypos, ispkzpos xin
	idist = ((ixpos-ispkxpos)^2+(iypos-ispkypos)^2+(izpos-ispkzpos)^2)^0.5
	xout idist"))

(defparameter *attenuation-discrete* 
  (list 
   "attenuation_discrete" 
   '("a" "aii")
   "asound, idist, iattenfunction xin
	index = idist*0.02
	irolloff  tablei  index, iattenfunction, 1, 0, 0
	igain limit (10^(irolloff*0.05)), 0, 1
	asound = asound * igain
	xout asound"))

(defparameter *airabsorption-discrete* 
  (list 
   "airabsorption_discrete" 
   '("a" "aii")
   "asound, idist, iairfunction xin
	index = idist*0.02
	icutoff  tablei  index, iairfunction, 1, 0, 0
	aspkfeed butterlp asound, icutoff		
	xout aspkfeed"))

(defparameter *timeoftravel-discrete* 
(list 
 "timeoftravel_discrete" 
 '("a" "aii")
 "asound, idist, idopplerfunction xin
  	idelaytime  = idist*2.9411764705882353	
  	idelayindex = idelaytime*0.0068
  	idelaytime  tablei idelayindex, idopplerfunction, 1, 0, 0
	adelaytime  limit idelaytime, 128/sr, 0.3
	aspkfeed vdelayxw asound, adelaytime, 0.3, 256 
	xout aspkfeed"))


;-------- CONTINUOUS ----------

(defparameter *xyz2aed-continuous*
  (list 
   "xyz2aed_continuous" 
   '("kkk" "kkk")
   "kxpos, kypos, kzpos xin
	kdistancexy = (kxpos^2+kypos^2)^0.5		;distance (2D)
	kradiansxy taninv2 kypos, kxpos 			;azimuth
	kazimuth = 90-(kradiansxy*57.29577951308)	;radians to degree 
	kdistancexyz = (kdistancexy^2+kzpos^2)^0.5	;distance (3D)  
	kradiansxyz taninv2 kzpos, kdistancexy		;elevation
	kelevation = kradiansxyz*57.29577951308	;radians to degree
	xout kazimuth, kelevation, kdistancexyz"))

(defparameter *distance-continuous* 
  (list 
   "distance_continuous" 
   '("k" "kkkiii")
   "kxpos, kypos, kzpos, ispkxpos, ispkypos, ispkzpos xin
	kdist = ((kxpos-ispkxpos)^2+(kypos-ispkypos)^2+(kzpos-ispkzpos)^2)^0.5
	xout kdist"))

(defparameter *attenuation-continuous* 
  (list 
   "attenuation_continuous" 
   '("a" "aki")
   "asound, kdist, iattenfunction xin
	kindex = kdist*0.02
	krolloff  tablei  kindex, iattenfunction, 1, 0, 0
	kgain limit (10^(krolloff*0.05)), 0, 1
	asound = asound * kgain
	xout asound"))

(defparameter *airabsorption-continuous* 
  (list 
   "airabsorption_continuous" 
   '("a" "aki")
   "asound, kdist, iairfunction xin
	kindex = kdist*0.02
	kcutoff tablei kindex, iairfunction, 1, 0, 0
	aspkfeed butterlp asound, kcutoff		
	xout aspkfeed"))

(defparameter *timeoftravel-continuous* 
  (list 
   "timeoftravel_continuous" 
   '("a" "aki")
   "asound, kdist, idopplerfunction xin
  	kdelaytime = kdist*2.9411764705882353	
  	kdelayindex = kdelaytime*0.0068
  	kdelaytime tablei kdelayindex, idopplerfunction, 1, 0, 0
	adelaytime limit kdelaytime, 128/sr, 0.3
	aspkfeed vdelayxw asound, adelaytime, 0.3, 256 
	xout aspkfeed"))


;%%%%%%%%%%%%%%%%%% Csound Macros for OMPrisma .orcs %%%%%%%%%%%%%

; maybe change all the (speakers) arguments to sth more generic like (positions)

(defun copy-source (speakers classname)
  (let ((thestring))
        (loop for item in speakers 
              for i = 1 then (+ i 1) do 
              (setf thestring (concatenate 'string thestring 
                                           (format nil "~1%asound~D = asound" i i i))))
      (list (format nil "~a_copy_source" classname) thestring)
      ))

; new 'class-specific' macros

(defun csound-envelopes (parameter-list classname)
  (let ((thestring))
    (loop for item in parameter-list do
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%k~a envelope i~a_envelope, i~a_frequency_envelope" item item item))))
    (list (format nil "~a_envelopes" classname) thestring)
      ))

(defun csound-trajectories (parameter-list classname)
  (let ((thestring))
    (loop for item in parameter-list do
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%k~a trajectory i~a_envelope, ktrajfrequency" item item))))
    (list (format nil "~a_trajectories" classname) thestring)
      ))

; ------------ DISCRETE ----------------------

; new class-specific macros

(defun xyz2aed-discrete (parameter-list)
  (let ((thestring))
    (loop for item in parameter-list
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%iazimuth~D, ielevation~D, idistance~D xyz2aed_discrete ixpos, iypos, izpos" i i i))))
    (list "xyz2aed_discrete" thestring)
    ))

(defun distance-discrete (speakers)
  (let ((thestring))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                     (format nil "~1%idistance~D distance_discrete ixpos, iypos, izpos, gisinkposx~D, gisinkposy~D, gisinkposz~D" i i i i))))
    (list "distance_discrete" thestring)
    ))

(defun attenuation-discrete (speakers)
  (let ((thestring))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf thestring (concatenate 'string thestring 
                                    (format nil "~1%asound~D attenuation_discrete asound~D, idistance~D, iattenfunction" i i i))))
  (list "attenuation_discrete" thestring)
  ))

(defun airabsorption-discrete (speakers)
  (let ((thestring))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%asound~D airabsorption_discrete asound~D, idistance~D, iairfunction" i i i))))
    (list "airabsorption_discrete" thestring)
    ))

(defun timeoftravel-discrete (speakers)
  (let ((thestring))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%asound~D timeoftravel_discrete asound~D, idistance~D, itimefunction" i i i))))
    (list "timeoftravel_discrete" thestring)
    ))

; ------------ CONTINUOUS ----------------------

(defun xyz2aed-continuous (list)
  (let ((thestring))
    (loop for item in list
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%kazimuth~D, kelevation~D, kdistance~D xyz2aed_continuous kxpos, kypos, kzpos" i i i))))
    (list "xyz2aed_continuous" thestring)
    ))

(defun distance-continuous (speakers)
  (let ((thestring))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%kdistance~D distance_continuous kxpos, kypos, kzpos, gisinkposx~D, gisinkposy~D, gisinkposz~D" i i i i))))
    (list "distance_continuous" thestring)
    ))

(defun attenuation-continuous (speakers)
  (let ((thestring))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf thestring (concatenate 'string thestring 
                                    (format nil "~1%asound~D attenuation_continuous asound~D, kdistance~D, iattenfunction" i i i))))
  (list "attenuation_continuous" thestring)
  ))

(defun airabsorption-continuous (speakers)
  (let ((thestring))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf thestring (concatenate 'string thestring 
                                    (format nil "~1%asound~D airabsorption_continuous asound~D, kdistance~D, iairfunction" i i i))))
  (list "airabsorption_continuous" thestring)
  ))

(defun timeoftravel-continuous (speakers)
  (let ((thestring))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf thestring (concatenate 'string thestring 
                                    (format nil "~1%asound~D timeoftravel_continuous asound~D, kdistance~D, itimefunction" i i i))))
  (list "timeoftravel_continuous" thestring)
  ))


;%%%%%%%%%%%%%%%%%% Csound Globals for OMPrisma .orcs %%%%%%%%%%%%%

(defun gblspeaker (ordinal self)
  (if (= (length self) 2)
      (list
       (list 
        (format nil "gisinkposx~D" ordinal) 
        (format nil "= ~D" (first self)))
       (list
        (format nil "gisinkposy~D" ordinal) 
        (format nil "= ~D" (second self)))
       (list
        (format nil "gisinkposz~D" ordinal) 
        (format nil "= ~D" 0)))
    (list
     (list 
      (format nil "gisinkposx~D" ordinal) 
      (format nil "= ~D" (first self)))
     (list
      (format nil "gisinkposy~D" ordinal) 
      (format nil "= ~D" (second self)))
     (list
      (format nil "gisinkposz~D" ordinal) 
      (format nil "= ~D" (third self))))
    ))
