(in-package :om)

; evaluate this line to generate the documentation
;; (omprisma-reference)

(defmethod special-ref-location ((self cs-spat-evt)) *prisma-ref-dir*)

(setf *prisma-ref-dir*
      (let ((lib (or *current-lib* (exist-lib-p "omprisma"))))
        (when lib
          (make-pathname :directory (append (pathname-directory (lib-pathname lib)) '("resources" "reference"))))))

(defun omprisma-reference ()
  (gen-chroma-reference (remove-if #'(lambda (item) 
                                       (and item (find (car item) '("Ambisonics" "Babo" "DBAP" "Panning" "Spat" "VBAP" "RVBAP" "VIMIC") :test 'string-equal)))
                                   (gen-chroma-entries (find-library "OMPrisma"))
                                   )
                        *prisma-ref-entries* *prisma-ref-dir* 
                        :title "OMPrisma Reference" 
                        :general-doc *prisma-general-doc* 
                        :doc-base *prisma-slots-doc*
                        :appendix *prisma-appendix*))

(setf *prisma-ref-entries* 
      '(("" 
         (
          ("Ambisonics"
           (ambi.discrete ambi.continuous ambi.trajectory))
          ("Babo" 
           (babo.discrete babo.continuous babo.trajectory))
          ("DBAP" 
           (dbap.discrete dbap.continuous dbap.trajectory))
          ("Panning" 
           (pan.discrete pan.continuous pan.trajectory))
          ("Spat"
           (spat.discrete spat.continuous spat.trajectory))  
          ("VBAP"
           (vbap.discrete vbap.continuous vbap.trajectory))
          ("RVBAP"
           (rvbap.discrete rvbap.continuous rvbap.trajectory))
          ("VIMIC"
           (vimic.discrete vimic.continuous vimic.trajectory))
         )))
        )

(setf *prisma-slots-doc* 
     '(      
        ("air-fun"		"lookup-table for air-absorption (hf-cutoff [Hz] as a function of distance).
        			0= no air-absorption, 1= approx. 3rd-order-polynomial curve, 2= exponential (100000Hz/distance),
        			else tables [GEN]")
        ("atten-fun"		"lookup-table for distance-attenuation (attenuation [dB] as a function of distance).
        			0= no attenuation, 1= approx. 1.5 dB/unit, 2= approx. 6 db/doubled distance,
        			else tables [GEN]")
        ("doppler-fun"		"lookup-table for time-of-flight (time-delay as a function of distance).
        			0 = no doppler, 1 = linear doppler-shift with speed of sound 340m/s, else tables [GEN]")
        ("time-fun"             "lookup-table for time-of-flight (time-delay as a function of distance).
        			0 = no time-delay, 1 = linear time-delay at 340m/s, else tables [GEN]")
        ("blur"                 "blur for dbap (percent) [float]")
        ("blur-env"             "blur -envelope [GEN-07]")
        ("blur-freqenv"         "blur -frequency envelope [GEN-07]")
        ("center-radius"	"radius of surrounding ring array of speakers [meter]")
        
	("durs" 		"Durations (sec) [float]")
	("e-dels" 		"Entry Delays [sec]")
        ("soundfile"		"Audio file [name, sound, string, pathname or GEN01]")
        ("gain"			"gain factor (dBFS for values <= 0.0, linear for values > 0.) [float]")
        ("gain-env"		"gain factor envelope (linear) scaled over the duration of an event [GEN-07]")
        ("mic-params"	        "microphone-parameters (directivity-patterns as tables)")
        ("mode"		        "if 1 = dbap (intensity of all speakers = 1), if 2 = omni-directional microphones [int]")
        ("order"		"ambisonic encoding order (0-3) [float]")
        ("order-env"		"ambisonic encoding order -envelope [GEN-07]")
        ("order-freqenv"	"ambisonic encoding order -frequency envelope in Hz [GEN-07]")
        ("pan-fun"		"panning lookup-table.
        			0= equal power, 1= sigmoid (s-shape), 2= linear, else tables [GEN07]")
        ("reverb-params"	"reverb-parameters (FDN-reverb: filtering & diffusion/modulation).
        			1= bright reverb, 2=dark reverb, else tables [GEN-02]")
        ("room-params"		"room-parameters (shoebox-model: room-geometry & wall-parameters) [GEN-02]")
        ("spread"		"spread or 'width' of the sound source (0=point source, 100=omni/all speakers) [float]")
        ("spread-env"		"spread -envelope [GEN-07]")
        ("spread-freqenv"	"spread -frequency envelope in Hz [GEN-07]")
        ("speaker-weights"      "defines weights for individual speakers as a table [GEN-02].")
        ("startpos"		"playback starting point in the soundfile (in seconds, if < 0 it will add silence at the beginning) [float]")
        ("pos-env"		"continuous 3D-trajectory in cartesian coordinates, see SpatDIF coordinate system [3Dtrajectory]")
        ("xpos"			"x-position (cartesian x-coordinate), see SpatDIF coordinate system [float]")
        ("xpos-env"		"x-position -envelope [GEN-07]")
        ("xpos-freqenv"		"x-position -frequency envelope in Hz [GEN-07]")
        ("ypos"			"y-position (cartesian y-coordinate), see SpatDIF coordinate system [float]")
        ("ypos-env"		"y-position -envelope [GEN-07]")
        ("ypos-freqenv"		"y-position -frequency envelope in Hz [GEN-07]")
        ("zpos"			"z-position (cartesian z-coordinate), see SpatDIF coordinate system [float]")
        ("zpos-env"		"z-position -envelope [GEN-07]")
        ("zpos-freqenv"		"z-position -frequency envelope in Hz [GEN-07]")
        ("reso-params"          "Resonator-parameters. 1=default, 2=less reverberant, 3=bighall [GEN-02]")
        ("action-time" 		"Start time of the whole event (sec) [float]")
        ("numcols" 		"Number of components for the event [int]")
        ("user-fun" 		"A processing function applied to each component when the event is being evaluated [lambda/fun-name]")
        ))


(setf *prisma-general-doc* 
" OMPrisma,  2009-2011 Marlon Schumacher (IDMIL/DCS/CIRMMT, McGill University)
 Library for Spatial Sound Synthesis with OpenMusic")

(setf *prisma-appendix*
      (list "Csound Tables Conventions"
            (append (cadr *chroma-appendix*)
             '("" "---------------------------------------------------------------------------" "")       
            '("panning functions"
              (100 "equal power 		(default size: 32769 points)")
              (101 "sigmoid (s-shape)	        (default size: 32769 points)")
              (102 "linear			(default size: 32769 points)")
              ""
              "distance-attenuation functions"
              (200      "no attenuation")
              (201	"approx. 1.5 db/unit exponential amplitude rolloff				(default size: 513 points)")
              (202	"approx. 6 db/doubled distance inverse square-law amplitude rolloff		(default size: 32769 points)")
              ""
              "Air-absorption functions"
              (301	"approximation of 3rd-order-polynomial air-absorption curve (D.Valente)		(default size: 32769 points)")
              (302	"exponential frequency rolloff as function of distance (100000Hz/distance)	(default size: 32769 points)")	
              ""
              "Doppler-functions"
              (401	"linear doppler function (speed of sound: 340m/s)				(default size: 4097 points)")
              ""
              "room-params (spat)"
              (451	"default room                                                                   (default size: 64 points)")
              ""
              "reverb-params (rvbap)"
              (461	"dark reverb									(default size: 16 points)")
              (462	"bright reverb								        (default size: 16 points)")
              ""
              "reso-params (babo)"
              (471      "defaults                                                                       (default size: 16 points)")
              (472      "less reverberant                                                               (default size: 16 points)")
              (473      "big hall                                                                       (default size: 16 points)")
              ""
              "speaker-weights (dbap)"
              (501      "unity gain for all speakers                                                    (default size: 513 points)")
              "" ""
              "The naming-convention in OMPrisma for slots of type cs-table is:"
              ""
              "'env' for envelopes (meaning tables are looked up as a function of time)"
              "'fun' for lookup-tables (meaning tables are looked up as a function of another variable)"
              "'params' for parameter-tables (meaning tables act as an array (data-storage) for discrete parameter-values)"
              ))))