;OMPrisma,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)


;---rescale-cs-table

(defmethod! rescale-env (input &key min max)
            :icon 39   
            :initvals '(nil min max)
            :indoc '("nothing")
            :numouts 1
            :doc "bla bla"  
            (list input 'scale-cs-table min max))

#|
(defmethod! rescale-xenv (input xmin xmax)
            (list input #'(lambda (in min max) (scale-cs-table in min max)) xmin xmax))
|#

;; these should be made available as 'extensions' (just the algorithms)
;; and then in this file as a pre-defined user-fun
;; build the functions in a way that they can be used for discrete and continuous control!





;; W-PANNING %%%%%%%%%%%%%%%%%%%%%

;;Q: what is the unit for 'width'?

(defmethod! gamma ((distance number) (width number))
        (/ (* distance distance) (+ (* distance distance) (* width width))))

(defmethod! gamma->order ((gamma number))
            (omif (> gamma 0)
                  (expt (+ 1 (expt gamma -1)) -1/2)
                   0)
            )


;; ambisonic order as a function of distance + width
;; discrete control

(defmethod! w-panning ((distance number) (width number))
            :icon 67
            :initvals '(1 1)
            :indoc '("distance" "width")
            :numouts 2
            :doc "returns ambisonic order as a function of width an distance of a sound source. Implements W-panning function."
            (setf thegamma (gamma distance width)
                  theorderfactor (gamma->order thegamma))
            (values theorderfactor thegamma)
            )
            
            
;; need to do this for lists, too...
(defmethod! w-panning ((distance list) (width number))

            )

;; continuous control (with BPFs)


;; w-panning distance-attenuation control

(defmethod! W-distance ((r number) (b number) (a number) (k number))
            :icon 39   
            :initvals '(1 1 1 1)
            :indoc '("distance" "constant" "order" "dimensions")
            :numouts 1
            :doc "computes gain factor as a function of width and distance"             
            (/ (+ 1 r) 
               (+ (* (expt a -1)(+ 1 r))
                  (* (expt b -1)(expt r (+ 1 k)))))
            )
           

;; SOUND SURFACE PANNING %%%%%%%%%%%%%%%%

;; first a method only for rectangles, later a method for polygons
;; SSP could (similar to W-panning) later be scaled according to distance


(defmethod! 2D-mesh ((xminmax list) (yminmax list) (resolution number))
            :icon 39   
            :initvals '(nil nil 0.05)
            :indoc '("nothing")
            :numouts 1
            :doc "bla bla"
            
            (let* ((xmin (first xminmax))
                   (xmax (second xminmax))
                   (xsize (- xmax xmin))
                   (ymin (first yminmax))
                   (ymax (second yminmax))
                   (ysize (- ymax ymin))
                   (xpts (arithm-ser xmin xmax resolution))
                   (ypts (arithm-ser ymin ymax resolution)))
                  
            (flat (loop for yitem in ypts collect
                  (loop for xitem in xpts collect
                        (list xitem yitem)
                        )
                  ) 1)
            )
)

(defmethod! 2D-mesh2 ((xminmax list) (yminmax list) (numsamples number))
            :icon 39   
            :initvals '(nil nil 0.05)
            :indoc '("nothing" "nothing" "nothing")
            :numouts 1
            :doc "bla bla"
            
            (let* ((xmin (first xminmax))
                   (xmax (second xminmax))
                   (xsize (- xmax xmin))
                   (xresolution (/ xsize numsamples))
                   (ymin (first yminmax))
                   (ymax (second yminmax))
                   (ysize (- ymax ymin))
                   (yresolution (/ ysize numsamples))
                   (xpts (arithm-ser xmin xmax xresolution))
                   (ypts (arithm-ser ymin ymax yresolution)))
                  
            (flat (loop for yitem in ypts collect
                  (loop for xitem in xpts collect
                        (list xitem yitem)
                        )
                  ) 1)
            )
)