;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)

(defparameter *ambi-default-tables*
  (list
   "; LOOKUP-TABLES ----------"
   "; "
   "; Distance functions"  
   "f200 0 513 -7 0 512; no distance attenuation"
   "f201 0 513 -27 0 -6  10 0  512 -75 ; appr. 1.5 db/m exponential rolloff as function of distance"
   "f202 0 32769 -6 -6  640 0  20000 -35 12129 -36 ; approx 6 db/doubledistance inverse square-law rolloff"
   "; "
   "; Airabsorption functions"
   "f301 0 32769 -6 20048 640 20048 10284 4000 10922 3000 10923 2000 ; approximation of D.Valente's air-absorption curve"
   "f302 0 32769 -25 0 20048 640 20048 32768 2000 ; exponential frequency rolloff as function of distance (100000/distance)"
   "; "
   "; Time-of-flight functions"
   "f401 0 4097 -7 0 4096 0.1470588 ; linear function "
   "; ")
  )

; %%%%%%%%%%%%%%%%%%% AMBI SETUP %%%%%%%%%%%%%%%%%%%%

;; combined method for lists and strings (presets)
(defmethod! ambi-setup ((self t) &optional osc)
            :icon 14 
            :initvals '("2D.3rd-order" t)
            :menuins '((0 (("2D.1st-order" "2D.1st-order")("2D.2nd-order" "2D.2nd-order")
                           ("2D.3rd-order" "2D.3rd-order")("3D.1st-order" "3D.1st-order")
                           ("3D.2nd-order" "3D.2nd-order")("3D.3rd-order" "3D.3rd-order")
                           ("UHJ" "UHJ")
                           )))                       
            :indoc '("select dimensions/order-preset from menu or connect 'speaker-setup' to deduce automatically from loudspeaker setup" "when active sends OSC-messages to the Multiplayer")
            :numouts 1
            :doc "sets encoding order and dimensionality (optionally synchronizes Multiplayer)"

            (when (stringp self)
              (setf params; (channels dimensions order)
                     (cond ((equal self "2D.1st-order") '(3 2 1))
                           ((equal self "2D.2nd-order") '(5 2 2))
                           ((equal self "2D.3rd-order") '(7 2 3))
                           ((equal self "3D.1st-order") '(4 3 1))
                           ((equal self "3D.2nd-order") '(9 3 2))
                           ((equal self "3D.3rd-order") '(16 3 3))
                           ((equal self "UHJ") '(2 0 2))
                           )))
            (when (listp self)
              (setf params (deduce-encoding-params self)))
             ;(print params)
              (setf channels (first params)                   
                    dimensions (second params)
                    order (third params))

;;OSC MultiPlayer communication----
               (when osc 
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/dimensions" dimensions))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/dimensions" dimensions))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/order" order))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/order" order))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/audio/bypass" 0))
                )
               (if (equal self "UHJ")
                   (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/format" "UHJ"))
                 (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/format" "Bformat"))
                 )
;;cs-inits-------------------------
               (list
                (cs-nchnls channels)
                (cs-globals (ambi-globals dimensions order))
                (cs-opcodes (list *envelope* *trajectory*
                                  *xyz2aed-discrete* *attenuation-discrete* *airabsorption-discrete* *timeoftravel-discrete*
                                  *xyz2aed-continuous* *attenuation-continuous* *airabsorption-continuous* *timeoftravel-continuous*))
                (cs-macros (ambi-macros '(t)))
                )
               )

(defmethod! ambi-setup ((self BPC) &optional osc)
            (ambi-setup (point-pairs self) osc)
            )

(defmethod! ambi-setup ((self 3DC) &optional osc)
            (ambi-setup (point-pairs self) osc)
            )
; 
; ====== Helper Function to deduce order/dimensions from LS-setup

(defmethod! deduce-encoding-params ((speakers list))
            :numouts 1
            (setf dims (if (> (length (first speakers)) 2)
                              (if 
                                  (find-if-not 'zerop (x->dx (third (mat-trans speakers))))
                                  ;if the delta between any of the zpositions is not 0 then it's in 3 dimenstions
                                  3
                                2)
                         2))
            (setf numspeakers (length speakers))
            (setf order
                  (clip (if (equal dims 3)
                            (ceiling (- (sqrt numspeakers) 1))
                          (ceiling (- (/ numspeakers 2) 1))) 
                        1 3)) ;note: this might later be changed when higher-orders are available
            (setf channels 
                  (if (equal dims 2)
                      (+ 1 (* 2 order))
                       (expt (+ 1 order) 2)))
                  (list channels dims order))

; for bpcs/3dcs
(defmethod! deduce-encoding-params ((speakers bpf))
            (deduce-encoding-params (point-pairs speakers))
            )

#|
(defmethod! deduce-encoding-params ((speakers t))
            (deduce-encoding-params (point-pairs speakers))
            )
|#

; %%%%%%%%%%%%%%%%%%% AMBI GLOBALS %%%%%%%%%%%%%%%%%%%%

(defun ambi-globals (dims order)
  (list
  (list 
   (format nil "gidims") 
   (format nil "= ~D" dims))
  (list
   (format nil "giorder") 
   (format nil "= ~D" order))
    ))


; %%%%%%%%%%%%%%%%%%% AMBI MACROS %%%%%%%%%%%%%%%%%%%%

(defun ambi-macros (self)
  (list (copy-source '(t) 'ambi) (csound-envelopes '(xpos ypos zpos order) 'ambi) (csound-trajectories '(xpos ypos zpos order) 'ambi)
; discrete control
  (xyz2aed-discrete '(t)) (attenuation-discrete '(t)) (airabsorption-discrete '(t)) (timeoftravel-discrete '(t))
; continuous control
  (xyz2aed-continuous '(t)) (attenuation-continuous '(t)) (airabsorption-continuous '(t)) (timeoftravel-continuous '(t)))
  )
