;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)

(defparameter *babo-default-tables*
  (list
   "; RESONATOR-parameters"
   "f471  0 16 -2 0.99 0.1 0 0 0 .3 .5 .8 1 14.39 11.86 10    ;defaults"
   "f472  0 16 -2 0.4  0.9 0 0 0 .2 .5 1. .5  14.39 11.86 10  ;less reverberant"
   "f473  0 16 -2 0.2  0.9 0 0 0 .2 .5 1. 1 43.18 35.59 30    ;big hall"
   "; ")
  )

; %%%%%%%%%%%%%%%%%%% BABO SETUP %%%%%%%%%%%%%%%%%%%%

(defmethod! babo-setup ((self list) &optional osc)
            :icon 70  
            :initvals '(nil t)
            :indoc '("specify loudspeaker configurations via bpc, 3DC or list of x-y-tuplets / x-y-z-triplets"  "when active synchronizes Multiplayer settings")
            :numouts 1
            :doc "describes speaker setups and UDOs for babo-classes. Connect to inits of synthesize method"

            (setf channels (length self))
            (setf dims (length (first self)))

;;send Open Sound Control messages
            (when osc
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/audio/bypass" 1))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/audio/bypass" 1))
                )
    (list
     (cs-nchnls channels)
     (cs-globals (babo-globals self))
     (cs-opcodes (list *envelope* *trajectory* *babo-discrete* *babo-continuous*))
     (cs-macros (babo-macros self))
    ))

(defmethod! babo-setup ((self BPC) &optional osc)
            (babo-setup (point-pairs self) osc)
            )

(defmethod! babo-setup ((self 3DC) &optional osc)
            (babo-setup (point-pairs self) osc)
            )



; %%%%%%%%%%%%%%%%% BABO GLOBALS %%%%%%%%%%%%%%%%%%%

; 'speakers' is a list of tuplets describing speakers positions, e.g. ((-1 1) (1 1)) or ((-1 1 0) (1 1 0)) ...

(defun babo-globals (speakers)
  (let ((speakerlist))
    (setf speakerlist
        (loop for speaker in speakers 
                        for i = 1 then (+ i 1) collect
                        (gblspeaker i speaker)))
    (flat speakerlist 1)
    ))

; %%%%%%%%%%%%%%%%% BABO OPCODES %%%%%%%%%%%%

(defparameter *babo-discrete*
  (list 
   "babo_discrete" 
   '("a" "aiiiiiiiiiii")
   "asound, iposx, iposy, iposz, isinkposx, isinkposy, isinkposz, isizex, isizey, isizez, idiffusion, iresoparams xin
	tableiw isinkposx, 2, iresoparams
	tableiw isinkposy, 3, iresoparams
	tableiw isinkposz, 4, iresoparams
	aspkfeed, adummy babo asound, iposx, iposy, iposz, isizex, isizey, isizez, idiffusion, iresoparams
	xout aspkfeed"))

(defparameter *babo-continuous*
  (list 
   "babo_continuous" 
   '("a" "akkkiiiiiiii")
   "asound, kposx, kposy, kposz, isinkposx, isinkposy, isinkposz, isizex, isizey, isizez, idiffusion, iresoparams xin
	tableiw isinkposx, 2, iresoparams
	tableiw isinkposy, 3, iresoparams
	tableiw isinkposz, 4, iresoparams
	aspkfeed, adummy babo asound, kposx, kposy, kposz, isizex, isizey, isizez, idiffusion, iresoparams
	xout aspkfeed"))


; %%%%%%%%%%%%%%%%% BABO MACROS %%%%%%%%%%%%

(defun babo-macros (speakers)
  (list (csound-envelopes '(xpos ypos zpos) 'babo)  (csound-trajectories '(xpos ypos zpos) 'babo)
        (babo-discrete speakers) (babo-continuous speakers)
  ))


(defun babo-discrete (speakers)
  (let ((macrostr))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf macrostr (concatenate 'string macrostr 
                                    (format nil "~1%aspkfeed~D babo_discrete asound, ixpos, iypos, izpos, gisinkposx~D, gisinkposy~D, gisinkposz~D, isizex, isizey, isizez, idiffusion, iresoparams" i i i i))))
  (setf macrostr (concatenate 'string macrostr
                              (format nil "~1%outc aspkfeed1")))
  (loop for item in (cdr speakers)
        for i = 2 then (+ i 1) do
        (setf macrostr (concatenate 'string macrostr
                                    (format nil ", aspkfeed~D" i))))
  (list "babo_discrete" macrostr)
  ))

(defun babo-continuous (speakers)
  (let ((macrostr))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf macrostr (concatenate 'string macrostr 
                                      (format nil "~1%aspkfeed~D babo_continuous asound, kxpos, kypos, kzpos, gisinkposx~D, gisinkposy~D, gisinkposz~D, isizex, isizey, isizez, idiffusion, iresoparams" i i i i))))
    (setf macrostr (concatenate 'string macrostr
                                (format nil "~1%outc aspkfeed1")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do
          (setf macrostr (concatenate 'string macrostr
                                      (format nil ", aspkfeed~D" i))))
  (list "babo_continuous" macrostr)
  ))


; %%%%%%%%%%%%%%%%% BABO Reso-function %%%%%%%%%%%%


;(defvar *reso-params* '(0.99 0.1 0 0 0 0. .5 .8 1 14.39 11.86 10)) ;change this in the default

(defmethod! babo-resonator (id size-x size-y size-z &key (decay 99) (hi-decay 10) (early-atten 80) (diffusion 10) (direct 50))
   :initvals '("?" 14.39 11.86 10 99 10 80 10 50)
   :icon 70
   :doc "defines reso-parameters for Ball-in-a-Box model (babo)"
 (let ((reso-params (list (* decay 0.01) (* hi-decay 0.01)
                          0 0 0 0.0
                          (* direct 0.01)
                          (* early-atten 0.01)
                          (* diffusion 0.01)
                          size-x size-y size-z)))
    (make-instance 'gen-cs-table
                   :id id
                   :size 16 :stime 0 :gen-num -2
                   :param-list reso-params)
    ))



#| ;old version
;;; BABO - RESONATOR FUNCTION %%%%%%%%%%%%%

(defvar *resonator-list* '(0 8 -2 0.55 0.5 0 0 0 1 0. 0.8))

(defmethod! babo-resonator2 (id decay hidecay (receiver-position list) pickup-distance direct-atten early-atten)
    :initvals '(101 nil nil nil nil nil nil) 
    :icon 37
    :doc "defines resonator-parameters for babo"

(let* ((resonator-list (copy-list *resonator-list*))
       (pickup-distance 1)
       (receiver-position '(0 1 0)))
  (setf (nth 0 resonator-list) decay)
  (setf (nth 1 resonator-list) hidecay)
  (setf (nth 2 resonator-list) (first receiver-position))
  (setf (nth 3 resonator-list) (second receiver-position))
  (setf (nth 4 resonator-list) (third receiver-position))
  (setf (nth 5 resonator-list) pickup-distance)
  (setf (nth 6 resonator-list) direct-atten)
  (setf (nth 7 resonator-list) early-atten)
  (make-instance 'gen-cs-table
                 :id id
                 :size 8 :stime 0 :gen-num -2
                 :param-list resonator-list)
  
))
|#
