;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)


(defparameter *dbap-default-tables*
  (list
   "; LOOKUP-tables *****"
   "; "
   "; Distance functions"  
   "f201 0 513 -27 0 12 10 0  512 -75 ; appr. 1.5 db/m exponential rolloff as function of distance"
   "f202 0 32769 -6 6  640 0  20000 -35 12129 -36 ; approx 6 db/doubledistance inverse square-law rolloff"
   "; Airabsorption functions"
   "f301 0 32769 -6 20048 640 20048 10284 4000 10922 3000 10923 2000 ; approximation of D.Valente's air-absorption curve"
   "f302 0 32769 -25 0 20048 640 20048 32768 2000 ; exponential frequency rolloff as function of distance (100000/distance)"
   "; Time-of-travel functions"
   "f401 0 4097 -7 0 4096 0.1470588 ; linear function "
   "; Speaker weights"
   "f501 0 513 -7 1 512 1"
   "; "))

; %%%%%%%%%%%%%%%%%%% DBAP SETUP %%%%%%%%%%%%%%%%%%%%

(defmethod! dbap-setup ((self list) &optional osc)
            :icon 73  
            :initvals '(nil t)
            :indoc '("specify loudspeaker configurations via bpc, 3DC or list of x-y-tuplets / x-y-z-triplets"  "when active synchronizes Multiplayer settings")
            :numouts 1
            :doc "describes speaker setups, opcodes and macros for dbap-classes."
            (setf channels (length self))
            (setf dims (length (first self)))
;;send Open Sound Control messages
            (when osc
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/audio/bypass" 1))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/audio/bypass" 1))
                )                    
            (list (cs-nchnls channels)
                  (cs-globals (dbap-globals self))
                  (cs-opcodes (list *envelope* *trajectory* 
                                    *distance-discrete* *dbap-discrete* *airabsorption-discrete* *timeoftravel-discrete* 
                                    *distance-continuous* *dbap-continuous* *airabsorption-continuous* *timeoftravel-continuous*))
                  (cs-macros (dbap-macros self))
                  ))

(defmethod! dbap-setup ((self BPC) &optional osc)
            (dbap-setup (point-pairs self) osc)
            )

(defmethod! dbap-setup ((self 3DC) &optional osc)
            (dbap-setup (point-pairs self) osc)
            )


; %%%%%%%%%%%%%%%%% DBAP GLOBALS %%%%%%%%%%%%%%%%%%%

(defun dbap-globals (speakers)
  (let ((speakerlist)
        (covariance))
    (setf speakerlist
        (loop for speaker in speakers 
                        for i = 1 then (+ i 1) collect
                        (gblspeaker i speaker)))
    (setf covariance (list (list "gidistance_covariance" (format nil "= ~D" (distance-covariance speakers)))))
   (x-append (flat speakerlist 1) covariance)
    ))




; %%%%%%%%%%%%%%%%% DBAP OPCODES %%%%%%%%%%%%%%%%%%%

(defparameter *dbap-discrete*
  (list 
   "dbap_discrete" 
   '("i" "iiiii")
   "idist, iblur, ispkindex, iattenfunction, iweightfunction xin
	idist = (idist^2+iblur^2)^0.5
	index = idist*0.02
	irolloff  tablei  index, iattenfunction, 1, 0, 0
	;note that in the DBAP paper the attenuation is doubled (2d*a)?
	iweight tab_i ispkindex-1, iweightfunction, 0
	irolloff = irolloff * 1/iweight
	ispkfeed limit (10^(irolloff*0.05)), 0, 1
	xout ispkfeed"))

(defparameter *dbap-continuous*
  (list 
   "dbap_continuous" 
   '("k" "kkiii")
   "kdist, kblur, ispkindex, iattenfunction, iweightfunction xin
	kdist = (kdist^2+kblur^2)^0.5
	kindex = kdist*0.02
	krolloff  tablei  kindex, iattenfunction, 1, 0, 0
	;note that in the DBAP paper the attenuation is doubled (2d*a)?
	iweight tab_i ispkindex-1, iweightfunction, 0
	krolloff = krolloff * 1/iweight
	kspkfeed limit (10^(krolloff*0.05)), 0, 1
	xout kspkfeed"))


; %%%%%%%%%%%%%%%%% DBAP MACROS %%%%%%%%%%%%%%%%%%%

(defun dbap-macros (speakers)
  (list
   (copy-source speakers 'dbap) (csound-envelopes '(xpos ypos zpos blur) 'dbap)  (csound-trajectories '(xpos ypos zpos blur) 'dbap)
; discrete control
   (airabsorption-discrete speakers) (timeoftravel-discrete speakers)
; continuous control
   (airabsorption-continuous speakers) (timeoftravel-continuous speakers)
; renderer-specific
  (distance-discrete speakers) (dbap-attenuation-discrete speakers)(dbap-discrete1 speakers) (dbap-discrete2 speakers) (dbap-discrete3 speakers) 
  (distance-continuous speakers) (dbap-attenuation-continuous speakers) (dbap-continuous1 speakers) (dbap-continuous2 speakers) (dbap-continuous3 speakers) 
))

(defun dbap-attenuation-discrete (speakers)
  (let ((thestring))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%ispkfeed~D dbap_discrete idistance~D, iblur, ~D, iattenfunction, iweightfunction" i i i))))
    (list "dbap_attenuation_discrete" thestring)
    ))

; maybe instead of "kspkfeed-n" better "kgainch-n"
(defun dbap-attenuation-continuous (speakers)
  (let ((thestring))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf thestring (concatenate 'string thestring 
                                    (format nil "~1%kspkfeed~D dbap_continuous kdistance~D, kblur, ~D, iattenfunction, iweightfunction" i i i))))
  (list "dbap_attenuation_continuous" thestring)
  ))

(defun dbap-discrete1 (speakers)
  (let ((macrostr))
    (setf macrostr (concatenate 'string macrostr (format nil "~1%igainfactor = 1/(ispkfeed1^2")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do 
          (setf macrostr (concatenate 'string macrostr 
                                      (format nil "+ispkfeed~D^2" i))))
    (setf macrostr (concatenate 'string macrostr ")"))
    (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf macrostr (concatenate 'string macrostr 
                                    (format nil "~1%aspkfeed~D = igainfactor * (ispkfeed~D^2) * asound~D" i i i))))
    (setf macrostr (concatenate 'string macrostr
                                (format nil "~1%outc aspkfeed1")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do
          (setf macrostr (concatenate 'string macrostr
                                      (format nil ", aspkfeed~D" i))))
    (list "dbap_discrete1" macrostr)))

(defun dbap-discrete2 (speakers)
  (let ((macrostr))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf macrostr (concatenate 'string macrostr 
                                      (format nil "~1%aspkfeed~D = ispkfeed~D * asound~D" i i i))))
    (setf macrostr (concatenate 'string macrostr
                                (format nil "~1%outc aspkfeed1")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do
          (setf macrostr (concatenate 'string macrostr
                                      (format nil ", aspkfeed~D" i))))
    (list "dbap_discrete2" macrostr)))
  
(defun dbap-discrete3 (speakers)
  (list "dbap_discrete3" ";--- NOT YET IMPLEMENTED ----"))

(defun dbap-continuous1 (speakers)
  (let ((macrostr))
    (setf macrostr (concatenate 'string macrostr (format nil "~1%kgainfactor = 1/(kspkfeed1^2")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do 
          (setf macrostr (concatenate 'string macrostr 
                                      (format nil "+kspkfeed~D^2" i))))
    (setf macrostr (concatenate 'string macrostr ")"))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf macrostr (concatenate 'string macrostr 
                                      (format nil "~1%aspkfeed~D = kgainfactor * (kspkfeed~D^2) * asound~D" i i i))))
    (setf macrostr (concatenate 'string macrostr
                                (format nil "~1%outc aspkfeed1")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do
          (setf macrostr (concatenate 'string macrostr
                                      (format nil ", aspkfeed~D" i))))
    (list "dbap_continuous1" macrostr)))

(defun dbap-continuous2 (speakers)
  (let ((macrostr))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf macrostr (concatenate 'string macrostr 
                                      (format nil "~1%aspkfeed~D = kspkfeed~D * asound~D" i i i))))
    (setf macrostr (concatenate 'string macrostr
                                (format nil "~1%outc aspkfeed1")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do
          (setf macrostr (concatenate 'string macrostr
                                      (format nil ", aspkfeed~D" i))))
    (list "dbap_continuous2" macrostr)))

(defun dbap-continuous3 (speakers)
  (list "dbap_continuous3" ";--- NOT YET IMPLEMENTED ----"))
  

;------------ WEIGHTS ----------------

(defmethod! dbap-weights ((self bpf) &optional (table-id "?"))
            :icon 73 
            :initvals '(nil "?")
            :indoc '("bpf/bpf-lib or list describing speaker weights" "table-id")
            :numouts 1
            :doc "creates a table of speaker-weights for dbap"

     (make-instance 'gen-cs-table
                    :id table-id
                    :size (length (y-points self)) 
                    :stime 0 
                    :gen-num -2
                    :param-list (y-points self))
     )

(defmethod! dbap-weights ((self list) &optional (table-id "?"))
            (when (bpf-p (first self))
              (mapcar #'(lambda (thebpf theid) (dbap-weights thebpf theid))
                      self (repeat-n table-id (length self))))
            (when (numberp (first self))
              (make-instance 'gen-cs-table
                             :id table-id
                             :size (length self) 
                             :stime 0 
                             :gen-num -2
                             :param-list self)
              ))

(defmethod! dbap-weights ((self bpf-lib) &optional (table-id "?"))
            (dbap-weights (bpf-list self) table-id)
            )


;;; calculate distance covariance from center-position to speakers
(defun distance-covariance (speakerpositions)
  (when (equal (length (first speakerpositions)) 2)
    (setf speakerpositions (mat-trans (x-append (mat-trans speakerpositions) (list (make-list (length speakerpositions) :initial-element 0))))))
  (let* ((speakerstrans (mat-trans speakerpositions))        
         (xposmean (om-mean (first speakerstrans)))
         (yposmean (om-mean (second speakerstrans)))
         (zposmean (om-mean (third speakerstrans)))
         (center-position (x-append xposmean yposmean zposmean)))
         (setf sumofdistances (om-sum2
          (loop for speaker in speakerpositions
                collect
                (3d-distance speaker center-position))))
         (sqrt (/ sumofdistances (length speakerpositions)))
         ))
