;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)

(defparameter *panning-default-tables*
  (list
   "; LOOKUP-TABLES ----------"
   "; Panning functions"                 
   "f100 0 32769 9 .25 1 0 ; equal power"
   "f101 0 32769 19 .5 .5 270 .5 ; sigmoid (s-shape)"                        
   "f102 0 32769 7 0 32768 1 ; linear"
   "; "
   "; Distance functions"  
   "f200 0 513 -7 0 512; no distance attenuation"
   "f201 0 513 -27 0 -6  10 0  512 -75 ; appr. 1.5 db/m exponential rolloff as function of distance"
   "f202 0 32769 -6 -6  640 0  20000 -35 12129 -36 ; approx 6 db/doubledistance inverse square-law rolloff"
   "; "
   "; Airabsorption functions"
   "f301 0 32769 -6 20048 640 20048 10284 4000 10922 3000 10923 2000 ; approximation of D.Valente's air-absorption curve"
   "f302 0 32769 -25 0 20048 640 20048 32768 2000 ; exponential frequency rolloff as function of distance (100000/distance)"
   "; "
   "; Time-of-travel function"
   "f401 0 4097 -7 0 4096 0.1470588 ; linear function "
   "; ")
  )


(defun numtosymbol (arg)
  (if (equal (type-of arg) 'single-float)
    (make-symbol (format nil "~d" arg))
    arg)
  )


; %%%%%%%%%%%%%%%%%%% PAN SETUP %%%%%%%%%%%%%%%%%%%%

(defmethod! pan-setup ((preset t) &optional osc)
            :icon 09 
            :initvals '(stereo t)
            :menuins '((0 (("stereo" 'stereo) ("quad" 'quad)
                           ("5.0" '5.0)) ;(numtosymbol '5.0)
                           ))                       
            :indoc '("select panning type" "when active sends OSC-messages to the Multiplayer")
            :outdoc '("connect to :inits of synthesize method")
            :numouts 1
            :doc "configures panning modes for pan-classes."

            (let ((channels
                   (cond ((equal preset 'stereo) 2)
                         ((equal preset 'quad) 4)
                         ((equal preset '5.0) 5)
                         )))
              
              ;;OSC MultiPlayer communication----

              (if channels                 
                   (progn
                   (when osc
                     (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/audio/bypass" 1))
                     (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/audio/bypass" 1))
                     )
;;cs-inits-------------------------
                    (list
                     (cs-nchnls channels)
                     (cs-globals (pan-globals preset))
                     (cs-opcodes (list *envelope* *trajectory*
                                       *xyz2aed-discrete* *attenuation-discrete* *airabsorption-discrete* *timeoftravel-discrete*
                                       *xyz2aed-continuous* *attenuation-continuous* *airabsorption-continuous* *timeoftravel-continuous*))
                     (cs-macros (pan-macros '(t)))
                     )
                    )
                (progn
                  (om-message-abort "not a valid loudspeaker-configuration for this class")
                  ))
              ))


(defmethod! pan-setup ((preset bpf) &optional osc)
        (let* ((numspeakers (length (point-pairs preset)))
               (thepreset 
                (cond ((equal numspeakers 2) 'stereo)
                      ((equal numspeakers 4) 'quad)
                      ((equal numspeakers 5) '5.0))))
               (if thepreset
                   (progn
                     (om-beep-msg (format nil "assuming ~s setup" thepreset))
                     (pan-setup thepreset osc))
                 (om-message-abort "not a valid loudspeaker-configuration for this class"))
               ))

(defmethod! pan-setup ((preset list) &optional osc)
        (let* ((numspeakers (length preset))
               (thepreset 
                (cond ((equal numspeakers 2) 'stereo)
                      ((equal numspeakers 4) 'quad)
                      ((equal numspeakers 5) 5.0))))
               (if thepreset
                   (progn
                     (om-beep-msg (format nil "assuming ~s setup" thepreset))
                     (pan-setup thepreset osc))
                 (om-message-abort "not a valid loudspeaker-configuration for this class"))
               ))


; %%%%%%%%%%%%%%%%%%% PAN GLOBALS %%%%%%%%%%%%%%%%%%%%

(defun pan-globals (panmode)
  (setf themode
   (cond ((equal panmode 'stereo) 1)
         ((equal panmode 'quad) 2)
         ((equal panmode '5.0) 3)))
  (list
  (list 
   (format nil "gipanmode") 
   (format nil "= ~D" themode))
  ))

; %%%%%%%%%%%%%%%%%%% PAN MACROS %%%%%%%%%%%%%%%%%%%%

(defun pan-macros (self)
  (list (copy-source '(t) 'pan) (csound-envelopes '(xpos ypos) 'pan) (csound-trajectories '(xpos ypos) 'pan)
; discrete control
  (xyz2aed-discrete '(t)) (attenuation-discrete '(t)) (airabsorption-discrete '(t)) (timeoftravel-discrete '(t))
; continuous control
  (xyz2aed-continuous '(t)) (attenuation-continuous '(t)) (airabsorption-continuous '(t)) (timeoftravel-continuous '(t)))
  )
