;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)

(defparameter *rvbap-default-tables*
  (list
   "; LOOKUP-tables *****"
   "; "
   "; Distance functions"  
   "f200 0 513 -7 0 512; no distance attenuation"
   "f201 0 513 -27 0 -6  10 0  512 -75 ; appr. 1.5 db/m exponential rolloff as function of distance"
   "f202 0 32769 -6 -6  640 0  10000 -30 22129 -36 ; approx 6 db/doubledistance inverse square-law rolloff"
   "; "
   "; Airabsorption functions"
   "f301 0 32769 -6 20048 640 20048 10284 4000 10922 3000 10923 2000 ; approximation of D.Valente's air-absorption curve (best fit 3rd-order-polynomial)"
   "f302 0 32769 -25 0 20048 640 20048 32768 2000 ; exponential frequency rolloff as function of distance (100000/distance)"
   "; "
   "; Time functions"
   "f401 0 4097 -7 0 4096 0.1470588 ; linear function "
   "; "
   "; Reverb functions"
   (reduce #'string+ (repeat-n " 7000 .6" 64) :initial-value "f461 0 256 -2 50 20 50") ;bright room
   (reduce #'string+ (repeat-n " 3000 10" 64) :initial-value "f462 0 256 -2 50 20 50") ;dark room
   "; "))

; %%%%%%%%%%%%%%%%%%% RVBAP SETUP %%%%%%%%%%%%%%%%%%%%

(defmethod! rvbap-setup ((self list) &optional osc)
;; ToDo: if it's a list of atoms the values must be azimuth!
            :icon 46  
            :initvals '(nil t)
            :indoc '("specify loudspeaker configurations via bpc, 3DC or list of x-y-z-triplets or x-y-tuplets"  
                     "when active synchronizes Multiplayer settings")
            :numouts 1
            :doc "describes a speaker setup for rvbap-classes. connect to :inits of synthesize method"
            :outdoc '("connect to :inits of synthesize method")

;;OSC MultiPlayer communication----            
            (when osc
              (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/audio/bypass" 1))
              (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/audio/bypass" 1))
              )
;;cs-inits-------------------------
            (list
             (cs-nchnls (length self))
             (cs-globals (rvbap-globals self))
             (cs-opcodes (list *envelope* *trajectory* 
                               *xyz2aed-discrete* *attenuation-discrete* *airabsorption-discrete* *timeoftravel-discrete*
                               *xyz2aed-continuous* *attenuation-continuous* *airabsorption-continuous* *timeoftravel-continuous*
                               *rvbap-rolloff-discrete* *rvbap-rolloff-continuous*))
             (cs-macros (rvbap-macros self))
                     ))

(defmethod! rvbap-setup ((self BPC) &optional osc)
            :numouts 1
            (rvbap-setup (point-pairs self) osc)
            )

(defmethod! rvbap-setup ((self 3DC) &optional osc)
            :numouts 1
            (rvbap-setup (point-pairs self) osc)
            )


; %%%%%%%%%%%%%%%% RVBAP - REVERB %%%%%%%%%%%

; I think in Prisma1 it was possible to globally scale the level, feedback and spread.
; Better approach: spread, level, feedback, cutoff, modulation per speaker + global scalers ;maybe a function sth. like rvbap-revparam 
; (30 50 75 7000 .6)

(defmethod! rvbap-params (&key  (level -6) (feedback 60) (cutoff-freq 7000) (modulation .6))
            :initvals '(-6 60 7000 .6)
            :icon 46
            ;doc
            (list level feedback cutoff-freq modulation)
            )

; could make some defvars as rvbap presets 

(defmethod! rvbap-reverb (id (level number) (spread number) (feedbackfactor number) (cutoff-factor number) (pitchmod-factor number) &rest params)
            
                :initvals '("?" -6 20 1 1 1 '(-6 60 7000 .6))
                :icon 46
                :doc "defines reverb settings for rvbap"
                :indoc '("GEN-table-id"  "level of reverb in percent (linear)" "feedback factor (linear)" "factor for cutoff-frequency (linear)" "pitchmodulation-factor (linear)" 
                                         "lists of sublists containg level (db), feedback (percent), cutoff-frequency (Hz), modulation (percent)")

                (setf reverb-list (arithm-ser 0 512 1)        
                      reverb-list (subs-posn reverb-list '(0 1 2 3 4) (list (db->lin level) spread feedbackfactor cutoff-factor pitchmod-factor))
                      reverb-list (subs-posn reverb-list (arithm-ser 5 512 4) (db->lin -6))
                      reverb-list (subs-posn reverb-list (arithm-ser 6 512 4) (* 0.01 60))
                      reverb-list (subs-posn reverb-list (arithm-ser 7 512 4) 7000)
                      reverb-list (subs-posn reverb-list (arithm-ser 8 512 4) .6)

                      )

                (loop for parameter in params
                      for i = 5 then (+ i 4) do 
                      (setf reverb-list (subs-posn reverb-list i (db->lin (first parameter)))            ;level
                            reverb-list (subs-posn reverb-list (+ i 1) (* 0.01 (second parameter)))      ;feedback
                            reverb-list (subs-posn reverb-list (+ i 2) (third parameter))                ;cutoff
                            reverb-list (subs-posn reverb-list (+ i 3) (fourth parameter)))              ;pitchmod
                      )

              (make-instance 'gen-cs-table
                             :id id
                             :size (* 4 128) 
                             :stime 0 
                             :gen-num -2
                             :param-list reverb-list)
              )

#|
(defmethod! rvbap-reverb ((id number) (cutoff-factor number) (pitchmod-factor number) &key (cf1 10000) (pm1 10) (cf2 10000) (pm2 10) (cf3 10000) (pm3 10) (cf4 10000) (pm4 10))   
                :initvals '(461 1 1 nil) 
                :icon 46
                :doc "defines reverb settings for rvbap"
               ;:menuins '((3 (("bright room" 'bright) ("dark-room" 'dark))))
            (let ((reverb-list '(0 0 1 0 2 0 3 0 4 0 5 0 6 0 7 0 8 1)) cutoff-list pitchmod-list)             
              (setf cutoff-list (x-append (* cutoff-factor cf1) (* cutoff-factor cf2) (* cutoff-factor cf3) (* cutoff-factor cf4))) 
              (setf pitchmod-list (x-append (* pitchmod-factor pm1) (* pitchmod-factor pm2) (* pitchmod-factor pm3) (* pitchmod-factor pm4)))
              (setf reverb-list (subs-posn reverb-list '(1 5 9 13) cutoff-list))
              (setf reverb-list (subs-posn reverb-list '(3 7 11 15) pitchmod-list))
             ; (when (equal presets 'bright)
             ;   (setf reverb-list '(0 11300 1 6 2 10000 3 3 4 9900 5 7 6 11200 7 5 8 1)))
             ; (when (equal presets 'dark)
             ;   (setf reverb-list '(0 4000 1 10 2 3000 3 10 4 2700 5 15 6 3000 7 20 8 1)))
              (make-instance 'gen-cs-table
                             :id id
                             :size 16 :stime 0 :gen-num -27
                             :param-list reverb-list)
              ))
|#

; %%%%%%%%%%%%%%%%% RVBAP GLOBALS %%%%%%%%%%%%

(defun rvbap-globals (speakers)
;2D case ----------------------
            (if (= (length (first speakers)) 2)             
                (progn
                  (setf str " 2")
                  (setf str (concatenate 'string str 
                                         (format nil ", ~D" (length speakers))))
                  (loop for item in speakers do
                        (setf azi (xy->ad (first item) (second item)))
                        (setf str (concatenate 'string str 
                                               (format nil ", ~,3f" azi)))))   
;3D case ---------------------          
                (progn
                  (setf str " 3")
                  (setf str (concatenate 'string str 
                                         (format nil ", ~D" (length speakers))))                
                  (loop for item in speakers do
                        (multiple-value-bind (a e d) (xyz->aed (first item) (second item) (third item))
                          (setf str (concatenate 'string str 
                                                 (format nil ", ~,3f" a)))
                          (setf str (concatenate 'string str 
                                                 (format nil ", ~,3f" e)))))))
            (list (list "vbaplsinit" str)))


; %%%%%%%%%%%%%%%%% RVBAP OPCODES %%%%%%%%%%%%

(defparameter *rvbap-rolloff-discrete*
  (list 
   "rvbap_rolloff_discrete" 
   '("i" "ii")
   "idist, iattenfunction xin
	index = idist*0.02
	irolloff  tablei  index, iattenfunction, 1, 0, 0
	iamp limit (10^(irolloff*0.05)), 0, 1
	xout iamp"))

(defparameter *rvbap-rolloff-continuous*
  (list 
   "rvbap_rolloff_continuous" 
   '("k" "ki")
   "kdist, iattenfunction xin
	kindex = kdist*0.02
	krolloff  tablei  kindex, iattenfunction, 1, 0, 0
	kamp limit (10^(krolloff*0.05)), 0, 1
	xout kamp"))


; %%%%%%%%%%%%%%%%% RVBAP MACROS %%%%%%%%%%%%

(defun rvbap-macros (self)
  (list (copy-source '(t t) 'rvbap) (csound-envelopes '(xpos ypos zpos spread) 'rvbap) (csound-trajectories '(xpos ypos zpos spread) 'rvbap)
; discrete control
  (xyz2aed-discrete '(t)) (attenuation-discrete '(t)) (airabsorption-discrete '(t)) (timeoftravel-discrete '(t))
; continuous control
  (xyz2aed-continuous '(t)) (attenuation-continuous '(t)) (airabsorption-continuous '(t)) (timeoftravel-continuous '(t))
; rvbap-specific
  (rvbap-reverb-panning-discrete self) (rvbap-reverb-panning-continuous self) (rvbap-reverb-params self)
  ))

(defun rvbap-reverb-panning-discrete (speakers)
  (setf rvbapstr
    (cond ((<= (length speakers) 4) 
           (format nil "~1%als1, als2, als3, als4 vbap4 alclrevsound, iazimuth1, ielevation1, kspread"))
          ((<= (length speakers) 8) 
           (format nil "~1%als1, als2, als3, als4, als5, als6, als7, als8 vbap8 alclrevsound, iazimuth1, ielevation1, krevspread"))
          ((> (length speakers) 8) 
           (format nil "~1%als1, als2, als3, als4, als5, als6, als7, als8, als9, als10, als11, als12, als13, als14, als15, als16 vbap16 alclrevsound, iazimuth1, ielevation1, krevspread"))))
  (setf rvbapstr (concatenate 'string rvbapstr (format nil "~1%ars0 = 0"))) ; create a 'dummy signal'
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf rvbapstr (concatenate 'string rvbapstr 
                                    (format nil "~1%ars~D = aglrevsound+als~D" i i))))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf rvbapstr (concatenate 'string rvbapstr 
                                    (format nil "~1%ar~D, ardummy reverbsc (ars~D*irevlevel~D), ars0, (irevfeedback~D*iglbrevfeedbackfactor), (irevcutoff~D*iglbrevcutofffactor), sr, (ipitchmod~D*iglbrevpitchmodfactor), 0" i i i i i i))))
  (setf vbapstr
    (cond ((<= (length speakers) 4) 
           (format nil "~1%a1, a2, a3, a4 vbap4 asound1, iazimuth1, ielevation1, kspread"))
          ((<= (length speakers) 8) 
           (format nil "~1%a1, a2, a3, a4, a5, a6, a7, a8 vbap8 asound1, iazimuth1, ielevation1, krevspread"))
          ((> (length speakers) 8)
           (format nil "~1%a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16 vbap16 asound1, iazimuth1, ielevation1, krevspread"))))
  (setf outstr (format nil "~1%outc a1+ar1"))
  (loop for item in (cdr speakers)
        for i = 2 then (+ i 1) do
        (setf outstr (concatenate 'string outstr (format nil ", a~D+ar~D" i i))))
  (setf rvbapstr (concatenate 'string rvbapstr vbapstr outstr))

  (list "rvbap_reverb_panning_discrete" rvbapstr)
  )

(defun rvbap-reverb-panning-continuous (speakers)
  (setf rvbapstr
    (cond ((<= (length speakers) 4) 
           (format nil "~1%als1, als2, als3, als4 vbap4 alclrevsound, kazimuth1, kelevation1, kspread"))
          ((<= (length speakers) 8) 
           (format nil "~1%als1, als2, als3, als4, als5, als6, als7, als8 vbap8 alclrevsound, kazimuth1, kelevation1, krevspread"))
          ((> (length speakers) 8) 
           (format nil "~1%als1, als2, als3, als4, als5, als6, als7, als8, als9, als10, als11, als12, als13, als14, als15, als16 vbap16 alclrevsound, kazimuth1, kelevation1, krevspread"))))
  (setf rvbapstr (concatenate 'string rvbapstr (format nil "~1%ars0 = 0"))) ; create a 'dummy signal'
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf rvbapstr (concatenate 'string rvbapstr 
                                    (format nil "~1%ars~D = aglrevsound+als~D" i i))))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf rvbapstr (concatenate 'string rvbapstr 
                                    (format nil "~1%ar~D, ardummy reverbsc (ars~D*irevlevel~D), ars0, (irevfeedback~D*iglbrevfeedbackfactor), (irevcutoff~D*iglbrevcutofffactor), sr, (ipitchmod~D*iglbrevpitchmodfactor), 0" i i i i i i))))
  (setf vbapstr
        (cond ((<= (length speakers) 4) 
               (format nil "~1%a1, a2, a3, a4 vbap4 asound1, kazimuth1, kelevation1, kspread"))
              ((<= (length speakers) 8) 
               (format nil "~1%a1, a2, a3, a4, a5, a6, a7, a8 vbap8 asound1, kazimuth1, kelevation1, krevspread"))
              ((> (length speakers) 8)
               (format nil "~1%a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13, a14, a15, a16 vbap16 asound1, kazimuth1, kelevation1, krevspread"))))
  (setf outstr (format nil "~1%outc a1+ar1"))
  (loop for item in (cdr speakers)
        for i = 2 then (+ i 1) do
        (setf outstr (concatenate 'string outstr (format nil ", a~D+ar~D" i i))))
  (setf rvbapstr (concatenate 'string rvbapstr vbapstr outstr))
  (list "rvbap_reverb_panning_continuous" rvbapstr)
  )

(defun rvbap-reverb-params (speakers)
  (let ((macrostr
         (format nil "~1%iglbrevlevel	tab_i 0, ireverbfunction, 0 ~1%iglbrevspread	tab_i 1, ireverbfunction, 0 ~1%iglbrevfeedbackfactor	tab_i 2, ireverbfunction, 0 ~1%iglbrevcutofffactor	tab_i 3, ireverbfunction, 0 ~1%iglbrevpitchmodfactor	tab_i 4, ireverbfunction, 0"
         )))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf macrostr (concatenate 'string macrostr 
                                    (format nil "~1%irevlevel~D tab_i ~D, ireverbfunction, 0 " i (+ (* i 4) 1))))
        (setf macrostr (concatenate 'string macrostr 
                                    (format nil "~1%irevfeedback~D tab_i ~D, ireverbfunction, 0 " i (+ (* i 4) 2))))
        (setf macrostr (concatenate 'string macrostr 
                                    (format nil "~1%irevcutoff~D tab_i ~D, ireverbfunction, 0 " i (+ (* i 4) 3))))
        (setf macrostr (concatenate 'string macrostr 
                                    (format nil "~1%ipitchmod~D tab_i ~D, ireverbfunction, 0 " i (+ (* i 4) 4))))
        )
  (list "rvbap_reverb_params" macrostr)
  ))

(defun rvbap-reverb-params (speakers)
  (let ((macrostr
         (format nil "~1%irevlevel	tab_i 0, ireverbfunction, 0 ~1%irevspread	tab_i 1, ireverbfunction, 0 ~1%irevfeedback	tab_i 2, ireverbfunction, 0"
         )))
  (loop for item in speakers
        for i = 1 then (+ i 1) do 
        (setf macrostr (concatenate 'string macrostr 
                                    (format nil "~1%irevcutoff~D tab_i ~D, ireverbfunction, 0 " i (+ (* i 2) 1))))
        (setf macrostr (concatenate 'string macrostr 
                                    (format nil "~1%ipitchmod~D tab_i ~D, ireverbfunction, 0 " i (+ (* i 2) 2))))
        )
  (list "rvbap_reverb_params" macrostr)
  ))