;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)

(defparameter *spat-default-tables*
  (list
   "; Shoebox room function"
   "f451 0 64 -2 4 48.0 -1.0 0.3 1.0 -1.0 1.0 21.982 0.05 0.9 4000.0 0.6 0.7071 2.0 1.0 1.753 0.05 0.9 3500.0 0.5 0.7071 2.0 1.0  15.22 0.05 0.9 6000.0 0.8 0.7071 2.0 1.0 9.317 0.05 0.9 5000.0 0.8 0.7071 2.0 1.0 17.545 0.05 0.9 5000.0 0.8 0.7071 2.0 1.0 12.156 0.05 0.9 5000.0 0.8 0.7071 2.0 ; spat room parameter - function"
))

; %%%%%%%%%%%%%%%%%%% SPAT SETUP %%%%%%%%%%%%%%%%%%%%

(defmethod! spat-setup ((self t) &optional osc)
            :icon 37 
            :initvals '("2D.3rd-order" t)
            :menuins '((0 (("2D.1st-order" "2D.1st-order")("2D.2nd-order" "2D.2nd-order") ;this shouldn't have different orders anymore
                           ("2D.3rd-order" "2D.3rd-order")("3D.1st-order" "3D.1st-order")
                           ("3D.2nd-order" "3D.2nd-order")("3D.3rd-order" "3D.3rd-order")
                           ("UHJ" "UHJ")
                           )))                       
            :indoc '("select dimensions/order-preset from menu or specify as a string" "when active sends OSC-messages to the Multiplayer")
            :numouts 1
            :doc "sets encoding order and dimensionality (optionally synchronizes Multiplayer)"
            (when (stringp self)
              (setf params; (channels dimensions order)
                     (cond ((equal self "2D.1st-order") '(3 2 1))
                           ((equal self "2D.2nd-order") '(5 2 2))
                           ((equal self "2D.3rd-order") '(7 2 3))
                           ((equal self "3D.1st-order") '(4 3 1))
                           ((equal self "3D.2nd-order") '(9 3 2))
                           ((equal self "3D.3rd-order") '(16 3 3))
                           ((equal self "UHJ") '(2 0 2))
                           )))
            (when (listp self)
              (setf params (deduce-encoding-params self)))
            (print params)
            (setf channels (first params)
                  dimensions (second params)
                  order (third params))

;;OSC MultiPlayer communication----
                (when osc 
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/dimensions" dimensions))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/dimensions" dimensions))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/order" order))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/order" order))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/audio/bypass" 0))
                )
                    (if (equal self "UHJ")
                        (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/format" "UHJ"))
                      (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/format" "Bformat"))
                      )
;cs-inits----------------------------
                    (list
                     (cs-nchnls channels)
                     (cs-globals (spat-globals dimensions order))
                     (cs-opcodes (list *envelope* *trajectory*
                                       *xyz2aed-discrete* *attenuation-discrete* *airabsorption-discrete*
                                       *xyz2aed-continuous* *attenuation-continuous* *airabsorption-continuous*))
                     (cs-macros (spat-macros '(t)))
                     )
                    )

(defmethod! spat-setup ((self BPC) &optional osc)
            (spat-setup (point-pairs self) osc)
            )

(defmethod! spat-setup ((self 3DC) &optional osc)
            (spat-setup (point-pairs self) osc)
            )

; %%%%%%%%%%%%%%%%%%% SPAT GLOBALS %%%%%%%%%%%%%%%%%%%%

(defun spat-globals (dims order)
  (list
  (list 
   (format nil "gidims") 
   (format nil "= ~D" dims))
  (list
   (format nil "giorder") 
   (format nil "= ~D" order))
    ))


; %%%%%%%%%%%%%%%%%%% SPAT MACROS %%%%%%%%%%%%%%%%%%%%

(defun spat-macros (self)
  (list (copy-source '(t) 'spat) (csound-envelopes '(xpos ypos zpos order) 'spat) (csound-trajectories '(xpos ypos zpos order) 'spat)
; discrete control
  (xyz2aed-discrete '(t)) (attenuation-discrete '(t)) (airabsorption-discrete '(t))
; continuous control
  (xyz2aed-continuous '(t)) (attenuation-continuous '(t)) (airabsorption-continuous '(t)))
  )


;;; SPAT - ROOM FUNCTION %%%%%%%%%%%

(defparameter *room-ceiling-position* '(6 7 8 9 10 11 12 13))
(defparameter *room-floor-position* '(14 15 16 17 18 19 20 21))
(defparameter *room-front-position* '(22 23 24 25 26 27 28 29))
(defparameter *room-back-position* '(30 31 32 33 34 35 36 37))
(defparameter *room-right-position* '(38 39 40 41 42 43 44 45))
(defparameter *room-left-position* '(46 47 48 49 50 51 52 53))
(defvar *room-list* '(0 1 2 3 1 -1 1 21.982 0.05 0.9 4000 0.6 0.7071 2 1 1.753 0.05 0.9 4000 0.6 0.7071 
2 1 15.22 0.05 0.9 4000 0.6 0.7071 2 1 9.317 0.05 0.9 4000 0.6 0.7071 2 1 17.545 0.05 0.9 4000 0.6 0.7071 
2 1 12.156 0.05 0.9 4000 0.6 0.7071 2))

;----calculate imdel for room-model-----
(defun imdel (recursions width height depth)
  (* (+ recursions 1) (/ (sqrt (+ (* width width) (* height height) (* depth depth))) 340)))
;; is this really used inside the function for table-generation (spat-room)?
;; should be implemented!

(defmethod! spat-wall (distance reflection-level random &optional (eq-mode 2) (eq-freq 4000) (eq-level 0.6) (eq-q 0.7))
   :initvals '(12 0.8 0.05 2 4000 0.6 0.7) 
   :icon 52
   :menuins '((3 (("peak" 0) ("low shelf" 1) ("high shelf" 2))))
   :doc "defines wall properties for shoebox room-model"
    (list distance random (om-scale reflection-level -1 1 0 1) eq-freq eq-level eq-q eq-mode))


(defmethod! spat-room (id recursion-depth center-size dimensions &key front back left right floor ceiling)
    :initvals '("?" 7 1 '2d nil nil nil nil nil nil) 
    :icon 51
    :doc "defines room-parameters for shoebox room-model"
  :menuins '((3 (("3D" '3d) ("2D" '2d))))
  (let ((room-list (copy-list *room-list*))
        (dim (if (equal dimensions '3d) 1 0)))
     ;; early reflections
     (setf (nth 0 room-list) recursion-depth)
     (setf (nth 2 room-list) -1) ; imdelay
     (setf (nth 3 room-list) -1) ; irlen
     (setf (nth 4 room-list) center-size)     
     (setf (nth 5 room-list) -1)
     (when front 
       (setf room-list (subs-posn room-list *room-front-position* (cons 1 front))))
     (when back 
       (setf room-list (subs-posn room-list *room-back-position* (cons 1 back))))
     (when left 
       (setf room-list (subs-posn room-list *room-left-position* (cons 1 left))))
     (when right 
       (setf room-list (subs-posn room-list *room-right-position* (cons 1 right))))
     (when floor 
       (setf room-list (subs-posn room-list *room-floor-position* (cons dim floor))))
     (when ceiling 
       (setf room-list (subs-posn room-list *room-ceiling-position* (cons dim ceiling))))
     
     (make-instance 'gen-cs-table
                    :id id
                    :size 64 :stime 0 :gen-num -2
                    :param-list room-list)
     ))