;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)


; try to return the csound-path sth like (find-csound-orc-path / (get-orc
; then write the path into a variable 
; make a function that returns the path of a csound-table
; 

(defparameter  *vbap-default-tables*
  (list
   "; LOOKUP-tables *****"
   "; "
   "; Distance functions"
   "f200 0 513 -7 0 512; no distance attenuation"
   "f201 0 513 -27 0 -6  10 0  512 -75 ; appr. 1.5 db/m exponential rolloff as function of distance"
   "f202 0 32769 -6 -6  640 0  10000 -30 22129 -36 ; approx 6 db/doubledistance inverse square-law rolloff"
   "; "
   "; Airabsorption functions"
   "f301 0 32769 -6 20048 640 20048 10284 4000 10922 3000 10923 2000 ; approximation of D.Valente's air-absorption curve"
   "f302 0 32769 -25 0 20048 640 20048 32768 2000 ; exponential frequency rolloff as function of distance (100000/distance)"
   "; "
   "; Time functions"
   "f401 0 4097 -7 0 4096 0.1470588 ; linear function "
   "; "))


; %%%%%%%%%%%%%%%%% VBAP SETUP %%%%%%%%%%%%%%%%%%%

(defmethod! vbap-setup ((self list) &optional osc)
;; ToDo: if it's a list of atoms the values must be azimuth!
            :icon 15  
            :initvals '(nil t)
            :indoc '("specify loudspeaker configurations using a bpc, 3DC or list of x-y-tuplets (or x-y-z-triplets)"  
                     "when active synchronizes Multiplayer settings")
            :numouts 1
            :doc "describes a speaker setup for vbap-classes. connect to :inits of synthesize method"

;;OSC MultiPlayer communication----
            (when osc
              (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/audio/bypass" 1))
              (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/audio/bypass" 1))
              )
;;cs-inits-------------------------
            (list
             (cs-nchnls (length self))
             (cs-globals (vbap-globals self))
             (cs-opcodes (list *envelope* *trajectory*
                               *xyz2aed-discrete* *attenuation-discrete* *airabsorption-discrete* *timeoftravel-discrete*
                               *xyz2aed-continuous* *attenuation-continuous* *airabsorption-continuous* *timeoftravel-continuous*))
             (cs-macros (vbap-macros '(t)))
                     ))
            
(defmethod! vbap-setup ((self BPC) &optional osc)
            :numouts 1
            (vbap-setup (point-pairs self) osc)
            )

(defmethod! vbap-setup ((self 3DC) &optional osc)
            :numouts 1
            (vbap-setup (point-pairs self) osc)
            )


; %%%%%%%%%%%%%%%%% VBAP GLOBALS %%%%%%%%%%%%%%%%%%%

(defun vbap-globals (speakers)
;2D case ----------------------
            (if (= (length (first speakers)) 2)             
                (progn
                  (setf str " 2")
                  (setf str (concatenate 'string str 
                                         (format nil ", ~D" (length speakers))))
                  (loop for item in speakers do
                        (setf azi (xy->ad (first item) (second item)))
                        (setf str (concatenate 'string str 
                                               (format nil ", ~,3f" azi)))))   
;3D case ---------------------          
                (progn
                  (setf str " 3")
                  (setf str (concatenate 'string str 
                                         (format nil ", ~D" (length speakers))))                
                  (loop for item in speakers do
                        (multiple-value-bind (a e d) (xyz->aed (first item) (second item) (third item))
                          (setf str (concatenate 'string str 
                                                 (format nil ", ~,3f" a)))
                          (setf str (concatenate 'string str 
                                                 (format nil ", ~,3f" e)))))))
            (list (list "vbaplsinit" str)))


; %%%%%%%%%%%%%%%%% VBAP MACROS %%%%%%%%%%%%%%%%%%%

(defun vbap-macros (self)
  (list (copy-source '(t) 'vbap) (csound-envelopes '(xpos ypos zpos spread) 'vbap) (csound-trajectories '(xpos ypos zpos spread) 'vbap)
; discrete control
  (xyz2aed-discrete '(t)) (attenuation-discrete '(t)) (airabsorption-discrete '(t)) (timeoftravel-discrete '(t))
; continuous control
  (xyz2aed-continuous '(t)) (attenuation-continuous '(t)) (airabsorption-continuous '(t)) (timeoftravel-continuous '(t))
  ))

