;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)

; %%%%%%%%%%%% mic-array %%%%%%%%%%%%%%

(defclass! mic-array ()
  ((nmics :accessor nmics :initarg :nmics :initform 1)
   (micpatterns :accessor micpatterns :initarg :micpatterns :initform 0)
   (micmodes :accessor micmodes :initarg :micmodes :initform nil)
   (micyaws :accessor micyaws :initarg :micyaws :initform 0)
   (micgains :accessor micgains :initarg :micgains :initform 1)
   (micexps :accessor micexps :initarg :micexps :initform 1)   
   (bpcs :accessor bpcs :initform nil))
  (:doc "N/A")
  (:icon 76))

(defmethod initialize-instance :after ((self mic-array) &rest args)
  (unless (and (numberp (nmics self)) (>= (nmics self) 1))
    (om-beep-msg "MIC-ARRAY MUST CONTAIN AT LEAST 1 MIC")
    (setf (nmics self) 1))
  (loop for slot in '(micpatterns micyaws micgains micexps micmodes)
        for defval in '(0 0 1 1 nil) do
        (unless (and (consp (slot-value self slot)) (= (length (slot-value self slot)) (nmics self)))
          (setf (slot-value self slot) 
                (loop for i from 0 to (1- (nmics self)) collect
                     (or (if (consp (slot-value self slot))
                             (or (nth i (slot-value self slot)) (car (last (slot-value self slot))))
                           (slot-value self slot))
                         defval)))))
  (setf (bpcs self) (gen-bpcs self))
  )

(defmethod get-fonde-pict ((self mic-array)) *my-bg-pict*)

(defmethod gen-bpcs ((self mic-array))
  (loop for i from 0 to (1- (nmics self)) collect
        (ad->xy (microphone (nth i (micpatterns self)) 
                    (nth i (micyaws self))
                    (nth i (micgains self))
                    :exponent (nth i (micexps self))
                    :mode (if (nth i (micmodes self)) 2 1)
                    ) nil)))

(defmethod gen-bpfs ((self mic-array))
  (loop for i from 0 to (1- (nmics self)) collect
         (microphone (nth i (micpatterns self)) 
                    (nth i (micyaws self))
                    (nth i (micgains self))
                    :exponent (nth i (micexps self))
                    :mode (if (nth i (micmodes self)) 2 1)
                    )))

(defmethod objfromobjs ((self mic-array) (type BPC-Lib))
  (setf (bpf-list type) (gen-bpcs self))
  type)

;new method for BPFs
(defmethod objfromobjs ((self mic-array) (type BPF-Lib))
  (setf (bpf-list type) (gen-bpfs self))
  type)

(defmethod draw-obj-in-rect ((self mic-array) x x1 y y1 edparams view)
  (let ((pw (round (- x1 x) (nmics self))))
    (loop for i from 0 to (1- (nmics self)) do
          (om-draw-line (* i pw) y (* i pw) y1)
          (draw-obj-in-rect (nth i (bpcs self)) (+ 2 (* i pw)) (- (* (1+ i) pw) 3) (+ 2 y) (- y1 4) edparams view))))

;;; EDITOR

(defmethod class-has-editor-p ((self mic-array)) t)
(defmethod get-editor-class ((self mic-array)) 'mic-array-editor)

(defparameter mic-trackw 130)

(defmethod get-win-ed-size ((self mic-array)) 
  (om-make-point (+ 2 (min (* (+ mic-trackw 2) 16) (* (+ mic-trackw 2) (nmics self)))) 180))

(defclass mic-array-editor (EditorView) 
  ((mic-panels :initform nil :accessor mic-panels :type list)))

(defclass mic-params-panel (om-view) 
  ((mic-array :accessor mic-array :initarg :mic-array :initform nil)
   (micid :accessor micid :initarg :micid :initform 0)))

(defmethod initialize-instance :after ((self mic-array-editor) &rest l)
   (declare (ignore l))
   (om-set-bg-color self *om-dark-gray-color*)
   (initialize-panels self))

(defmethod initialize-panels ((self mic-array-editor))
  (om-with-delayed-redraw self
    (setf (mic-panels self)
          (loop for i from 0 to (1- (nmics (object self))) collect
                (om-make-view 'mic-params-panel
                              :micid i
                              :mic-array (object self)
                              :bg-color *om-light-gray-color*
                              :position (om-make-point (+ 2 (* i (+ mic-trackw 2))) 2) 
                              :size (om-make-point mic-trackw (- (h self) 4)))))
    
    (apply 'om-add-subviews (cons self (mic-panels self)))
    (let ((size (get-win-ed-size (object self))))
      (oa::set-not-resizable (window self) (om-point-h size) (om-point-v size))) 
    ))

(defmethod update-editor-after-eval ((self mic-array-editor) val)
  (setf (object self) val)
  (initialize-panels self))

(defmethod initialize-instance :after ((self mic-params-panel) &rest l)
  (let ((pos 0)
        (mic-patterns '(("omni" 1)
                        ("sub-cardioid 1" 0.75)
                        ("sub-cardioid 2" 0.63) 
                        ("cardioid" 0.5) 
                        ("super-cardioid" 0.366) 
                        ("hyper-cardioid" 0.25) 
                        ("figure-of-eight" 0)))
        yawtxt gaintxt)
    (setf pos (+ pos 5))
    (om-add-subviews self (om-make-dialog-item 'om-pop-up-dialog-item 
                               (om-make-point 2 pos) 
                               (om-make-point (- mic-trackw 4) 12)
                               ""
                               :di-action (om-dialog-item-act item
                                            (setf (nth (micid self) (micpatterns (mic-array self)))
                                                  (second (nth (om-get-selected-item-index item) mic-patterns)))
                                            (setf (bpcs (mic-array self)) (gen-bpcs (mic-array self)))
                                            (report-modifications (om-view-container self)))
                               :font *om-default-font1*
                               :range (mapcar 'car mic-patterns)
                               :value (car (find (nth (micid self) (micpatterns (mic-array self))) mic-patterns :key 'cadr :test '=))
                               ))
    (setf pos (+ pos 30))

    (om-add-subviews self (om-make-view 'graphic-numbox 
                                        :position (om-make-point 70 (+ pos 5)) 
                                        :size (om-make-point 28 28) 
                                        :pict (om-load-and-store-picture "dial" 'di)
                                        :nbpict 65
                                        :pict-size (om-make-point 24 24)
                                        :di-action (om-dialog-item-act item
                                                      (om-set-dialog-item-text yawtxt (string+ "yaw " (number-to-string (value item))))
                                                      (setf (nth (micid self) (micyaws (mic-array self))) (value item))
                                                      (setf (bpcs (mic-array self)) (gen-bpcs (mic-array self)))
                                                      (report-modifications (om-view-container self)))
                                        :font *om-default-font2*
                                        :value (nth (micid self) (micyaws (mic-array self)))
                                        :min-val -180
                                        :max-val 180))
    
   
    (om-add-subviews self (setf yawtxt (om-make-dialog-item 'om-static-text
                                                            (om-make-point 70 (+ pos 32)) 
                                                            (om-make-point 65 16)
                                                            (string+ "yaw " (number-to-string (nth (micid self) (micyaws (mic-array self)))))
                                                            :font *om-default-font1*
                                                            )))
    
    
    ;(setf pos (+ pos 30))
    (om-add-subviews self (om-make-dialog-item 'om-slider  
                                               (om-make-point 20 pos) 
                                               (om-make-point 30 85) ""
                                               :di-action (om-dialog-item-act item
                                                            (om-set-dialog-item-text gaintxt 
                                                                                      (format nil "gain ~3f" (* (om-slider-value item) 0.01)))
                                                            (setf (nth (micid self) (micgains (mic-array self))) (* (om-slider-value item) 0.01))
                                                            (setf (bpcs (mic-array self)) (gen-bpcs (mic-array self)))
                                                            (report-modifications (om-view-container self)))
                                               :increment 1
                                               :range '(0 100)
                                               :value (round (* (nth (micid self) (micgains (mic-array self))) 100))
                                               :direction :vertical
                                               :tick-side :none
                                               ))

        (om-add-subviews self (setf gaintxt (om-make-dialog-item 'om-static-text
                                             (om-make-point 70 (+ pos 60)) 
                                             (om-make-point 60 16)
                                             (format nil "gain ~3f" (nth (micid self) (micgains (mic-array self))))
                                             :font *om-default-font1*
                                             )))
    
    (setf pos (+ pos 86))
    (om-add-subviews self (om-make-dialog-item 'om-static-text
                                              (om-make-point 20 pos) 
                                              (om-make-point 40 16)
                                              "Exp."
                                              :font *om-default-font1*
                                              ))
    
    (om-add-subviews self (om-make-dialog-item 'numbox
                                               (om-make-point 70 pos) 
                                               (om-make-point 36 18) 
                                               (format nil " ~2f" (nth (micid self) (micexps (mic-array self))))
                                               :value (nth (micid self) (micexps (mic-array self)))
                                               :font *om-default-font1*
                                               :incr 0.01
                                               :min-val 0.0 :max-val 2.0
                                               :afterfun #'(lambda (box) 
                                                             (setf (nth (micid self) (micexps (mic-array self))) (value box))
                                                             (setf (bpcs (mic-array self)) (gen-bpcs (mic-array self)))
                                                             (report-modifications (om-view-container self)))
                                               ))
    
    (setf pos (+ pos 22))

    (om-add-subviews self (om-make-dialog-item 'om-check-box
                                               (om-make-point 14 pos) 
                                               (om-make-point 110 10) 
                                               "clip neg. phase"
                                               :font *om-default-font1*
                                               :checked-p (nth (micid self) (micmodes (mic-array self)))
                                               :di-action (om-dialog-item-act item 
                                                            (setf (nth (micid self) (micmodes (mic-array self))) (om-checked-p item))
                                                             (setf (bpcs (mic-array self)) (gen-bpcs (mic-array self)))
                                                             (report-modifications (om-view-container self))
                                                             )))

    ))
    


; %%%%%%%%%%%%%%%%% GET MIC PARAMS %%%%%%%%%%%%%%

(defmethod! speakers->mics ((self list))
            (let* ((theyaws (concentric-yaw self))
                  (themic-array (make-instance 'mic-array
                                               :nmics (length self)
                                               :micpatterns 0.5
                                               :micyaws theyaws
                                               :micgains 1.0)))
              themic-array))

(defmethod! speakers->mics ((self 3dc))
            (speakers->mics (point-pairs self))
            )

(defmethod! speakers->mics ((self bpc))
            (speakers->mics (point-pairs self))
            )


(defmethod objfromobjs ((self 3dc) (type mic-array))
  (let* ((theyaws (concentric-yaw self))
         (new (make-instance 'mic-array
               :nmics (length theyaws)
               :micpatterns 0.5
               :micyaws theyaws
               :micgains 1.0
               )))
    new))

(defmethod objfromobjs ((self bpc) (type mic-array))
  (let* ((theyaws (concentric-yaw self))
         (new (make-instance 'mic-array
               :nmics (length theyaws)
               :micpatterns 0.5
               :micyaws theyaws
               :micgains 1.0
               )))
    new))

(defmethod! concentric-yaw ((self list)) ;a list of point-pairs
            (let* ((thetranslist (mat-trans self))
                  (theyaws (xy->ad (first thetranslist) (second thetranslist))))
              theyaws))

(defmethod! concentric-yaw ((self 3dc))
            (concentric-yaw (point-pairs self)))

(defmethod! concentric-yaw ((self bpc))
            (concentric-yaw (point-pairs self)))


; -----------Microphone Patterns-------------------

; this should have the check for positive even integer for exponent if mode is 'natural'

(defmethod! microphone ((pattern number) (yaw number) (gain number) &key (mode 2) (exponent 1) (resolution 1))
            :icon 80 
            :initvals '(0 0 1 1 1 1)
            :indoc '("pattern" "yaw" "gain" "exponent" "resolution")
            :menuins '((0 (("omni" 1)("sub-cardioid 1" 0.75) ("sub-cardioid 2" 0.63) ("cardioid" 0.5) 
                           ("super-cardioid" 0.366) ("hyper-cardioid" 0.25) ("figure-of-eight" 0)))
                       (3 (("natural" 1) ("absolute" 2)("clipped" 3))))
            :doc "Specify microphone directivities for ViMiC (Virtual Microphone Control)."
            (when (< exponent 0) (progn
                                   (setf exponent 1)
                                   (om-beep-msg "Exponent cannot be negative. Setting to 1.")))
            (if (and (equal mode 1) (floatp exponent))
                (progn
                  (setf exponent (om-round exponent))
                  (om-beep-msg (format nil "For natural mode exponent must be a positive integer.
Setting to ~d." (om-round exponent)))))
            (let* (
                  (myresolution (deg->rad resolution))
                  (myseries (arithm-ser 0 (* 2 pi) myresolution))
                  (xpoints (second (multiple-value-list (om// (om+ yaw (rad->deg myseries)) 360))))
                  (ypoints (cond ((equal mode 1) (om* (om^ (om+ pattern (om* (- 1 pattern) (om-cos myseries))) exponent) gain))
                                 ((equal mode 2) (om* (om^ (om-abs (om+ pattern (om* (- 1 pattern) (om-cos myseries)))) exponent) gain))
                                 ((equal mode 3) (om* (om^ (om-clip (om+ pattern (om* (- 1 pattern) (om-cos myseries))) 0 10.0) exponent) gain))))
                  (sortedpointlist (sort-list (mat-trans (list xpoints ypoints)) :test '< :key 'first))
                  (finalxpoints (append xpoints '(0 360)))
                  (finalypoints (append ypoints (x-transfer sortedpointlist '(0 0))))
                  (scaledypoints (om/ finalypoints (list-max finalypoints))))
              (simple-bpf-from-list finalxpoints scaledypoints 'bpf 20))
              )


;; for compatibility

(defmethod! mic-pattern ((pattern number) (yaw number) (gain number) &key (mode 2) (exponent 1) (resolution 1))
            (microphone pattern yaw gain :mode mode :exponent exponent :resolution resolution))

;; %%%%%%%%%% SOURCE/MIC DIRECTIVITIES %%%%%%%%%%%%%%%

; convert polar plot as BPC (XY, cartesian coordinates) into BPF (x=angle, y=magnitude)

(defmethod! gen-directivity ((self bpc) &optional (samples 360))
            :icon 77
            :initvals '(nil 360)
            (let* ((theadbpc (xy->ad (om-sample (close-bpc self) samples) nil))
                   (xpoints (second (multiple-value-list (om// (x-points theadbpc) 360))))
                   (ypoints (y-points theadbpc))
                   (sortedpointlist (sort-list (mat-trans (list xpoints ypoints)) :test '< :key 'first))
                   (finalxpoints (append xpoints '(0 360)))
                   (finalypoints (append ypoints (x-transfer sortedpointlist '(0 0))))
                   (scaledypoints (om/ finalypoints (list-max finalypoints))))
              (simple-bpf-from-list finalxpoints scaledypoints 'bpf 20))
              )

(defmethod! gen-directivity ((self bpc-lib) &optional (samples 360))
            (let ((bpclist (bpf-list self)))
              (mapcar #'(lambda (thebpc) (gen-directivity thebpc samples)) 
                      bpclist)
              ))

(defmethod! gen-directivity ((self list) &optional (samples 360))
            (mapcar #'(lambda (thebpc) (gen-directivity thebpc samples)) 
                    self)
            )

(defmethod! gen-directivity ((self mic-array) &optional (samples 360))
            (gen-directivity (objfromobjs self (make-instance 'BPC-lib)) samples))

(defmethod! source-directivity ((self bpc) &optional (samples 360))
            :icon 79
            :initvals '(nil 360)
            (gen-directivity self samples))

(defmethod! bpc->directivity ((self bpc) &optional (samples 360))
            :icon 79
            :initvals '(nil 360)
            (gen-directivity self samples))

(defmethod! mic-directivities ((self t) &optional (samples 360))
            :icon 79
            (gen-directivity self samples))

(defmethod! directivity->bpc ((self bpf) &optional (samples 360))
            (let ((thexybpc (ad->xy self)))
              ))
                  
; temporary conversion function here ... should be added to 3D-tools.lisp
(defmethod! ad->xy ((a bpf) (d list))
            (if (consp d)
                (multiple-value-bind (x y) (ad->xy (y-points a) d)
                  (values (simple-bpf-from-list (x-points a) x 'bpf (decimals a))
                          y))
              (multiple-value-bind (x y) (ad->xy (x-points a) (y-points a))
                (values (simple-bpf-from-list x y 'bpc (decimals a))
                nil)))
            )

; %%%%%%%% visualization %%%%%%%%%%%

; this needs a better polymorphism

(defmethod! display-vimic ((mic-positions 3dc) (microphones list) &optional trajectories)
            :icon 79
            (let* ((pointlist (loop for mic in microphones collect
                                    (multiple-value-list (ad->xy (x-points mic) (y-points mic)))))
                   (mic-patterns  (loop for points in pointlist collect
                                        (3dc-from-list (first points) (second points) '(0 0) '3DC (decimals (car microphones)))
                                                       ))
                   (thesetup  (loop for pattern in mic-patterns
                                    for xpos in (x-points mic-positions)
                                    for ypos in (y-points mic-positions)
                                    for zpos in (z-points mic-positions)
                                    collect (traj-translate pattern :x xpos :y ypos :z zpos)
                                    )))
             ;(print pointlist)
             ;(print mic-patterns)
              (if trajectories
                  (x-append (bpf-colour thesetup) trajectories)
                (bpf-colour thesetup))
              ))


(defmethod! display-vimic ((mic-positions bpc) (microphones list) &optional trajectories)
            :icon 79
            (let* ((mic-patterns (loop for mic in microphones collect
                                       (ad->xy mic nil)))
                   (thesetup  (loop for pattern in mic-patterns
                                    for xpos in (x-points mic-positions)
                                    for ypos in (y-points mic-positions)
                                    collect (traj-translate pattern :x xpos :y ypos)
                                    )))
             ; (print mic-patterns)
              (if trajectories
                  (x-append (bpf-colour thesetup) trajectories)
                (bpf-colour thesetup))
              ))

(defmethod! display-vimic ((mic-positions list) (microphones list) &optional trajectories)
            (let* ((translist (mat-trans mic-positions))
                   (xpoints (first translist))
                   (ypoints (second translist))
                   (mic-patterns (loop for mic in microphones collect
                                       (ad->xy mic nil)))
                   (thesetup  (loop for pattern in mic-patterns
                                    for xpos in xpoints
                                    for ypos in ypoints
                                    collect (traj-translate pattern :x xpos :y ypos)
                                    )))
              (if trajectories
                  (x-append (bpf-colour thesetup) trajectories)
                (bpf-colour thesetup))
              ))

(defmethod! display-vimic ((mic-positions bpc) (microphones mic-array) &optional trajectories)
            :icon 77
            (let ((thepatterns (bpcs microphones)))
            (setf thesetup 
                  (loop for pattern in thepatterns
                  for xpos in (x-points mic-positions) 
                  for ypos in (y-points mic-positions) 
                  collect
                  (traj-translate pattern :x xpos :y ypos)
                  )))
            (if trajectories
            (x-append (bpf-colour thesetup) trajectories)
              (bpf-colour thesetup))
            )

(defmethod! display-vimic ((mic-positions bpc) (microphones bpc-lib) &optional trajectories)
            :icon 77
            (let ((thepatterns (bpf-list microphones)))
            (setf thesetup 
                  (loop for pattern in thepatterns
                  for xpos in (x-points mic-positions) 
                  for ypos in (y-points mic-positions) 
                  collect
                  (traj-translate pattern :x xpos :y ypos)
                  )))
            (if trajectories
            (x-append (bpf-colour thesetup) trajectories)
              (bpf-colour thesetup))
            )

(defmethod! display-vimic ((mic-positions bpc) (microphones bpf-lib) &optional trajectories)
            :icon 77
            (let ((thebpfs (bpf-list microphones)))
              (display-vimic mic-positions thebpfs))
            )

;special case... the second bpc is only to call another function to make a mic-array out of it
(defmethod! display-vimic ((mic-positions bpc) (microphones bpc) &optional trajectories)
            :icon 77
            (display-vimic mic-positions (speakers->mics microphones))
            )


;======================= CSOUND RELATED ===========================

(defparameter *vimic-default-tables*
  (list
   "; LOOKUP-tables *****"
   "; "
   "; Distance functions"
   "f200 0 513 -7 0 512; no distance attenuation"
   "f201 0 513 -27 0 -6  10 0  512 -75 ; appr. 1.5 db/m exponential rolloff as function of distance"
   "f202 0 32769 -6 -6  640 0  10000 -30 22129 -36 ; approx 6 db/doubledistance inverse square-law rolloff"
   "; "
   "; Airabsorption functions"
   "f301 0 32769 -6 20048 640 20048 10284 4000 10922 3000 10923 2000 ; approximation of D.Valente's air-absorption curve"
   "f302 0 32769 -25 0 20048 640 20048 32768 2000 ; exponential frequency rolloff as function of distance (100000/distance)"
   "; "
   "; Time-of-travel functions"
   "f401 0 4097 -7 0 4096 0.1470588 ; linear function "
   "; "
   "; Microphone directivities / Source directivity (omnidirectional)"
   "f601 0 65537 -7 1 65536 1"
   "; "))


; %%%%%%%%%%%%%%%%%%% VIMIC SETUP %%%%%%%%%%%%%%%%%%%%

(defmethod! vimic-setup ((self list) &optional osc)
            :icon 79  
            :initvals '(nil t)
            :indoc '("specify loudspeaker configurations via bpc, 3DC or list of x-y-tuplets / x-y-z-triplets"  "when active synchronizes Multiplayer settings")
            :numouts 1
            :doc "describes speaker setups, opcodes and macros for vimic-classes."
            (setf channels (length self))
            ;(setf dims (length (first self)))
            ;send Open Sound Control messages -> this might later be controlled by the class 'speaker-setup'
            (when osc
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/ambisonic-decoder/audio/bypass" 1))
                (om-send-osc-message *multiplayer-out-port* *multiplayer-host*  (list "/soundfield-manipulator/audio/bypass" 1))
                )                    
            (list (cs-nchnls channels)
                  (cs-globals (vimic-globals self))
                  (cs-opcodes (list *envelope* *trajectory* *xyz2aed-discrete* *xyz2aed-continuous*
                                    *vimic-directivities-discrete* *attenuation-discrete* *airabsorption-discrete* *timeoftravel-discrete* 
                                    *vimic-directivities-continuous* *attenuation-continuous* *airabsorption-continuous* *timeoftravel-continuous*))
                  (cs-macros (vimic-macros self))
                  ))

(defmethod! vimic-setup ((self BPC) &optional osc)
            (vimic-setup (point-pairs self) osc)
            )

(defmethod! vimic-setup ((self 3DC) &optional osc)
            (vimic-setup (point-pairs self) osc)
            )

; %%%%%%%%%%%%%%%%% VIMIC GLOBALS %%%%%%%%%%%%%%%%%%%

(defun vimic-globals (speakers)
  (let ((speakerlist))
    (setf speakerlist
        (loop for speaker in speakers 
                        for i = 1 then (+ i 1) collect
                        (gblspeaker i speaker)))
    (flat speakerlist 1)
    ))

; %%%%%%%%%%%%%%%%% VIMIC OPCODES %%%%%%%%%%%%%%%%%%%

;-> maybe later use tab_i for faster table-lookup

(defparameter *vimic-directivities-discrete*
  (list 
   "vimic_directivities_discrete" 
   '("a" "aiiii")
   "asound, isource_azimuth, isource_yaw, isource_directfun, imic_directfun xin
	isource_azimuth = (isource_azimuth+360.) % 360.
	isource_lookup = (-1*(180.0-isource_azimuth))+(360.0-isource_yaw)
	isource_lookup = (isource_lookup % 360) * 0.002777777777778 ; = /360.
	imic_lookup = (isource_azimuth % 360) * 0.002777777777778 ; = /360.
	isource_fact tablei  isource_lookup, isource_directfun, 1, 0, 1
	imic_fact tablei  imic_lookup, imic_directfun, 1, 0, 1
	amicsound = asound * isource_fact * imic_fact
	xout amicsound"))

;;; needs to be done still...

(defparameter *vimic-directivities-continuous*
  (list 
   "vimic_directivities_continuous" 
   '("a" "akkii")
   "asound, ksource_azimuth, ksource_yaw, isource_directfun, imic_directfun xin
	ksource_azimuth = (ksource_azimuth+360.) % 360.
	ksource_lookup = (-1*(180.0-ksource_azimuth))+(360.0-ksource_yaw)
	ksource_lookup = (ksource_lookup % 360) * 0.002777777777778
	kmic_lookup = (ksource_azimuth % 360) * 0.002777777777778 ; = /360.
	ksource_fact tablei  ksource_lookup, isource_directfun, 1, 0, 1
	kmic_fact tablei  kmic_lookup, imic_directfun, 1, 0, 1
	amicsound = asound * ksource_fact * kmic_fact
	xout amicsound"))

#|
"asound, iyaw, idirectfunction, ispknumber, iazimuth, imicparams xin
	iazimuth = (iazimuth+360.) % 360.
		print iazimuth
	idirectlookup = (-1*(180.0-iazimuth))+(360.0-iyaw)
		print idirectlookup
	idirectlookup = (idirectlookup % 360) * 0.002777777777778 ; = /360.
		print idirectlookup
	ipatternlookup = iazimuth+((ispknumber-1)*361.)
		print ipatternlookup
	idirectfact tablei  idirectlookup, idirectfunction, 1, 0, 1	;allow wraparound saves modulo computation
	ipatternfact tablei  ipatternlookup, imicparams, 0, 0, 0 ; maybe later tab_i if it interpolates
		print idirectfact
		print ipatternfact
	asound = asound * idirectfact * ipatternfact
	xout asound"
|#

; %%%%%%%%%%%%%%%%% VIMIC MACROS %%%%%%%%%%%%%%%%%%%

(defun vimic-macros (speakers)
  (list (copy-source speakers 'vimic)
; set p-fields
   (mic-pfields-discrete speakers 17) (mic-pfields-continuous speakers 21) (mic-pfields-trajectory speakers 17)
; envelopes
   (csound-envelopes '(xpos ypos zpos yaw) 'vimic)  (csound-trajectories '(xpos ypos zpos yaw) 'vimic)
; discrete control
   (attenuation-discrete speakers) (airabsorption-discrete speakers) (timeoftravel-discrete speakers)
; continuous control
   (attenuation-continuous speakers) (airabsorption-continuous speakers) (timeoftravel-continuous speakers)
; renderer-specific
  (vimic-vectors-discrete speakers) (vimic-directivities-discrete speakers) 
  (vimic-vectors-continuous speakers) (vimic-directivities-continuous speakers)  
))

;make a function called: mic-setup to convert a list of bpcs into a mic-setup table. (maybe offer a number of presets? or make a number of BPC-libs as presets ->maybe the function should be called mic-setup (the one containing presets))


(defun mic-pfields-discrete (speakers p-field)
  (let ((thestring)
        (p-field (- p-field 1)))
        (loop for item in speakers 
              for i = 1 then (+ i 1) do 
              (setf thestring (concatenate 'string thestring 
                                           (format nil "~1%imicdirectfun~D = (p~D = 0 ? 601 : p~D)" i (+ p-field i) (+ p-field i)))))
      (list "mic_pfields_discrete" thestring)
      ))



(defun mic-pfields-continuous (speakers p-field)
  (let ((thestring)
        (p-field (- p-field 1)))
        (loop for item in speakers 
              for i = 1 then (+ i 1) do 
              (setf thestring (concatenate 'string thestring 
                                           (format nil "~1%imicdirectfun~D = (p~D = 0 ? 601 : p~D)" i (+ p-field i) (+ p-field i)))))
      (list "mic_pfields_continuous" thestring)
      ))

(defun mic-pfields-trajectory (speakers p-field)
  (let ((thestring)
        (p-field (- p-field 1)))
        (loop for item in speakers 
              for i = 1 then (+ i 1) do 
              (setf thestring (concatenate 'string thestring 
                                           (format nil "~1%imicdirectfun~D = (p~D = 0 ? 601 : p~D)" i (+ p-field i) (+ p-field i)))))
      (list "mic_pfields_trajectory" thestring)
      ))


;asound vimic_directivities_discrete asound, iyaw, idirectfunction, ispknumber, iazimuth, imicparams

(defun vimic-vectors-discrete (speakers)
  (let ((thestring))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%iazimuth~D ielevation~D idistance~D xyz2aed_discrete (ixpos-gisinkposx~D), (iypos-gisinkposy~D), (izpos-gisinkposz~D)" i i i i i i))))
    (list "vimic_vectors_discrete" thestring)
    ))

(defun vimic-directivities-discrete (speakers)
  (let ((macrostr))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf macrostr (concatenate 'string macrostr 
                                       (format nil "~1%asound~D vimic_directivities_discrete asound~D, iazimuth~D, iyaw, idirectfunction, imicdirectfun~D" i i i i))))
    
    (setf macrostr (concatenate 'string macrostr
                                (format nil "~1%outc asound1")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do
          (setf macrostr (concatenate 'string macrostr
                                      (format nil ", asound~D" i))))
    (list "vimic_directivities_discrete" macrostr)
    ))

(defun vimic-vectors-continuous (speakers)
  (let ((thestring))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf thestring (concatenate 'string thestring 
                                       (format nil "~1%kazimuth~D kelevation~D kdistance~D xyz2aed_continuous (kxpos-gisinkposx~D), (kypos-gisinkposy~D), (kzpos-gisinkposz~D)" i i i i i i))))
    (list "vimic_vectors_continuous" thestring)
    ))

(defun vimic-directivities-continuous (speakers)
  (let ((macrostr))
    (loop for item in speakers
          for i = 1 then (+ i 1) do 
          (setf macrostr (concatenate 'string macrostr 
                                       (format nil "~1%asound~D vimic_directivities_continuous asound~D, kazimuth~D, kyaw, idirectfunction, imicdirectfun~D" i i i i))))
    (setf macrostr (concatenate 'string macrostr
                                (format nil "~1%outc asound1")))
    (loop for item in (cdr speakers)
          for i = 2 then (+ i 1) do
          (setf macrostr (concatenate 'string macrostr
                                      (format nil ", asound~D" i))))
    (list "vimic_directivities_continuous" macrostr)
    ))


