;OMprisma 2.0,  2010 M.Schumacher (CIRMMT, McGill University)
; 
;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.
;
;See file LICENSE for further informations on licensing terms.
;
;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.
;
;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,10 USA.
;
;Authors: M. Schumacher

(in-package :om)

;;simple conversions from BPCs and 3DCs to lists of X, Y, Z / A, E, D
;---------------------------------------------------------------------
(defmethod! BPC->AD ((self BPC))
                   (multiple-value-list (xy->ad (x-points self) (y-points self)))
                   )
 
(defmethod! BPC->XY ((self BPC))
                   (multiple-value-list (list (x-points self) (y-points self)))
                 )

(defmethod! 3DC->AED ((self 3DC))
                   (multiple-value-list (xyz->aed (x-points self) (y-points self) (z-points self)))
                   )
 
(defmethod! 3DC->XYZ ((self 3DC))
                   (multiple-value-list (list (x-points self) (y-points self) (z-points self)))
                 )

(defmethod! BPC->XY ((self BPC))
                   (multiple-value-list (list (x-points self) (y-points self)))
                 )
;----------------------------------------------------------------------

;;; these functions should be called traj-utilities instead of 3DC utilities (with methods for BPC 3DC 3Dtrajectory etc.)
;; yes, a defmethod with trqj-loop is better than this defuns
(defmethod! loop-3dc ((self 3DC))
            (let* ((initial-points (point-pairs self))
                  (points (mat-trans (append (butlast initial-points) (list (car initial-points))))))
            (3DC-from-list (first points) (second points) (third points) (type-of self) (decimals self))
            ))

(defmethod! close-3dc ((self 3DC))
            (let* ((initial-points (point-pairs self))
                  (points (mat-trans (append initial-points (list (car initial-points))))))
            (3DC-from-list (first points) (second points) (third points) (type-of self) (decimals self))
            ))

(defun close-bpc (self)
            (let* ((initial-points (point-pairs self))
                  (points (mat-trans (append initial-points (list (car initial-points))))))
            (simple-bpf-from-list (first points) (second points) (type-of self) (decimals self))
            ))

;;; tempo-breakpoint-function

(defmethod! tempo-traj ((self list) &optional section-start)
            :icon 21  
            :initvals '(nil 0.)
            :indoc '("nada")
            :numouts 1
            :doc "nada"
  (let ((str "t"))
    (loop for item in self do 
          (setf str (concatenate 'string str 
                                 (format nil " ~,3f" item))))
    (if section-start
        (setf str (x-append " " ";****************************************" 
                            ";----------- score-statements -----------"
                            (format nil "s ~D" section-start) str))
   (list " " ";****************************************" 
         ";----------- score-statements -----------" str)))
  )


(defmethod! tempo-traj ((self bpf) &optional section-start)
  (let ((str "t"))
    (setf self (flat (mat-trans (list (x-points self) (y-points self)))))
    (loop for item in self do 
          (setf str (concatenate 'string str 
                                 (format nil " ~,3f" item))))
    (if section-start
        (setf str (x-append " " ";****************************************" 
                            ";----------- score-statements -----------"
                            (format nil "s ~D" section-start) str))
   (list " " ";****************************************" 
         ";----------- score-statements -----------" str)))
  )

;;for compatibility
(defmethod! time-traj ((self t) &optional section-start)
            (tempo-traj self section start))


;----calculate time-delay as a function of distance

(defmethod! dist-delay ((e-dels number) (distance number) &optional (units-per-sec 340))
            :icon '(236)   
            :initvals '(nil nil 340) ;
            :indoc '("e-dels" "distance" "units-per-second")
            :numouts 1
            :doc "bla bla"
   (let ((del (/ distance units-per-sec)))
        (+ e-dels del) 
       ))

(defmethod! dist-delay ((e-dels list) (distance list) &optional (units-per-sec 340))
            (mapcar #'(lambda (ed dist) (dist-delay ed dist units-per-sec)) 
                    e-dels distance))

(defmethod! dist-delay ((e-dels list) (distance number) &optional (units-per-sec 340))
   (dist-delay e-dels (make-list (length e-dels) :initial-element distance) units-per-sec))

(defmethod! dist-delay ((e-dels number) (distance list) &optional (units-per-sec 340))
   (dist-delay (make-list (length distance) :initial-element e-dels) distance units-per-sec))



#|
set fog-1 tablesize automatically from number of samples of soundfile
function to get the samplerate: om-sound-n-samples
take the next higher multiple of 2
|#



;;; SCALE-CS-TABLE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

(defmethod! scale-cs-table ((self t) min max) 
            :icon 39
            :initvals '(nil nil nil)
            :indoc '("a bpf or cs-table" "minimum y-value" "maximum y-value")
            :numouts 1
            :doc "rescales a cs-table"
                     (let (myGen7 themin themax)
                       (setf themin min)
                       (setf themax max)
                       (unless (and (numberp min) (numberp max))
                         (if (numberp min)
                             (setf themax (list-max (y-points self))
                                   themin min)
                           (setf themin (list-min (y-points self))
                                 themax max)))
                       (setf myGen7 (simple-bpf-from-list (x-points self) (om-scale (y-points self) themin themax) 'Gen-07 (decimals self)))
                       (setf (id myGen7) (id self))
                       (setf (size myGen7) (size self))
                       (setf (stime myGen7) (stime self))
                       mygen7))


;;; 3D-Spline %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

(defmethod! splines ((self 3Dc) &optional (resolution 10) (degree 3))
            :icon '(234)  
            :initvals '(nil 10 3)
            :indoc '("bpf or bpc" "resolution" "degree")
            :numouts 1
            :doc "splines a bpf"
            (let (
                  (x (splines (simple-bpf-from-list '(0 10) (x-points self) 'bpf (decimals self))  resolution degree))
                  (y (splines (simple-bpf-from-list '(0 10) (y-points self) 'bpf (decimals self))  resolution degree))
                  (z (splines (simple-bpf-from-list '(0 10) (z-points self) 'bpf (decimals self))  resolution degree)))
              (3dc-from-list (y-points x) (y-points y) (y-points z) '3Dc (decimals self))
            ))


;*** for lists and libs

(defmethod! splines ((self t) &optional (resolution 10) (degree 3))
            (if (> (* resolution (length (y-points self))) 1000)
                (get-spline-obj self 1000 degree)
              (get-spline-obj self (* (length (y-points self)) resolution) degree))
            )

(defmethod! splines ((self list) &optional (resolution 10) (degree 3))
  (mapcar #'(lambda (theobject) (splines theobject resolution degree)) 
                    self))

(defmethod! splines ((self bpf-lib) &optional (resolution 10) (degree 3))
  (mapcar #'(lambda (theobject) (splines theobject resolution degree)) 
                    (bpf-list self)))

(defmethod! splines ((self bpc-lib) &optional (resolution 10) (degree 3))
  (mapcar #'(lambda (theobject) (splines theobject resolution degree)) 
                    (bpf-list self)))

(defmethod! splines ((self 3dc-lib) &optional (resolution 10) (degree 3))
  (mapcar #'(lambda (theobject) (splines theobject resolution degree)) 
                    (bpf-list self)))


;;; BPF-samp %%%%%%%%%%%%%%%

(defmethod! bpf-samp ((self t) &key (samples 1000) (coefficient 1) decimals)
            :icon '(233)  
            :initvals '(nil 1000)
            :indoc '("samples a bpf or bpc")
            :numouts 2
            :doc "samples a bpf"
            (let (mybpfpoints mybpf dsamp)
            (unless decimals
              (setf decimals (decimals self)))
            (setf dsamp (multiple-value-list (om-sample self samples nil nil decimals)))
            (setf mybpf (first dsamp))
            (setf mybpfpoints (second dsamp))
              (values mybpf mybpfpoints)
              )
            )

;;; value->bpf %%%%%%%%%%%%%%%%

(defmethod! value->bpf ((myval number) &optional (amount 2) (decimals 7))
            :icon '(233)  
            :initvals '(nil 2 7)
            :indoc '("value" "number of points in line" "decimals")
            :numouts 1
            :doc "converts a value to a bpf (straight line)"
            (simple-bpf-from-list '(0 10) (repeat-n myval amount) 'bpf decimals)
            )

;;; pitch-class to linear %%%%%%%%%%%%
;;; there is a bug...

(defmethod! pc->linear ((chordseq chord-seq) (approx number) &optional scaling)
            :initval '(nil 2 0 100)
            (let (cercle list)
             (setf cercle (chord-seq2c chordseq approx))
             (setf list (om-scale (puntos cercle) (first scaling) (second scaling)))
            ))

;;; add-dur %%%%%%%%%%%%%%%%%%%%%%%%%%%
;;; add-dur-to-amp-envelopes to account for longer duration due to reverb-tails

(defmethod! add-dur ((dur number) (addtime number) &optional ampenv)
            :icon 21  
            :initvals '(nil 1 nil)
            :indoc '("durations" "time to add")
            :numouts 2
            :doc "adds to a given duration and creates or manipulates corresponding ampenvelope"
            (let (myampenv timeaxis)
            (if (bpf-p ampenv) 
                (let ()
                  (setf timeaxis (x-append (om-scale (x-points ampenv) 0. dur) (+ dur 0.1) (+ dur addtime)))
                  (setf myampenv (simple-bpf-from-list timeaxis (x-append (y-points ampenv) (list-min (y-points ampenv)) (list-min (y-points ampenv))) 'bpf 3)))
              (setf myampenv (simple-bpf-from-list (x-append 0 (- dur 0.01) dur (+ dur addtime)) '(1 1 0 0) 'bpf 10))
              )
            (values (+ dur addtime) myampenv)
            ))

;*** for lists

(defmethod! add-dur ((dur list) (addtime list) &optional ampenv)
            (add-dur-sub dur addtime ampenv)
            )

(defmethod! add-dur ((dur list) (addtime number) &optional ampenv)
            (setf addtime (repeat-n addtime (length dur)))
            (add-dur-sub dur addtime ampenv)
            )

(defmethod! add-dur ((dur number) (addtime list) &optional ampenv)
            (setf dur (repeat-n dur (length addtime)))
            (add-dur-sub dur addtime ampenv)
            )

(defun add-dur-sub (dur addtime ampenv) 
  (if ampenv
      (let* ((myampenv (if (listp ampenv) ampenv (repeat-n ampenv (length dur)))))
        (loop for item1 in dur for item2 in addtime for item3 in myampenv   
              collect (multiple-value-list  (add-dur item1 item2 item3 ) ) into tempout
              finally return  (values (first (mat-trans tempout)) (second (mat-trans tempout)))))
    
    (loop for item1 in dur for item2 in addtime              
          collect  (multiple-value-list  (add-dur item1 item2)) into tempout
          finally return  (values (first (mat-trans tempout)) (second (mat-trans tempout))))
    )
  )

;;; transposition with optional frequency shift %%%%%%

(defmethod! trsp-fshft ((self list) (transp number)  &optional (frqshft 0)) 
            :icon 22  
            :initvals '(nil 0 0)
            :indoc '("a list of midicents" "transposition in midicent" "frequency-shift in hz")
            :numouts 1
            :doc "nada"
            (let (mynewlist)
              (setf mynewlist (om+ frqshft (mc->f (om+ self transp))))
              )
            )

;;; frequency-shift with optional transposition %%%%%%

(defmethod! fshft-trsp ((self list) (frqshft number) &optional (transp 0)) 
            :icon 22  
            :initvals '(nil 0 0)
            :indoc '("a list of frequencies" "frequency-shift in hz" "transposition in midicent")
            :numouts 1
            :doc "nada"
            (let (mynewlist)
              (setf mynewlist (mc->f (om+ transp (f->mc (om+ self frqshft)))))
              )
            )
              

;;; lookup (exactly the same as bpf-get-val) %%%%%%%%%

(defmethod! bpf-lookup ((val null) (self bpf))
  :indoc '("a bpf" "an index or list of indeces for interpolated y-lookup")
  :icon '(233)
  :doc  "returns interpolated y-value for given index (x)"
  nil)

(defmethod! bpf-lookup ((val number) (self bpf))
  (table_xy  (x-points self) (y-points self) val  ))

(defmethod! bpf-lookup ((val list) (self bpf) )
  (cons (table_xy  (x-points self) (y-points self) (car val)) (bpf-get-val self (cdr val))))

;ToDo: make a method for lists 
;(om::defmethod! y-transfer ((self bpf) (y0 list) &optional (dec nil))
;  (y-transfer (point-pairs self) y0 dec))

;; this function might also be used in the traj-perturb methods!!
;; useful for combining value pairs
(defmethod! cons-append ((origlist list) (2ndlist list))
        (let (result)
              (setf result (subs-posn 2ndlist
                         (x-append 0 (- (length origlist) 1))
                         (x-append (first origlist) (last origlist))))
          )
        )

(defmethod! minmax ((alist list))
  :initvals '(nil)
  :icon '(209) 
  :indoc '("list")
  :doc "Returns min and max values in list."
  :numouts 2
  (values (list-min alist) (list-max alist))
  )


;;; Clusters %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
;; should be renamed 'traj-clusters'

(defmethod! clusters ((self list) (amount number) (deviation number))
            :icon 22   
            :initvals '(nil 7 5)
            :indoc '("list of values" "how many components" "how much deviation (in percent)")
            :numouts 2
            :doc "creates clusters around values"
            (let (freqlist sorted)
              (setf freqlist (x-append self 
                                       (loop for i from 1 to amount
                                             collect (perturbation self (om/ deviation 100)))))                        
              (setf freqlist (flat freqlist))
              (setf sorted (sort (copy-list freqlist) #'<))
              (values sorted freqlist)
            ))



;*** for BPFs
; needs to be fixed, since X has to be continuously incremented

(defmethod! clusters ((self bpf) (amount number) (deviation number))
            (let (
                  (xclusters (multiple-value-list (clusters (x-points self) amount deviation)))
                  (yclusters (multiple-value-list (clusters (y-points self) amount deviation))))
            (values 
             (simple-bpf-from-list (first xclusters) (first yclusters) 'bpf (decimals self)) 
             (simple-bpf-from-list (second xclusters) (second yclusters) 'bpf (decimals self))  
              ))
              )

;*** for BPCs

(defmethod! clusters ((self bpc) (amount number) (deviation number))
            (let (
                  (xclusters (multiple-value-list (clusters (x-points self) amount deviation)))
                  (yclusters (multiple-value-list (clusters (y-points self) amount deviation))))
            (values 
             (simple-bpf-from-list (first xclusters) (first yclusters) 'bpc (decimals self)) 
             (simple-bpf-from-list (second xclusters) (second yclusters) 'bpc (decimals self))  
              ))
              )


;*** for 3DCs

(defmethod! clusters ((self 3dc) (amount number) (deviation number))
            (let (
                  (xclusters (multiple-value-list (clusters (x-points self) amount deviation)))
                  (yclusters (multiple-value-list (clusters (y-points self) amount deviation)))
                  (zclusters (multiple-value-list (clusters (z-points self) amount deviation))))
            (values 
             (3dc-from-list (first xclusters) (first yclusters) (first zclusters) '3dc (decimals self)) 
             (3dc-from-list (second xclusters) (second yclusters) (second zclusters) '3dc (decimals self))  
              ))
              )

(defmethod! 2D-random ((low number) (high number))
            :icon '(200)   
            :initvals '(nil nil)
            :indoc '("low boundary" "high boundary")
            :numouts 2
            :doc "returns random 2D coordinate within boundaries"
            (values (om-random low high) (om-random low high))
            )

;; for some reason I can't get the mapcar working with multiple outputs
(defmethod! mc->pc ((midicent number))
            :icon '(141)   
            :initvals '(nil)
            :indoc '("midicent")
            :numouts 2
            :doc "returns octave and pitch-class from midicent value"
            :outdoc '("octave" "pitch-class")
           (multiple-value-bind (octave rest) (floor (* midicent 0.01) 12)
           (values octave rest))
           )

(defmethod! mc->pc ((midicent list))
            (mapcar #'(lambda (input)
                        (mc->pc input)) midicent)
            )

(defun covariance (scalars)
  (sqrt (/ (om-sum2 scalars) (length scalars)))
  )
              
(defmethod! centroid ((freqs list) (amps list))
           (/
            (loop for x in freqs 
                  for y in amps
                  finally
                  sum (* x y)
                  )
            (+ (om-sum amps) 0.000001) ;avoid division by zero
           ))

(defun 3d-distance (point1 point2)
 (sqrt (+ 
       (expt (- (first point1) (first point2)) 2) 
       (expt (- (second point1) (second point2)) 2) 
       (expt (- (third point1) (third point2)) 2))))

;(3d-distance  '(-0.707 0.707 0) '(0.707 0.707 0))

(defun 2d-distance (point1 point2)
 (sqrt (+ 
       (expt (- (first point1) (first point2)) 2) 
       (expt (- (second point1) (second point2)) 2)) 
       ))

(defmethod! sound-dur ((s list))
(mapcar 'sound-dur s))

;;; get number of samples of a sound
(defmethod! sound-samples ((self sound))
  :icon '(221)
  :initvals '(nil)
  :indoc '("sound file or pathname")
  :doc "return the number of samples in the sound"
  (om-sound-n-samples self)
  )

(defmethod! sound-samples ((sound-list list))
            (mapcar 'om-sound-n-samples sound-list)
            )

;;; get number of channels in a sound
(defmethod! sound-channels ((self sound))
  :icon '(221)
  :initvals '(nil)
  :indoc '("sound file or pathname")
  :doc "return the number of samples in the sound"
  (om-sound-n-channels self)
  )

(defmethod! sound-channels ((sound-list list))
            (mapcar 'om-sound-n-channels sound-list)
            )

;;; OM-scale with exponent

(defmethod! om-scale-exp ((self list) (minout number) (maxout number) (exponent number) &optional (minin 0) (maxin 0))
  :initvals '(1 0 1 1) 
  :indoc '("number or list"  "a number" "a number" "an exponent")
  :icon '(209)
  :doc 
  "Scales <self> (a number or list of numbers) considered to be in the interval [<minin> <maxin>] towards the interval [<minout> <maxout>].

If [<minin> <maxin>] not specified or equal to [0 0], it is bound to the min and the max of the list.

Ex. (om-scale 5 0 100 0 10)  => 50
Ex. (om-scale '(0 2 5) 0 100 0 10)  => (0 20 50)
Ex. (om-scale '(0 2 5) 0 100)  => (0 40 100)
 "
  (om-scale (om^ (om-scale self 0. 1. minin maxin) exponent) minout maxout 0. 1.)
  )

; URN function
(defmethod! urn ((min number) (max number))
            :icon '(176)
            :initvals '(nil nil)
            (permut-random (arithm-ser min max 1))
            )

;;; String-conversions

;;; Convert String into List

(defmethod! string->list ((self string))
            :icon '(141)  
            :initvals '(nil)
            :indoc '("string")
            :doc "converts a string into a list"
            (data-from-line self)
            )            

;;; Convert List into String

(defmethod! list->string ((self list))
  :icon '(141)  
  :initvals '(nil)
  :indoc '("list")
  :doc "converts a list into a string"
  (let (str)
    (loop for item in self do
          (setf str (concatenate 'string str 
                       (format nil " ~a" item))))
    str))

;;; Transposition Factor

(defmethod! transratio ((midicent number))
            :icon '(141)  
            :initvals '(0)
            :indoc '("converts transposition in midicents into transposition-factor")
            :numouts 1
            :doc "bla bla"
 (exp (* midicent 0.00057762265)
       ))    

(defmethod! transratio ((midicent list))
   (mapcar 'transratio midicent))

(defmethod! om-sum ((self list))
                  (loop for item in self
                  sum item)
                  )
; squared sum
(defmethod! om-sum2 ((self list))
                  (loop for item in self
                  sum (* item item))
                  )

;*** sine/cosine functions for lists

(defmethod! om-sin ((arg1 number))  
  :initvals '(nil)
  :icon '(209) 
  :indoc '("list")
  :doc "Sine for every item in list."
  (sin arg1))

(defmethod! om-sin ((arg1 list))   
  (mapcar #'(lambda (input)
              (sin input)) arg1))

;*** cosine function for lists

(defmethod! om-cos ((arg1 number))  
  :initvals '(nil) 
  :icon '(209) 
  :indoc '("list")
  :doc "Cos for every item in list."
  (cos arg1))

(defmethod! om-cos ((arg1 list))   
  (mapcar #'(lambda (input)
              (cos input)) arg1))

(defun clip (val &optional (min 0.0) (max 1.0))
" If val is below min, return min,
  if val is above max, return max,
  otherwise return val.
" 
  (let ((from min) (to max))
    (when (> min max) (setf from max) (setf to min))
    (cond
     ((> val to) to)
     ((< val from) from)
     (t val))))

(defmethod! om-clipold ((arg1 list) &optional (min 0.0) (max 1.0))
            (let ((repetitions (length arg1)))
                  (mapcar #'(lambda (input min max)
                               (clip input min max)) arg1 (repeat-n min repetitions) (repeat-n max repetitions)))
              )

;; %%% Trajectory-stuff %%%%%%%%%%%%%%%%%%

(defun append-point (point-pairs)
  (let ((carpointpairs (car point-pairs)))
  (append point-pairs (list (list (+ (first carpointpairs)) (second carpointpairs))))
  ))

(defmethod! closep ((self t))
            (let ((thepairs (point-pairs self)))
              (setf result (equal (car thepairs) (flat (last thepairs))))
              result))

(defmethod! pointpairs->xy ((self list))
            :icon '(141)  
            :numouts 2
  (let* ((translist (mat-trans self)))
    (values (first translist) (second translist))
    ))

(defmethod! traj-lib ((shape t) (resolution number) &rest args)
            ;shape is a menu with possible shapes/trajectories. args provides the necessary arguments for generation
            )

;-----------SPREAD->ORDER-----------
(defmethod! spread->order ((spread number))
            :icon '(141)  
            :initvals '(0)
            :indoc '("spread")
            :numouts 1
            :doc "converts spread into order"
 (om-scale spread 0 3 100 0))    

(defmethod! spread->order ((spread list))
 (om-scale spread 0 3 100 0)) 

(defmethod! spread->order ((spread bpf))
 (simple-bpf-from-list (x-points spread) (om-scale (y-points spread) 0 3 100 0) 'bpf 5)
 )
;-----------ORDER->SPREAD----------

(defmethod! order->spread ((order number))
            :icon '(141)  
            :initvals '(0)
            :indoc '("order")
            :numouts 1
            :doc "converts spread into order"
 (om-scale order 0 100 3 0))    

(defmethod! order->spread ((order list))
 (om-scale order 0 100 3 0)) 

(defmethod! order->spread ((order bpf))
 (simple-bpf-from-list (x-points order) (om-scale (y-points order) 0 100 3 0) 'bpf 5)
 )

; -------- overriding decimals-limit -----------

(defmethod check-decimals ((self bpf))
    (unless (and (integerp (decimals self))
                 (> (decimals self) 0) 
                 (<= (decimals self) 10))
    (cond ((not (integerp (decimals self)))
           (om-beep-msg "BPF decimals must be an integer value!")
           (setf (slot-value self 'decimals) 0))
          ((minusp (decimals self))
           (om-beep-msg "BPF decimals must be a positive integer!")
           (setf (slot-value self 'decimals) 0))
#|
           ((> (decimals self) 10)
            (om-beep-msg "BPFs only support up to 10 decimals!")
            (setf (slot-value self 'decimals) 10))
|#
           )))
